package datos;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import modelo.Sesion;
import modelo.ficheros.TipoFichero;
import modelo.vcompilacion.ModeloCompilacion;
import modelo.vedicion.ModeloEdicion;
import modelo.vejecucion.ModeloEjecucion;
import modelo.vestadisticas.ModeloEstadistica;
import modelo.vgprof.ModeloGprof;
import modelo.vtraduccion.ModeloTR;
import negocio.PreferenciasSingleton;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class SesionXMLParser {
	
	public  void guardarEnFichero ( File fichero, Sesion s )
	throws ParserConfigurationException, TransformerException {
		Document docXML = null;

		if ( fichero != null && s != null ) {
			DocumentBuilder docB = DocumentBuilderFactory.newInstance( )
			.newDocumentBuilder( );
			docXML = docB.newDocument( );


			/**
			 * ahora se va a escribir con los Modelos que tenemos
			 * el docXML
			 */
			if ( docXML != null ) {

				Element sesion = docXML.createElement( "sesion" );
				docXML.appendChild( sesion );

				escribirModelosEdicion ( docXML, sesion, s.getModelosEdicion( ) );

				escribirModelosCompilacion ( docXML, sesion, s.getModelosCompilacion( ) );

				escribirModelosEjecucion ( docXML, sesion, s.getModelosEjecucion( ) );

				escribirModelosGprof ( docXML, sesion, s.getModelosGprof( ) );

				escribirModelosTraduccion ( docXML, sesion, s.getModelosTraduccion( ) );

				escribirModelosEstadisticas ( docXML, sesion, s.getModelosEstadisticas( ) );


				/** Escritura en el fichero */
				TransformerFactory transformerFactory = TransformerFactory
				.newInstance();
				Transformer transformer = transformerFactory.newTransformer();
				DOMSource source = new DOMSource( docXML );
				StreamResult result = new StreamResult( fichero );
				transformer.transform(source, result);

			} }
	}
	
	public  Sesion leerDeFichero ( File fichero )
	throws ParserConfigurationException, SAXException, IOException, XPathExpressionException {
		
		Sesion s = new Sesion();

		/** Primero hay que pasar el fichero (File) a 
		 * Document para su posterior
		 * interpretación */

		if ( fichero != null ) {

			Document docXML = null;


			DocumentBuilder docB = DocumentBuilderFactory.newInstance( )
			.newDocumentBuilder( );
			docXML = docB.parse( fichero );


			/**
			 * ahora se va a leer para rellenar todo lo demás, una vez que tenemos
			 * el docXML
			 */
			if ( docXML != null ) {

				s.addVMEdicion( leerModelosEdicion ( docXML ) );

				s.addVMCompilacion( leerModelosCompilacion ( docXML ) );

				s.addVMEjecucion( leerModelosEjecucion ( docXML ) );

				s.addVMGprof( leerModelosGprof ( docXML ) );

				s.addVMTR( leerModelosTraduccion ( docXML ) );

				s.addVMEstadistica( leerModelosEstadisticas ( docXML ) );

			} 
		}
		
		return s;
	}
	

	private  Vector < ModeloCompilacion > leerModelosCompilacion (Document docXML ) 
	throws XPathExpressionException {

		Vector < ModeloCompilacion > vMoCompilacion = new Vector < ModeloCompilacion > ();
		
		/** primero comprobar que existen modelos de compilación en el fichero. */
		String expresion = "/sesion/compilacion";
		
		XPath xpath = XPathFactory.newInstance( ).newXPath( );

		/**
		 * si existe, rellenar las estructuras correspondientes
		 * (ModeloCompilacion)
		 */
		if ( xpath.evaluate( expresion , docXML , XPathConstants.NODE ) != null ) {

			/**
			 * a continuación, vamos a iniciar el proceso: 
			 * leer todos los ModeloCompilación
			 */

			String expression = "/sesion/compilacion/moCompilacion";
			NodeList nodosModelo = (NodeList) xpath.evaluate( expression ,
					docXML , XPathConstants.NODESET );

			/** Creamos todos los ModelosCompilación. */
			for ( int i = 0 ; i < nodosModelo.getLength( ) ; i++ ) {
				Node nodoMC = nodosModelo.item( i );
				Node n; // variable auxiliar

				ModeloCompilacion moC = new ModeloCompilacion( );

				/** Primero el nombre del futuro programa (nombre pestaña). */
				n = (Node) xpath.evaluate( "nombrePrograma" ,
						nodoMC , XPathConstants.NODE );
				moC.setNombrePrograma( n.getTextContent( ) );

				
				/** Segundo el directorio de trabajo. */
				n = (Node) xpath.evaluate( "dirTrabajo" ,
						nodoMC , XPathConstants.NODE );
				if (n != null) {
					File directorioTrabajo = new File ( n.getTextContent( ) );
					if ( directorioTrabajo.isDirectory( ) )
						moC.setDirectorioTrabajo( directorioTrabajo );
					else
						moC.setDirectorioTrabajo( null );
				} else
					moC.setDirectorioTrabajo( null );
				
				
				/** Tercero: comando auto. */
				n = (Node) xpath.evaluate( "auto" ,
						nodoMC , XPathConstants.NODE );
				if ( n != null)
					moC.setComandoAuto( true );
				else
					moC.setComandoAuto( false );
				
				
				/** Cuarto: cadena comando def. usuario. */
				n = (Node) xpath.evaluate( "comandoDU" ,
						nodoMC , XPathConstants.NODE );
				if ( n != null)
					moC.setCadenaCompilacionDefinidaUsuario( n.getTextContent( ) );
				
				
				/** Quinto: tipo de compilación. */
				n = (Node) xpath.evaluate( "tipo" ,
						nodoMC , XPathConstants.NODE );
				if ( n != null)
					moC.setTipo( TipoFichero.getTipoFichero( n.getTextContent( ) ) );
				else
					moC.setTipo( TipoFichero.DESCONOCIDO );
				
		
				/** Sexto: Comando específico. */
				Vector <String> vc = new Vector <String> ();
				
				n = (Node) xpath.evaluate( "compilador" ,
						nodoMC , XPathConstants.NODE );
				if ( n != null)
					vc.addElement( n.getTextContent( ) );
				else 
					vc.addElement( moC.getCompilador( ).firstElement( ) );

				NodeList nodosOpcionesCompilador = (NodeList) xpath.evaluate( "opcionesCompilador/opcion" ,
						nodoMC , XPathConstants.NODESET );

				if (nodosOpcionesCompilador != null ) {
					for ( int j = 0 ; j < nodosOpcionesCompilador.getLength( ) ; j++ ) {
						Node nodoOpt = nodosOpcionesCompilador.item( j );

						if (n != null && !nodoOpt.getTextContent( ).isEmpty() ) {
							String s = nodoOpt.getTextContent( );
							vc.addElement( s );
						}
					}
				}
				if ( vc.size( ) > 0 )
					moC.setCompilador( vc );
				
				
				/** Séptimo: Después todos los ficheros listados. */
				NodeList nodosFuentes = (NodeList) xpath.evaluate( "listaFuentes" ,
						nodoMC , XPathConstants.NODESET );

				if ( nodosFuentes != null ) {
					Node nodoFFuente;
					for ( int j = 0 ; j < nodosFuentes.getLength( ) ; j++ ) {
						nodoFFuente = nodosFuentes.item( j );

						n = (Node) xpath.evaluate( "pathFichero" , nodoFFuente ,
								XPathConstants.NODE );
						if (n != null) {
							File ficheroFuente = new File ( n.getTextContent( ) );
							if ( ficheroFuente != null && ficheroFuente.isFile( ) )
								moC.addFichero( ficheroFuente );
						}
					}
				}
				
				/** Para terminar, añadir este ModeloCompilación al vector
				 * de Modelos de Compilación  */
				vMoCompilacion.add( moC );
			}
		}
		return vMoCompilacion;			
		
	}

	private  Vector < ModeloEdicion > leerModelosEdicion (Document docXML ) throws XPathExpressionException {
		
		Vector < ModeloEdicion > vMoEdicion = new Vector < ModeloEdicion > ();
		
		/** primero comprobar que existen modelos de edición en el fichero. */
		String expresion = "/sesion/edicion";
		
		XPath xpath = XPathFactory.newInstance( ).newXPath( );

		/**
		 * si existe, rellenar las estructuras correspondientes
		 * (ModeloEdicion)
		 */
		if ( xpath.evaluate( expresion , docXML , XPathConstants.NODE ) != null ) {

			/**
			 * a continuación, vamos a iniciar el proceso: 
			 * leer todos los ModeloEdicion
			 */

			String expression = "/sesion/edicion/moEdicion";
			NodeList nodosModelo = (NodeList) xpath.evaluate( expression ,
					docXML , XPathConstants.NODESET );

			/** Creamos todos los ModelosEdidion. */
			for ( int i = 0 ; i < nodosModelo.getLength( ) ; i++ ) {
				Node nodoME = nodosModelo.item( i );
				Node n; // variable auxiliar

				ModeloEdicion moE = new ModeloEdicion( );

				/** Primero el nombre del futuro programa (nombre pestaña). */				
				n = (Node) xpath.evaluate( "pathFichero" , nodoME ,
						XPathConstants.NODE );
				if (n != null) {
					File ficheroFuente = new File ( n.getTextContent( ) );
					if ( ficheroFuente != null && ficheroFuente.isFile( ) )
						moE.setFichero( ficheroFuente );
				}
				
				/** Para terminar, añadir este ModeloEdicion al vector
				 * de Modelos de Edicion  */
				vMoEdicion.add( moE );
			}
		}
		return vMoEdicion;					
	}

	private  Vector < ModeloEjecucion > leerModelosEjecucion (Document docXML ) 
	throws XPathExpressionException {

		Vector < ModeloEjecucion > vMoEjecucion = new Vector < ModeloEjecucion > ();
		
		/** primero comprobar que existen modelos de ejecucion en el fichero. */
		String expresion = "/sesion/ejecucion";
		
		XPath xpath = XPathFactory.newInstance( ).newXPath( );

		/**
		 * si existe, rellenar las estructuras correspondientes
		 * (ModeloEjecucion)
		 */
		if ( xpath.evaluate( expresion , docXML , XPathConstants.NODE ) != null ) {

			/**
			 * a continuación, vamos a iniciar el proceso: 
			 * leer todos los ModeloEjecucion
			 */

			String expression = "/sesion/ejecucion/moEjecucion";
			NodeList nodosModelo = (NodeList) xpath.evaluate( expression ,
					docXML , XPathConstants.NODESET );

			/** Creamos todos los ModelosEjecución. */
			for ( int i = 0 ; i < nodosModelo.getLength( ) ; i++ ) {
				Node nodoMJ = nodosModelo.item( i );
				Node n; // variable auxiliar

				ModeloEjecucion moJ = new ModeloEjecucion( );

				/** Primero el nombre del futuro fichero de datos (nombre pestaña). */
				n = (Node) xpath.evaluate( "nombreFDatos" ,
						nodoMJ , XPathConstants.NODE );
				moJ.setNombreResultadosEjecucion( n.getTextContent( ) );

				
				/** Segundo el directorio de trabajo. */
				n = (Node) xpath.evaluate( "dirTrabajo" ,
						nodoMJ , XPathConstants.NODE );
				if (n != null) {
					File directorioTrabajo = new File ( n.getTextContent( ) );
					if ( directorioTrabajo.isDirectory( ) )
						moJ.setDirectorioTrabajo( directorioTrabajo );
					else
						moJ.setDirectorioTrabajo( null );
				} else
					moJ.setDirectorioTrabajo( null );
				
				
				/** Tercero: el path del programa. */
				n = (Node) xpath.evaluate( "pathPrograma" , nodoMJ ,
						XPathConstants.NODE );
				if (n != null) {
					File programa = new File ( n.getTextContent( ) );
					if ( programa != null && programa.isFile( ) )
						moJ.setPrograma( programa );
				}
				
				/** Cuarto: fichero resultados. */
				n = (Node) xpath.evaluate( "pathFicheroResultados" , nodoMJ ,
						XPathConstants.NODE );
				if (n != null) {
					File fresultados = new File ( n.getTextContent( ) );
					if ( fresultados != null && fresultados.isFile( ) )
						moJ.setFicheroResultados( fresultados );
				}
								
				
				/** Quinto: entradas */
				expression = "listaEntradas";
				NodeList nodosEntradas = (NodeList) xpath.evaluate( expression ,
						nodoMJ , XPathConstants.NODESET );

				if ( nodosEntradas != null ) {
					Node nodoEntrada;
					for ( int j = 0 ; j < nodosEntradas.getLength( ) ; j++ ) {
						nodoEntrada = nodosEntradas.item( j );

						n = (Node) xpath.evaluate( "entrada" , nodoEntrada ,
								XPathConstants.NODE );
						moJ.addEntrada( n.getTextContent( ) );
					}
				}
							
				/** Sexto: argmentos. */
				expression = "listaArgumentos";
				NodeList nodosArgumentos = (NodeList) xpath.evaluate( expression ,
						nodoMJ , XPathConstants.NODESET );

				if ( nodosArgumentos != null ) {
					Node nodoArgumento;
					for ( int j = 0 ; j < nodosArgumentos.getLength( ) ; j++ ) {
						nodoArgumento = nodosArgumentos.item( j );

						n = (Node) xpath.evaluate( "argumento" , nodoArgumento ,
								XPathConstants.NODE );
						moJ.addArgumento( n.getTextContent( ) );
					}
				}

				/** Para terminar, añadir este ModeloEjecucion al vector
				 * de Modelos de Ejecución  */
				vMoEjecucion.add( moJ );
			}
		}
		return vMoEjecucion;			
		
	}

	private  Vector < ModeloGprof > leerModelosGprof (Document docXML ) throws XPathExpressionException {

		Vector < ModeloGprof > vMoGprof = new Vector < ModeloGprof > ();
		
		/** primero comprobar que existen modelos de gprof en el fichero. */
		String expresion = "/sesion/gprof";
		
		XPath xpath = XPathFactory.newInstance( ).newXPath( );

		/**
		 * si existe, rellenar las estructuras correspondientes
		 * (ModeloGprof)
		 */
		if ( xpath.evaluate( expresion , docXML , XPathConstants.NODE ) != null ) {

			/**
			 * a continuación, vamos a iniciar el proceso: 
			 * leer todos los ModeloGprof
			 */

			String expression = "/sesion/gprof/moGprof";
			NodeList nodosModelo = (NodeList) xpath.evaluate( expression ,
					docXML , XPathConstants.NODESET );

			/** Creamos todos los ModelosGprof. */
			for ( int i = 0 ; i < nodosModelo.getLength( ) ; i++ ) {
				Node nodoMG = nodosModelo.item( i );
				Node n; // variable auxiliar

				ModeloGprof moG = new ModeloGprof( );

				/** Primero el nombre del futuro fichero con la salida de gprof (nombre pestaña). */
				n = (Node) xpath.evaluate( "nombreSalidaGprof" ,
						nodoMG , XPathConstants.NODE );
				moG.setNombreSalidaGprof( n.getTextContent( ) );
				
				/** Segundo el directorio de trabajo. */
				n = (Node) xpath.evaluate( "dirTrabajo" ,
						nodoMG , XPathConstants.NODE );
				if (n != null) {
					File directorioTrabajo = new File ( n.getTextContent( ) );
					if ( directorioTrabajo.isDirectory( ) )
						moG.setDirectorioTrabajo( directorioTrabajo );
					else
						moG.setDirectorioTrabajo( null );
				} else
					moG.setDirectorioTrabajo( null );
				
				
				/** Tercero: el path del programa. */
				n = (Node) xpath.evaluate( "pathPrograma" , nodoMG ,
						XPathConstants.NODE );
				if (n != null) {
					File programa = new File ( n.getTextContent( ) );
					if ( programa != null && programa.exists( ) && programa.isFile( ) )
						moG.setPrograma( programa );
				}
		
				/** Cuarto: archivo datos. */
				n = (Node) xpath.evaluate( "pathArchivoDatos" , nodoMG ,
						XPathConstants.NODE );
				if (n != null) {
					File gmon = new File ( n.getTextContent( ) );
					if ( gmon != null && gmon.exists( ) && gmon.isFile( ) )
						moG.setArchivoDatos( gmon );
				}
				
				/** Quinto: fichero salida. */
				n = (Node) xpath.evaluate( "pathFicheroSalida" , nodoMG ,
						XPathConstants.NODE );
				if (n != null) {
					File ficheroSalida = new File ( n.getTextContent( ) );
					if ( ficheroSalida != null && ficheroSalida.exists( ) && ficheroSalida.isFile( ) )
						moG.setFicheroSalida( ficheroSalida );
				}

				/** Sexto: salida. */
				n = (Node) xpath.evaluate( "salidaGprof" , nodoMG ,
						XPathConstants.NODE );
				if (n != null) {
					String salida = n.getTextContent( );
					if ( salida != null )
						moG.setSalida( salida );
				}
		

				/** Para terminar, añadir este ModeloGprof al vector
				 * de Modelos de Gprof  */
				vMoGprof.add( moG );
			}
		}
		
		return vMoGprof;	
	}

	private  Vector < ModeloTR > leerModelosTraduccion (Document docXML ) throws XPathExpressionException {

		Vector < ModeloTR > vMoTraduccion = new Vector < ModeloTR > ();
		
		/** primero comprobar que existen modelos de traduccion en el fichero. */
		String expresion = "/sesion/traduccion";
		
		XPath xpath = XPathFactory.newInstance( ).newXPath( );

		/**
		 * si existe, rellenar las estructuras correspondientes
		 * (ModeloTR)
		 */
		if ( xpath.evaluate( expresion , docXML , XPathConstants.NODE ) != null ) {

			/**
			 * a continuación, vamos a iniciar el proceso: 
			 * leer todos los ModeloTR
			 */

			String expression = "/sesion/traduccion/moTraduccion";
			NodeList nodosModelo = (NodeList) xpath.evaluate( expression ,
					docXML , XPathConstants.NODESET );

			/** Creamos todos los ModelosTR. */
			for ( int i = 0 ; i < nodosModelo.getLength( ) ; i++ ) {
				Node nodoMT = nodosModelo.item( i );
				Node n; // variable auxiliar

				ModeloTR moT = new ModeloTR( );

				/** Primero el nombre del futuro fichero XML (nombre pestaña). */
				n = (Node) xpath.evaluate( "nombreXML" ,
						nodoMT , XPathConstants.NODE );
				moT.setNombreXML( n.getTextContent( ) );
				
				/** Segundo el directorio de trabajo. */
				n = (Node) xpath.evaluate( "dirTrabajo" ,
						nodoMT , XPathConstants.NODE );
				if (n != null) {
					File directorioTrabajo = new File ( n.getTextContent( ) );
					if ( directorioTrabajo.isDirectory( ) )
						moT.setDirectorioTrabajo( directorioTrabajo );
					else
						moT.setDirectorioTrabajo( null );
				} else
					moT.setDirectorioTrabajo( null );
				
				
				/** Tercero: el path del fichero salida. */
				n = (Node) xpath.evaluate( "pathSalidaGprof" , nodoMT ,
						XPathConstants.NODE );
				if (n != null) {
					File salida = new File ( n.getTextContent( ) );
					if ( salida != null && salida.exists( ) && salida.isFile( ) )
						moT.setFicheroSalida( salida );
				}
		
				/** Cuarto: el path del fichero XML. */
				n = (Node) xpath.evaluate( "pathGprofXML" , nodoMT ,
						XPathConstants.NODE );
				if (n != null) {
					File fXML = new File ( n.getTextContent( ) );
					if ( fXML != null && fXML.exists( ) && fXML.isFile( ) )
						moT.setFicheroXML( fXML );
				}
				
				/** Quinto: salida. */
				n = (Node) xpath.evaluate( "salidaXML" , nodoMT ,
						XPathConstants.NODE );
				if (n != null) {
					String salida = n.getTextContent( );
					if ( salida != null )
						moT.setSalidaXML( salida );
				}
		
				/** Sexto: Comando específico. */
				Vector <String> vt = new Vector <String> ();
				
				n = (Node) xpath.evaluate( "traductor" ,
						nodoMT , XPathConstants.NODE );
				if ( n != null)
					vt.addElement( n.getTextContent( ) );
				else 
					vt.addElement( moT.getComandoTR( ).firstElement( ) );

				NodeList nodosOpcionesTraductor = (NodeList) xpath.evaluate( "opcionesTraductor/opcion" ,
						nodoMT , XPathConstants.NODESET );

				if (nodosOpcionesTraductor != null ) {
					for ( int j = 0 ; i < nodosOpcionesTraductor.getLength( ) ; j++ ) {
						Node nodoOpt = nodosOpcionesTraductor.item( j );

						if (n != null && !nodoOpt.getTextContent( ).isEmpty() ) {
							String s = nodoOpt.getTextContent( );
							vt.addElement( s );
						}
					}
				}
				if ( vt.size( ) > 0 )
					moT.setComandoTR( vt );

				/** Para terminar, añadir este ModeloTR al vector
				 * de Modelos de Traduccion */
				vMoTraduccion.add( moT );
			}
		}
		
		return vMoTraduccion;	
	}

	private  Vector < ModeloEstadistica > leerModelosEstadisticas (Document docXML ) throws XPathExpressionException {

		Vector < ModeloEstadistica > vMoEstadisticas = new Vector < ModeloEstadistica >();
		
		/** primero comprobar que existen modelos de estadistica en el fichero. */
		String expresion = "/sesion/estadisticas";
		
		XPath xpath = XPathFactory.newInstance( ).newXPath( );

		/**
		 * si existe, rellenar las estructuras correspondientes
		 * (ModeloEstadistica)
		 */
		if ( xpath.evaluate( expresion , docXML , XPathConstants.NODE ) != null ) {

			/**
			 * a continuación, vamos a iniciar el proceso: 
			 * leer todos los ModeloEstadistica
			 */

			String expression = "/sesion/estadisticas/moEstadistica";
			NodeList nodosModelo = (NodeList) xpath.evaluate( expression ,
					docXML , XPathConstants.NODESET );

			/** Creamos todos los ModelosEstadistica. */
			for ( int i = 0 ; i < nodosModelo.getLength( ) ; i++ ) {
				Node nodoMS = nodosModelo.item( i );
				Node n; // variable auxiliar

				ModeloEstadistica moS = new ModeloEstadistica( );

				/** Primero el path del fichero XML. */
				n = (Node) xpath.evaluate( "pathGprofXML" , nodoMS ,
						XPathConstants.NODE );
				if (n != null) {
					File fXML = new File ( n.getTextContent( ) );
					if ( fXML != null && fXML.exists( ) && fXML.isFile( ) )
						moS.setFicheroXML( fXML );
				}
				
				/** Para terminar, añadir este ModeloEstadistica al vector
				 * de Modelos de Estadísticas */
				vMoEstadisticas.add( moS );
			}
		}
		return vMoEstadisticas;	

	}

	private  void escribirModelosCompilacion (Document docXML, Node sesion, Vector<ModeloCompilacion> vMoCompilacion ) {

		/** sesion es el primer hijo del Root. */ 
		
		/** primero comprobaremos que hay algún modelo Compilación 
		 * para ver si añadimos esta rama.	 */
		if ( vMoCompilacion.size( ) > 0 ) {
			/** crear el hijo de la compilación. */
			Node compilacion = sesion.appendChild( docXML.createElement( "compilacion" ) );
			
			/** ahora crear un moCompilacion por cada modelo de Compilación del vector */
			Iterator <ModeloCompilacion > i = vMoCompilacion.iterator( );
			
			while ( i.hasNext( ) ) {
				ModeloCompilacion moC = i.next( );
				
				Node nMoCompilacion = compilacion.appendChild( docXML.createElement( "moCompilacion" ) );
				
				Element e; 
				
				/** Primero el nombre del futuro programa (nombre pestaña). */
				e = docXML.createElement( "nombrePrograma" );
				e.setTextContent( moC.getNombrePrograma( ) );
				nMoCompilacion.appendChild( e );
				
				/** Segundo el directorio de trabajo. */
				e = docXML.createElement( "dirTrabajo" );
				e.setTextContent( moC.getDirectorioTrabajo( ).getAbsolutePath( ) );
				nMoCompilacion.appendChild( e );				
				
				/** Tercero: comando auto. */
				if ( moC.isComandoAuto( ) )
					nMoCompilacion.appendChild( docXML.createElement( "auto" ) );				
				
				/** Cuarto: cadena comando def. usuario. */
				e = docXML.createElement( "comandoDU" );
				e.setTextContent( moC.getCadenaCompilacionDefinidaUsuario( ) );
				nMoCompilacion.appendChild( e );				
				
				/** Quinto: tipo de compilación. */
				e = docXML.createElement( "tipo" );
				e.setTextContent( moC.getTipo( ).toString( ) );
				nMoCompilacion.appendChild( e );
				
				/** Sexto: el comando de compilación individual (compilador y sus opciones) */
				Vector < String > vs = moC.getCompilador( );
				if ( /*!vs.equals( PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getComandoCompilar( moC.getTipo( ) ) ) &&*/ vs.size( ) > 0 ) {
					Iterator < String > j = vs.iterator( );
				
					e = docXML.createElement( "compilador" );
					e.setTextContent( j.next( ) );
					nMoCompilacion.appendChild( e );
					
					Node opciones = docXML.createElement( "opcionesCompilador" );
					while (j.hasNext( )) {
						String cc = j.next( );
						
						e = docXML.createElement( "opcion" );
						e.setTextContent( cc );
						opciones.appendChild( e );
					}
					nMoCompilacion.appendChild( opciones );
				}
				
				/** Séptimo: ahora crear una lista con los ficheros listados */
				ArrayList < File > al = moC.getListaFicheros( ); 
				Iterator < File > j = al.iterator( );

				if ( al.size( ) > 0 ) {
					Node nListaFuentes = nMoCompilacion.appendChild( docXML.createElement( "listaFuentes" ) );

					while ( j.hasNext( ) ) {
						File ff = j.next( );

						e = docXML.createElement( "pathFichero" );
						e.setTextContent( ff.getAbsolutePath( ) );
						nListaFuentes.appendChild( e );
					}
				}
			}
		}			
		
	}

	private  void escribirModelosEdicion (Document docXML, Node sesion, Vector<ModeloEdicion> vMoEdicion ) {

		/** sesion es el primer hijo del Root. */ 
		
		/** primero comprobaremos que hay algún modelo Edicion 
		 * para ver si añadimos esta rama.	 */
		if ( vMoEdicion.size( ) > 0 ) {
			/** crear el hijo de la edición. */
			Node edicion = sesion.appendChild( docXML.createElement( "edicion" ) );
			
			/** ahora crear un moCompilacion por cada modelo de Compilación del vector */
			Iterator <ModeloEdicion > i = vMoEdicion.iterator( );
			
			while ( i.hasNext( ) ) {
				ModeloEdicion moE = i.next( );
				
				Node nMoEdicion = edicion.appendChild( docXML.createElement( "moEdicion" ) );
				
				Element e; 
				
				/** Primero el path del fichero. */
				File ff = moE.getFichero( );
				e = docXML.createElement( "pathFichero" );
				e.setTextContent( ff.getAbsolutePath( ) );
				nMoEdicion.appendChild( e );
			}
		}			
		
	}

	private  void escribirModelosEjecucion (Document docXML, Node sesion, Vector<ModeloEjecucion> vMoEjecucion ) {

		/** sesion es el primer hijo del Root. */ 
		
		/** primero comprobaremos que hay algún modelo Ejecucion 
		 * para ver si añadimos esta rama.	 */
		if ( vMoEjecucion.size( ) > 0 ) {
			/** crear el hijo de la compilación. */
			Node ejecucion = sesion.appendChild( docXML.createElement( "ejecucion" ) );
			
			/** ahora crear un moEjecucion por cada modelo de Ejecución del vector */
			Iterator <ModeloEjecucion > i = vMoEjecucion.iterator( );
			
			while ( i.hasNext( ) ) {
				ModeloEjecucion moJ = i.next( );
				
				Node nMoEjecucion = ejecucion.appendChild( docXML.createElement( "moEjecucion" ) );
				
				Element e; 
				
				/** Primero el nombre del futuro programa (nombre pestaña). */
				e = docXML.createElement( "nombreFDatos" );
				e.setTextContent( moJ.getNombreResultadosEjecucion( ) );
				nMoEjecucion.appendChild( e );
				
				/** Segundo el directorio de trabajo. */
				e = docXML.createElement( "dirTrabajo" );
				e.setTextContent( moJ.getDirectorioTrabajo( ).getAbsolutePath( ) );
				nMoEjecucion.appendChild( e );				
				
				/** Tercero: programa. */
				e = docXML.createElement( "pathPrograma" );
				e.setTextContent( moJ.getPrograma( ).getAbsolutePath( ) );
				nMoEjecucion.appendChild( e );
				
				/** Cuarto: fichero resultados */
				File fresultados = moJ.getFicheroResultados( );
				if ( fresultados != null ) {
					e = docXML.createElement( "pathFicheroResultados" );
					e.setTextContent( fresultados.getAbsolutePath( ) );
					nMoEjecucion.appendChild( e );
				}

				/** Quinto: entradas. */
				ArrayList < String > al = moJ.getListaEntradas( ); 
				Iterator < String > j = al.iterator( );
				
				while ( j.hasNext( ) ) {
					String entrada = j.next( );
					
					Node nListaFuentes = nMoEjecucion.appendChild( docXML.createElement( "listaEntradas" ) );

					e = docXML.createElement( "entrada" );
					e.setTextContent( entrada );
					nListaFuentes.appendChild( e );
				}
				
				/** Sexto: argumentos. */
				al = moJ.getListaArgumentos(); 
				j = al.iterator( );
				
				while ( j.hasNext( ) ) {
					String argumento = j.next( );
					
					Node nListaFuentes = nMoEjecucion.appendChild( docXML.createElement( "listaArgumentos" ) );

					e = docXML.createElement( "argumento" );
					e.setTextContent( argumento );
					nListaFuentes.appendChild( e );
				}				

			}
		}					
	}

	private  void escribirModelosGprof (Document docXML, Node sesion, Vector<ModeloGprof> vMoGprof ) {

		/** sesion es el primer hijo del Root. */ 
		
		/** primero comprobaremos que hay algún modelo Gprof 
		 * para ver si añadimos esta rama.	 */
		if ( vMoGprof.size( ) > 0 ) {
			/** crear el hijo de la invocación a gprof. */
			Node gprof = sesion.appendChild( docXML.createElement( "gprof" ) );
			
			/** ahora crear un moGprof por cada modelo de Gprof del vector */
			Iterator <ModeloGprof > i = vMoGprof.iterator( );
			
			while ( i.hasNext( ) ) {
				ModeloGprof moG = i.next( );
				
				Node nMoGprof = gprof.appendChild( docXML.createElement( "moGprof" ) );
				
				Element e; 
				
				/** Primero el nombre del futuro programa (nombre pestaña). */
				e = docXML.createElement( "nombreSalidaGprof" );
				e.setTextContent( moG.getNombreSalidaGprof( ) );
				nMoGprof.appendChild( e );
				
				/** Segundo el directorio de trabajo. */
				e = docXML.createElement( "dirTrabajo" );
				e.setTextContent( moG.getDirectorioTrabajo( ).getAbsolutePath( ) );
				nMoGprof.appendChild( e );				
				
				/** Tercero: programa. */
				File programa = moG.getPrograma( );
				if( programa != null ) {
					e = docXML.createElement( "pathPrograma" );
					e.setTextContent( programa.getAbsolutePath( ) );
					nMoGprof.appendChild( e );
				}
				
				/** Cuarto: archivo datos. */
				File archivoDatos = moG.getArchivoDatos( );
				if( archivoDatos != null ) {
					e = docXML.createElement( "pathArchivoDatos" );
					e.setTextContent( archivoDatos.getAbsolutePath( ) );
					nMoGprof.appendChild( e );
				}
				
				/** Quinto: fichero salida. */
				File ficheroSalida = moG.getFicheroSalida( );
				if( ficheroSalida != null ) {
					e = docXML.createElement( "pathFicheroGprof" );
					e.setTextContent( ficheroSalida.getAbsolutePath( ) );
					nMoGprof.appendChild( e );
				}
				
				/** Sexto: salida. */
				String salida = moG.getSalidaGprof( );
				if( salida != null && salida.compareTo( "" ) != 0 ) {
					e = docXML.createElement( "salidaGprof" );
					e.setTextContent( salida );
					nMoGprof.appendChild( e );
				}
				
				/** Séptimo: el comando de gprof no se puede modificar, pero sí 
				 * sus opciones */
				Vector < String > vs = moG.getComandoGprof( );
				if ( !vs.equals( PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getComandoGprof( ) ) && vs.size( ) > 0 ) {
					Iterator < String > j = vs.iterator( );
				
					j.next( ); //pasar el primero comando no editable.
					
					Node opciones = docXML.createElement( "opcionesTraductor" );
					while (j.hasNext( )) {
						String cc = j.next( );
						
						e = docXML.createElement( "opcion" );
						e.setTextContent( cc );
						opciones.appendChild( e );
					}
					nMoGprof.appendChild( opciones );
				}
			}
		}
		
	}

	private  void escribirModelosTraduccion (Document docXML, Node sesion, Vector<ModeloTR> vMoTraduccion ) {

		/** sesion es el primer hijo del Root. */ 
		
		/** primero comprobaremos que hay algún modelo TR 
		 * para ver si añadimos esta rama.	 */
		if ( vMoTraduccion.size( ) > 0 ) {
			/** crear el hijo de la traducción. */
			Node traduccion = sesion.appendChild( docXML.createElement( "traduccion" ) );
			
			/** ahora crear un moTR por cada modelo de Traduccion del vector */
			Iterator <ModeloTR > i = vMoTraduccion.iterator( );
			
			while ( i.hasNext( ) ) {
				ModeloTR moT = i.next( );
				
				Node nMoTraduccion = traduccion.appendChild( docXML.createElement( "moTraduccion" ) );
				
				Element e; 
				
				/** Primero el nombre del futuro fichero XML (nombre pestaña). */
				e = docXML.createElement( "nombreXML" );
				e.setTextContent( moT.getNombreXML( ) );
				nMoTraduccion.appendChild( e );
				
				/** Segundo el directorio de trabajo. */
				e = docXML.createElement( "dirTrabajo" );
				e.setTextContent( moT.getDirectorioTrabajo( ).getAbsolutePath( ) );
				nMoTraduccion.appendChild( e );								
				
				
				/** Tercero: el path del fichero salida. */
				File fSalida = moT.getFicheroSalida( );
				if( fSalida != null ) {
					e = docXML.createElement( "pathSalidaGprof" );
					e.setTextContent( fSalida.getAbsolutePath( ) );
					nMoTraduccion.appendChild( e );
				}
		
				/** Cuarto: el path del fichero XML. */
				File fXML = moT.getFicheroXML( );
				if( fXML != null ) {
					e = docXML.createElement( "pathGprofXML" );
					e.setTextContent( fXML.getAbsolutePath( ) );
					nMoTraduccion.appendChild( e );
				}
				
				/** Quinto: salida. */
				String salida = moT.getSalidaXML( );
				if( salida != null && salida.compareTo( "" ) != 0 ) {
					e = docXML.createElement( "salidaXML" );
					e.setTextContent( salida );
					nMoTraduccion.appendChild( e );
				}
				
				/** Sexto: el comando de traducción individual (traductor y sus opciones) */
				Vector < String > vs = moT.getComandoTR( );
				if ( !vs.equals( PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getComandoTraductor( ) ) && vs.size( ) > 0 ) {
					Iterator < String > j = vs.iterator( );
				
					e = docXML.createElement( "traductor" );
					e.setTextContent( j.next( ) );
					nMoTraduccion.appendChild( e );
					
					Node opciones = docXML.createElement( "opcionesTraductor" );
					while (j.hasNext( )) {
						String cc = j.next( );
						
						e = docXML.createElement( "opcion" );
						e.setTextContent( cc );
						opciones.appendChild( e );
					}
					nMoTraduccion.appendChild( opciones );
				}
			}
		}
	}

	private  void escribirModelosEstadisticas (Document docXML, Node sesion, Vector < ModeloEstadistica > vMoEstadisticas) {
		
		/** sesion es el primer hijo del Root. */ 
		
		/** primero comprobaremos que hay algún modelo Estadistica 
		 * para ver si añadimos esta rama.	 */
		if ( vMoEstadisticas.size( ) > 0 ) {
			/** crear el hijo de la traducción. */
			Node estadisticas = sesion.appendChild( docXML.createElement( "estadisticas" ) );
			
			/** ahora crear un moEstadistica por cada modelo de Estadisticas del vector */
			Iterator <ModeloEstadistica > i = vMoEstadisticas.iterator( );
			
			while ( i.hasNext( ) ) {
				ModeloEstadistica moS = i.next( );
				
				Node nMoEstadistica = estadisticas.appendChild( docXML.createElement( "moEstadistica" ) );
				
				Element e; 
				
				
				/** Primero el path del fichero XML. */
				File festadisticas = moS.getFicheroXML( );
				if( festadisticas != null ) {
					e = docXML.createElement( "pathGprofXML" );
					e.setTextContent( festadisticas.getAbsolutePath( ) );
					nMoEstadistica.appendChild( e );
				}
				
			}
		}

	}
}
