/**
 * 
 */
package interfaz.dialogos.dialogoAbrir;


import java.awt.Component;
import java.awt.HeadlessException;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.ResourceBundle;

import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;

/**
 * @author angel2
 * 
 */
public class DialogoAbrir extends JFileChooser {

	/** serial version UID */
	private static final long	serialVersionUID	= 7631914512157381242L;
	
	/** recursos para dar soporte sobre idiomas */
	ResourceBundle					idiomas				= ResourceBundle
																.getBundle( "interfaz.idiomas.Recursos" );


	/** crear los filtros para todos los tipos de ficheros */
	FortranFilter				filtroFortran		= new FortranFilter( );
	CFilter						filtroC				= new CFilter( );
	PascalFilter				filtroPascal		= new PascalFilter( );
	LenguajesFilter				filtroLenguajes		= new LenguajesFilter( );
	ExeFilter					filtroEjecutables	= new ExeFilter();
	XMLFilter					filtroXML			= new XMLFilter();
	XMLgprofFilter				filtroXMLgprof		= new XMLgprofFilter();
	gprofFilter					filtrogprof			= new gprofFilter();
	OutFilter					filtroOut			= new OutFilter();
	SesionFilter				filtroSesion		= new SesionFilter();
	TodosFilter					filtroTodos			= new TodosFilter();
	SoloDirectoriosFilter		filtroSoloDirectorios = new SoloDirectoriosFilter();

	GprofFileView				vistaFicheros		= new GprofFileView( );

	/** botón para que se vean o no los archivos ocultos
	 * usar setFileHidingEnabled(false) de esta misma clase(JFileChooser). */ 
	private JCheckBox			showHidenFiles		= new JCheckBox(
															idiomas.getString( "eti_ocultos" ), false );
	{
		showHidenFiles.setToolTipText( idiomas.getString( "tip_ocultos" ) );
		showHidenFiles.addItemListener( new ItemListener( ) {
			@Override
			public void itemStateChanged ( ItemEvent item ) {
				setFileHidingEnabled( item.getStateChange( ) == ItemEvent.SELECTED ? false
						: true );
			}
		} );
	}
	
	/** Esto es para deseleccionar los ficheros que hubiera previamente*/
	File noFile = new File(""); 
	File [] noFiles = new File[0];


	public DialogoAbrir ( ) {
		super();
		/** valores iniciales */
		setFileHidingEnabled( !showHidenFiles.isSelected() );
		
		setFileView( vistaFicheros );
		setAccessory( showHidenFiles );
		setAcceptAllFileFilterUsed( false ); //hay uno propio
	}

	protected void añadirTodosLosFiltros ( ) {
		addChoosableFileFilter( filtroC );
		addChoosableFileFilter( filtroPascal );
		addChoosableFileFilter( filtroFortran );
		addChoosableFileFilter( filtroLenguajes );
		addChoosableFileFilter( filtroEjecutables );
		addChoosableFileFilter( filtrogprof );
		addChoosableFileFilter( filtroOut );
		addChoosableFileFilter( filtroXML );
		addChoosableFileFilter( filtroXMLgprof );
		addChoosableFileFilter( filtroTodos );
	}
	
	/** Mostrar el diálogo de abrir con todos los filtros. */
	@Override
	public int showOpenDialog( Component parent ) {
		this.setDialogTitle( idiomas.getString( "dlg_defaultOpen_title" ) );
		this.setSelectedFile( noFile );
		this.setSelectedFiles ( noFiles );
		this.setSelectedFile( null );
		this.setSelectedFiles( null );
		this.setFileFilter( null );
		añadirTodosLosFiltros();
		
		return super.showOpenDialog( parent );
	}

/*	public int showOpenDialog ( Component parent , FileFilter filtro )
			throws HeadlessException {
		int resultado;
		resetChoosableFileFilters( );
		addChoosableFileFilter( filtro );
		resultado = showOpenDialog( parent );
		añadirTodosLosFiltros( );
		return resultado;
	}*/

	/** Mostrar el diálogo de abrir para vista Edición y Compilación. */
	public int showOpenDialogEdicion ( Component parent, boolean multiselection )
			throws HeadlessException {
		
		this.setDialogTitle( idiomas.getString("dlg_edicion_title") );
		this.resetChoosableFileFilters( );
		this.setSelectedFile( noFile );
		this.setSelectedFiles ( noFiles );
		this.setSelectedFile( null );
		this.setSelectedFiles( null );
		this.setMultiSelectionEnabled( multiselection );
		
		addChoosableFileFilter( filtroLenguajes );
		addChoosableFileFilter( filtroC );
		addChoosableFileFilter( filtroPascal );
		addChoosableFileFilter( filtroFortran );
		addChoosableFileFilter( filtrogprof );
		addChoosableFileFilter( filtroTodos );
		setFileFilter( filtroLenguajes );
		
		return super.showOpenDialog( parent );
	}
	
	/** Mostrar el diálogo de abrir para vista Ejecución. */
	public int showOpenDialogEjecucion ( Component parent, boolean multiselection )
			throws HeadlessException {
		
		this.setDialogTitle( idiomas.getString( "dlg_ejecucion_title" ) );
		resetChoosableFileFilters( );
		this.setSelectedFile( noFile );
		this.setSelectedFiles ( noFiles );
		this.setSelectedFile( null );
		this.setSelectedFiles( null );
		this.setMultiSelectionEnabled( multiselection );
		
		addChoosableFileFilter( filtroEjecutables );
		addChoosableFileFilter( filtroTodos );
		setFileFilter( filtroEjecutables );
		
		return super.showOpenDialog( parent );
	}
	
	/** Mostrar el diálogo de abrir para vista Gprof. */
	public int showOpenDialogGprof ( Component parent , boolean multiselection )
			throws HeadlessException {
		
		this.setDialogTitle( idiomas.getString( "dlg_gprof_title" ) );
		this.resetChoosableFileFilters( );
		this.setSelectedFile( noFile );
		this.setSelectedFiles ( noFiles );
		this.setSelectedFile( null );
		this.setSelectedFiles( null );
		this.setMultiSelectionEnabled( multiselection );
		
		addChoosableFileFilter( filtrogprof );
		addChoosableFileFilter( filtroTodos );
		setFileFilter( filtrogprof );
		
		return super.showOpenDialog( parent );
	}

	/** Mostrar el diálogo de abrir para vista traducción. */
	public int showOpenDialogSalida ( Component parent , boolean multiselection )
			throws HeadlessException {
		
		this.setDialogTitle( idiomas.getString("dlg_salida_title") );
		resetChoosableFileFilters( );
		this.setSelectedFile( noFile );
		this.setSelectedFiles ( noFiles );
		this.setSelectedFile( null );
		this.setSelectedFiles( null );
		this.setMultiSelectionEnabled( multiselection );
		
		addChoosableFileFilter( filtroOut );
		addChoosableFileFilter( filtroTodos );
		setFileFilter( filtroOut );
		
		return super.showOpenDialog( parent );
	}

	/** Mostrar el diálogo de abrir para vista estadísticas. */
	public int showOpenDialogXML ( Component parent , boolean multiselection )
			throws HeadlessException {

		this.setDialogTitle( idiomas.getString( "dlg_xml_title" ) );
		resetChoosableFileFilters( );
		this.setSelectedFile( noFile );
		this.setSelectedFiles ( noFiles );
		this.setSelectedFile( null );
		this.setSelectedFiles( null );
		this.setMultiSelectionEnabled( multiselection );
		
		addChoosableFileFilter( filtroXMLgprof );
		addChoosableFileFilter( filtroXML );
		addChoosableFileFilter( filtroTodos );
		setFileFilter( filtroXMLgprof );
		
		return super.showOpenDialog( parent );
	}
	
	public int showOpenDialogTODO( Component parent, boolean multiseleccion ) {
		
		this.setDialogTitle( idiomas.getString( "dlg_defaultOpen_title" ) );
		this.setSelectedFile( noFile );
		this.setSelectedFiles ( noFiles );
		this.setSelectedFile( null );
		this.setSelectedFiles( null );
		this.setFileFilter( null );
		//this.setAcceptAllFileFilterUsed( true );
		this.añadirTodosLosFiltros( );
		this.setMultiSelectionEnabled( multiseleccion );
		
		return super.showOpenDialog( parent );
	}
	
	/** Mostrar el diálogo de abrir para vista Edición y Compilación. */
	public int showOpenDialogSesion ( Component parent, boolean multiselection )
			throws HeadlessException {
		
		this.setDialogTitle( idiomas.getString("dlg_sesion_title") );
		this.resetChoosableFileFilters( );
		this.setSelectedFile( noFile );
		this.setSelectedFiles ( noFiles );
		this.setSelectedFile( null );
		this.setSelectedFiles( null );
		this.setMultiSelectionEnabled( multiselection );
		
		addChoosableFileFilter( filtroSesion );
		addChoosableFileFilter( filtroTodos );
		
		return super.showOpenDialog( parent );
	}
	
	public int showSaveDialogSesion ( Component parent ) {
		
		this.setDialogTitle( idiomas.getString("dlg_sesion_save_title") );
		this.resetChoosableFileFilters( );
		this.setSelectedFile( noFile );
		this.setSelectedFiles ( noFiles );
		this.setSelectedFile( null );
		this.setSelectedFiles( null );
		this.setMultiSelectionEnabled( false );
		
		addChoosableFileFilter( filtroSesion );
		addChoosableFileFilter( filtroTodos );
		
		return super.showSaveDialog( parent );
	}

	
	/**
	 * Función main para pruebas.
	 */
	public static void main ( String [] args ) {

		SwingUtilities.invokeLater( new Runnable () {
			public void run () {
				new DialogoAbrir( ).showOpenDialog( new JFrame( "prueba" ) );				
			}
		});
		
		SwingUtilities.invokeLater( new Runnable () {
			public void run () {
				new DialogoAbrir( ).showOpenDialogEdicion( new JFrame( "prueba" ), false );				
			}
		});
		
		SwingUtilities.invokeLater( new Runnable () {
			public void run () {
				new DialogoAbrir( ).showOpenDialogEjecucion( new JFrame( "prueba" ), false );				
			}
		});

		SwingUtilities.invokeLater( new Runnable () {
			public void run () {
				new DialogoAbrir( ).showOpenDialogGprof ( new JFrame( "prueba" ), false );				
			}
		});
		
		SwingUtilities.invokeLater( new Runnable () {
			public void run () {
				new DialogoAbrir( ).showOpenDialogSalida( new JFrame( "prueba" ), false );				
			}
		});
		
		SwingUtilities.invokeLater( new Runnable () {
			public void run () {
				new DialogoAbrir( ).showOpenDialogXML( new JFrame( "prueba" ), false );				
			}
		});
		
		SwingUtilities.invokeLater( new Runnable () {
			public void run () {
				new DialogoAbrir( ).showOpenDialogTODO( new JFrame( "prueba" ), false );				
			}
		});
	}

}
