package interfaz.vcompilacion;

import java.awt.Color;
import java.awt.Font;
import java.io.File;
import java.io.IOException;

import java.util.ResourceBundle;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import interfaz.acciones.CrearYGestionarAcciones;
import interfaz.dialogos.error.DialogoError;
import interfaz.semaforo.JSemaforo;

import modelo.vcompilacion.ModeloCompilacion;
import negocio.PreferenciasSingleton;
import negocio.ejecutarOrdenes.InterfazOrdenes;
import net.miginfocom.swing.MigLayout;

public class JPanelCompilacion extends JPanel implements ChangeListener,
		InterfazOrdenes {

	/** recursos para dar soporte sobre idiomas */
	ResourceBundle				idiomas				= ResourceBundle
															.getBundle( "interfaz.idiomas.Recursos", 
																	PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	/** recursos de iconos etc... */
	ResourceBundle				recursos			= ResourceBundle
															.getBundle( "interfaz.Recursos" , 
																	PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	/** recursos de idiomas para errores */
	static ResourceBundle errores = ResourceBundle.getBundle("interfaz.errores.Errores", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );

	private static final long	serialVersionUID	= -7191529819222353003L;
	
	/************************ PROPIEDADES ***********************/
	
	private JLabel				etiCmd				= new JLabel(
															idiomas
																	.getString( "eti_comando" ) );
	private JTextField			txtCmd				= new JTextField( );
	{
		txtCmd.setEditable( false );
	}
	
	private JLabel				etiDir				= new JLabel(
															idiomas
																	.getString( "eti_directorioTrabajo" ) );
	private JTextField			txtDir				= new JTextField( );
	{
		txtDir.setEditable( false );
	}
	
	private JLabel				etiStatus			= new JLabel(
															idiomas
																	.getString( "eti_estado" ) );
	private JSemaforo			imgStatus			= new JSemaforo( );
	private JLabel				etiSalida			= new JLabel(
															idiomas
																	.getString( "eti_salida" ) );
	private JTextArea			atxSalida			= new JTextArea( );
	{
		atxSalida.setEditable( false );
		atxSalida.setBackground( Color.BLACK );// fondo negro
		atxSalida.setForeground( Color.WHITE );// letras blancas
		//atxSalida.setFont( atxSalida.getFont( ).deriveFont( Font.BOLD ) );// negritas
		atxSalida.setFont( PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getEstadisticasFont( ).deriveFont( Font.BOLD ) );// negritas
		atxSalida.setLineWrap( true );
		atxSalida.setWrapStyleWord( true );
	}
	private JLabel				etiFicheros			= new JLabel(
															idiomas
																	.getString( "eti_ficheros" ) );
//	private DefaultListModel	molstFicheros		= new DefaultListModel( );
	private JList				lstFicheros			= new JList( /*molstFicheros*/ );
	{
		lstFicheros
				.setSelectionMode( ListSelectionModel.MULTIPLE_INTERVAL_SELECTION );
	}
	private JButton				botAñadir			= new JButton( );
	private JButton				botQuitar			= new JButton( );
	private JButton				botCompilar			= new JButton( );
	/*private JButton				botNombrePrograma	= new JButton( );*/
	private JButton				botCambiarDT		= new JButton( );
	private JButton				botEditarCmd		= new JButton( );
	
	/************************ CONTRUCTOR ***********************/
	private ModeloCompilacion	moCompilacion = null;

	public JPanelCompilacion ( ) {
		super( new MigLayout( "fill" ) );

		setModel( null );
		crearComponentes( );		
	}
	
	public JPanelCompilacion ( ModeloCompilacion mo ) {
		super( new MigLayout( "fill" ) );

		setModel( mo );
		crearComponentes( );
	}
	
	/************************ METODOS ***********************/
	public boolean isVacio () {
		return this.moCompilacion == null;
	}
	
	
	protected void setModel ( ModeloCompilacion mo ) {
		this.moCompilacion = mo;
		
		if ( this.moCompilacion != null ) {
			if( this.moCompilacion.getDirectorioTrabajo( ) == null ) 
				this.moCompilacion.setDirectorioTrabajo( PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias().getWorkspaceCompilacion( ) );
			this.moCompilacion.addChangeListener( this );
			this.lstFicheros.setModel( this.moCompilacion.getFicherosListModel( ) );
			/*this.lstFicheros.addListSelectionListener( moCompilacion );*/
		}
		actualizarModel();
	}
	
	public ModeloCompilacion getModel () {
		return this.moCompilacion;
	}

	protected void crearComponentes ( ) {
		setBorder( new EtchedBorder( ) );
		setOpaque( true );

		// activar botones:
		{
			botAñadir.setHideActionText( true );
			botAñadir.setAction( CrearYGestionarAcciones.getInstance().getAccionAñadirFicheros( ) );

			botQuitar.setHideActionText( true );
			botQuitar.setAction( CrearYGestionarAcciones.getInstance().getAccionQuitarFicheros( ) );
			
		}
		
		// construir paneles superior e inferior:
		JPanel panelSuperior = new JPanel( new MigLayout( "fill" ) );
		{
			panelSuperior.add( etiFicheros ,
					"wmin 50lp, growx, pushx, cell 0 0 2 1" );
			panelSuperior.add( botAñadir ,
					"gapbefore unrel, cell 1 1 1 2, flowy, top" );
			panelSuperior.add( new JScrollPane( lstFicheros ) ,
					"wmin 50lp, grow, push, cell 0 1 1 2" );
			panelSuperior.add( botQuitar , "gapbefore unrel, cell 1 1 1 2" );
		}

		{
			botCompilar.setHideActionText( true );
			botCompilar.setAction( CrearYGestionarAcciones.getInstance().getAccionCompilar( ) );

			/*
			 * botNombrePrograma.setHideActionText( true );
			 * botNombrePrograma.setAction( CrearYGestionarAcciones.getInstance().getAccionNombreEjecutable( ) );
			 */

			this.botCambiarDT.setAction( CrearYGestionarAcciones.getInstance()
					.getAccionCambiarDirectorioTrabajoCompilacion( ) );
			this.botCambiarDT.setIcon( null );
			this.botCambiarDT.setText( "..." );

			this.botEditarCmd.setAction( CrearYGestionarAcciones.getInstance().
					getAccionEditarComandoCompilacion( ) );
			this.botEditarCmd.setIcon( null );
			this.botEditarCmd.setText( "..." );

			// txtCmd.setAction(
			// gui.getAccionCambiarDirectorioTrabajoCompilacion( ) );
			// this.txtDir.setAction( gui.getAccionEditarComandoCompilacion( )
			// );
		}

		JPanel panelInferior = new JPanel( new MigLayout( "fill" ) );

		JPanel pnlEstatus;
		{
			pnlEstatus = new JPanel( new MigLayout( ) );
			pnlEstatus.add( etiStatus , "wrap" );
			pnlEstatus.add( imgStatus , "alignx center, aligny top" );
			pnlEstatus.setBorder( BorderFactory
					.createEtchedBorder( EtchedBorder.LOWERED ) );
		}
		panelInferior.add( pnlEstatus , "sizegroupy a" );

		JPanel pnlCompilando;
		{
			pnlCompilando = new JPanel( new MigLayout( ) );
			pnlCompilando.add( etiCmd , "sgy Cmd" );
			pnlCompilando.add( txtCmd , "growx,pushx,wmin 100pt, sgy Cmd" );
			pnlCompilando.add( botEditarCmd , "sgy Cmd, wrap" );
			pnlCompilando.add( etiDir , "sgy Cmd" );
			pnlCompilando.add( txtDir , "growx,pushx,wmin 60pt, sgy Cmd" );
			pnlCompilando.add( botCambiarDT , "sgy Cmd, wrap" );
			pnlCompilando.setBorder( BorderFactory
					.createEtchedBorder( EtchedBorder.LOWERED ) );
		}
		panelInferior.add( pnlCompilando , "growx, pushx, sizegroupy a" );
		panelInferior.add( botCompilar , "gapbefore unrel, wrap" );
		/*
		 * panelInferior.add( botCompilar , "gapbefore unrel, split 2, flowy" );
		 * panelInferior.add( botNombrePrograma , "gapbefore unrel, wrap" );
		 */

		panelInferior.add( etiSalida , "wrap" );
		panelInferior.add( new JScrollPane( atxSalida ) , "span, grow" );

		JSplitPane panelDivide = new JSplitPane( JSplitPane.VERTICAL_SPLIT );
		panelDivide.setTopComponent( panelSuperior );
		panelDivide.setBottomComponent( panelInferior );
		panelDivide.setOneTouchExpandable( true );

		add( panelDivide , "grow" );
	}

	/**
	 * quita los ficheros de la lista
	 */
	public Object [] getFicherosSeleccionados ( ) {
		return isVacio()? null : lstFicheros.getSelectedValues( );
	}

	public void empezarOrden ( String comando ) {
		if ( !isVacio( ) ) {
			imgStatus.setAMBAR( );
			imgStatus.setToolTipText( idiomas.getString( "eti_ejecutando" ) );
			txtCmd.setText( comando );
		}
	}

	@Override
	public void ordenErrorIndeterminado( String comando, File directorio, Exception e ){
		String saltoLinea = "\n";
		String descripcion;

		descripcion = errores.getString( "err_errorEjecuciónOrden" )
				+ saltoLinea + errores.getString( "err_errorEjecuciónOrden_orden" )
				+ saltoLinea + comando
				+ saltoLinea + errores.getString( "err_errorEjecución_directorio" )
				+ saltoLinea + directorio.getName( )
				+ saltoLinea
				+ saltoLinea + errores.getString( "err_detalles:" )
				+ saltoLinea + e.getMessage( );
		
		DialogoError.getInstance( ).showErrorDialog( descripcion );
	}
	
	public void terminarOrden ( int resultado , String salida, String errores ) {
		if ( !isVacio( ) ) {
			atxSalida.append( salida + errores );

			if ( resultado == 0 ) {

				imgStatus.setVERDE( );
				imgStatus.setToolTipText( idiomas
						.getString( "eti_SalidaCompilacionExito" ) );
				atxSalida.append( idiomas
						.getString( "eti_SalidaCompilacionExito" ) );
			} else {

				imgStatus.setROJO( );
				imgStatus.setToolTipText( idiomas
						.getString( "eti_SalidaCompilacionFracaso" ) );
				atxSalida.append( idiomas
						.getString( "eti_SalidaCompilacionFracaso" ) );
			}
			atxSalida.append( "\n\n" );

			/*
			 * final String finalT = t;
			 * 
			 * EventQueue.invokeLater(new Runnable() { public void run() {
			 * ponerSalida(finalT); } });
			 */
		}
	}

	@Override
	public void stateChanged ( ChangeEvent e ) {
		if ( e.getSource( ) instanceof ModeloCompilacion ) {
			this.actualizarModel( );
		}
		
	}

	private void actualizarModel ( ) {

		if ( this.moCompilacion != null ) {
	
			/*txtDir*/
			String directorioTrabajo = new String( "" );
			if ( this.moCompilacion.getDirectorioTrabajo( ) != null ) {
				try {
					directorioTrabajo = this.moCompilacion
							.getDirectorioTrabajo( ).getCanonicalPath( );
				} catch ( IOException e ) {
					e.printStackTrace( );
					directorioTrabajo = idiomas
							.getString( "eti_errorDirectorioTrabajo" );
				}
			}
			txtDir.setText( directorioTrabajo );
			
			/* txtCmd */
			String [] comp = this.moCompilacion.getComandoCompilacion( );
			String comps = new String( );
			for ( int i = 0 ; i < comp.length ; i++ ) {
				comps = comps + comp[i] + " ";
			}
			this.txtCmd.setText( comps );
		}
		else {
			/*txtDir*/
			txtDir.setText( "" );
			
			/* txtCmd */
			this.txtCmd.setText( "" );
			
		}

	}

}
