/**
 * Clase VistaEjecucion.java es una aplicación Java 6.
 * Clase que gestiona la vista ejecución. Desde lo visual hasta las acciones.
 * Pertenece a la aplicación guigprof para el proyecto fin de carrera. 
 * Autor: Ángel García Fernández.
 * Tutor: Francisco Palomo Lozano e Inmaculada Medina Bulo
 * Fecha: Agosto de 2007.
 */

package interfaz.vedicion;


import interfaz.GUI;
import interfaz.Vista;
import interfaz.acciones.CrearYGestionarAcciones;
import interfaz.acciones.InterfazAbrirEdicion;
import interfaz.acciones.InterfazActualizar;
import interfaz.acciones.InterfazCerrar;
import interfaz.acciones.InterfazEditar;
import interfaz.dialogos.dialogoAbrir.DialogoAbrir;
import interfaz.dialogos.error.DialogoError;

import java.awt.Component;
import java.awt.EventQueue;
import java.awt.Font;
import java.io.File;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import modelo.ficheros.UtilidadesExtensionFichero;
import modelo.ficheros.UtilidadesFichero;
import modelo.vedicion.ModeloEdicion;
import negocio.PreferenciasSingleton;
import negocio.ejecutarOrdenes.EjecutorComandos;
import net.miginfocom.swing.MigLayout;


/**
 * @author angel2
 *
 */
public class VistaEdicion extends JPanel implements Vista, InterfazAbrirEdicion, InterfazCerrar,
								InterfazEditar, InterfazActualizar {

	/** serial version uid*/
	private static final long serialVersionUID = 759911159280491446L;
	
	/** Esta es la clase principal, ventana principal del interfaz o gui. */
	private GUI gui;

	/** recursos para dar soporte sobre idiomas */
	static ResourceBundle idiomas = ResourceBundle.getBundle("interfaz.idiomas.Recursos", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	/** recursos de iconos etc... */
	static ResourceBundle recursos = ResourceBundle.getBundle("interfaz.Recursos", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	/** recursos de idiomas para errores */
	static ResourceBundle errores = ResourceBundle.getBundle("interfaz.errores.Errores", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	
	/** dialogo Abrir **/ 
	DialogoAbrir					dialogoAbrir		= new DialogoAbrir( );
	
	/** barra de tareas específica de esta vista. */
	JToolBar barraDeTareas = new JToolBar();

	/**********Constructor PANEL Y GESTION PANEL***************/
	public VistaEdicion ( GUI gui ) {
		super(new MigLayout("fill"));
		this.gui = gui;
		idiomas = ResourceBundle.getBundle("interfaz.idiomas.Recursos", PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale());
		crearPanel();
		crearBarraTareas();
	}

	/** Panel general de la vista compilación */
	//private JPanel panel;
	/** Panel de pestañas inferior, donde se muestra la salida de la compilación */
	private JTabbedPane	pestañas	= new JTabbedPane( );
	{
		ChangeListener cambioPestaña = new ChangeListener () {

			@Override
			public void stateChanged ( ChangeEvent arg0 ) {
				activarActions(visible);
			}
			
		};
		pestañas.addChangeListener( cambioPestaña );
	}

	/*public JPanel getPanel () {
		return panel;
	}*/

	protected void crearPanel () {
		/*Aquí hay que hacer el panel*/
		setOpaque(true);

		/* Se añade las pestañas. */
		add(pestañas, "grow");
		
		/* A continuación, se crea un panel inicial vacío. */
		pestañas.addTab( "" , new JPanelEdicion( ) );
	}
	
	protected void añadirPestaña ( ModeloEdicion me ) {
		File f = me.getFichero();
		/* leer fichero */
		UtilidadesFichero utilidadesFichero = new UtilidadesFichero();

		final String nombre = f.getName( );
		final String camino = utilidadesFichero.caminoFichero(f);
		final Icon icon = (Icon) UtilidadesExtensionFichero.getIcono(f);
		
		JPanelEdicion pestañaNueva = new JPanelEdicion( me );

		/** Crear la pestaña correctamente */
		pestañas.addTab(nombre, pestañaNueva);
		int index = pestañas.indexOfComponent(pestañaNueva);
		
		javax.swing.JLabel tabRenderer = new javax.swing.JLabel ( nombre );
		tabRenderer.setIcon(icon);
		tabRenderer.setIconTextGap(5);
		tabRenderer.setHorizontalTextPosition(SwingConstants.RIGHT);
		//tabRenderer.setToolTipText(camino);
				
		pestañas.setTabComponentAt(index, tabRenderer);
		pestañas.setToolTipTextAt(index, camino);
		
		pestañas.setSelectedComponent(pestañaNueva);
	}

	protected void crearBarraTareas () {
		JButton botAbrirFuente = new JButton ();
		{
			//botAbrirFuente .setAction(new AccionAbrirFuente(this));
			botAbrirFuente .setHideActionText(true);
			botAbrirFuente .setAction(CrearYGestionarAcciones.getInstance().getAccionAbrirFuente());
		}

		JButton botCerrarFuente = new JButton ();
		{
			//botCerrarFuente.setAction(new AccionCerrar(this));
			botCerrarFuente.setHideActionText(true);
			botCerrarFuente.setAction(CrearYGestionarAcciones.getInstance().getAccionCerrar());
		}

		
		JButton botCompilar = new JButton ();
		{
			botCompilar.setHideActionText(true);
			botCompilar.setAction(CrearYGestionarAcciones.getInstance().getAccionCompilarFuente());
		}


		JButton botEditar = new JButton ();
		{
			//botEditar.setAction( new AccionEditar(this) );
			botEditar.setHideActionText(true);
			botEditar.setAction(CrearYGestionarAcciones.getInstance().getAccionEditar() );
		}

		JButton botActualizar = new JButton ();
		{
			botActualizar.setHideActionText(true);
			botActualizar.setAction(CrearYGestionarAcciones.getInstance().getAccionActualizar());
		}

		{
			barraDeTareas.setAutoscrolls(true);
			barraDeTareas.setFloatable(false);
			barraDeTareas.add(botAbrirFuente);
			barraDeTareas.add(botCerrarFuente);
			barraDeTareas.addSeparator( );
			barraDeTareas.add(botEditar);
			barraDeTareas.add( botActualizar );
			barraDeTareas.addSeparator();
			barraDeTareas.add(botCompilar);
		}
	}

	/*********** métodos *************/
	
	/** Se usa para recoger el fichero de la pestaña activa.
	 * @return
	 */
	public void editar ( ) {

		JPanelEdicion pe = getPanelActivo( );
		if ( pe != null ) {
			ModeloEdicion me = pe.getModel( );
			if ( me != null ) {
				final String [] orden = new String [] { PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getEditor( ),
						me.getFichero( ).getName( ) };
				final File dirTrabajo = me.getFichero( ).getParentFile( );

				try {
					/** realizar edición*/
					final Thread editor = new EjecutorComandos( orden , dirTrabajo , pe );
					
					editor.start( );
					
					/** hilo para actualizar botones cuando acabe la ejecución de edición */
					new Thread() {
						public void run () {
							try {
								editor.join();
								
								activarActions(visible);
								
							} catch ( InterruptedException e ) {
								DialogoError.getInstance( ).showErrorDialog( 
										errores.getString("err_errorEdicion") );
								e.printStackTrace( );
							}
							
						}
					}.start( );
				} catch ( Exception e ) {

					DialogoError.getInstance( ).showErrorDialog( 
							errores.getString("err_errorEdicion") );
				}
			}
											
		}
	}

	
	/** Para hacer el abrirFuente se mete el fichero y se lleva un control de los ficheros.
	 * Se hace en segundo plano para que se puedan abrir más de uno a la vez.
	 * */
	public void abrirEdicion (final ModeloEdicion me) {
		

		if ( me != null ) {
		
			final File f = me.getFichero( );
			
			if ( f != null ) {
			
				if( pestañas.getTabCount( ) == 1 && pestañas.getComponentAt( 0 ) instanceof JPanelEdicion && ( (JPanelEdicion) pestañas.getComponentAt( 0 )).isVacio() ) {
					
					pestañas.removeAll( );
			
					añadirPestaña( me );
			
				} else {
					EventQueue.invokeLater(new Runnable() {
						public void run() {	
							añadirPestaña( me );

						}	
					});
				}
			}
		}
	}

	/** Abrir un fichero fuente desde el dialogo, crear modelo y abrir */
	public void abrirEdicion ( ) {
		File [] ficheros;
		// quizás mejor en el event dispaching thread...
		/** Mostrar el dialogo de abrir (DialogoAbrir). */
		
		if ( dialogoAbrir.showOpenDialogEdicion( this, true ) == JFileChooser.APPROVE_OPTION ) {
			// *podría hacerlo para múltiples ficheros seleccionados...
			ficheros = dialogoAbrir.getSelectedFiles( );

			/**
			 * Comprobar fichero seleccionado y abrir una nueva pestaña.
			 */
			for ( int i = 0 ; i < ficheros.length ; i++ ) {

				if ( ficheros[i].isFile( ) && ficheros[i].canRead( ) ) {
					ModeloEdicion me = new ModeloEdicion();
					me.setFichero( ficheros[i] );
					this.abrirEdicion( me );
				} else {
					DialogoError.getInstance( ).showErrorDialog( 
							errores.getString("err_AbrirFuente") );
				}
			}
		}
		
	}
	
	/** Para actualizar los contenidos de un fichero abierto previamente. */
	@Override
	public void actualizar ( ) {
		JPanelEdicion pc = getPanelActivo( );
		if ( pc != null ) {
			ModeloEdicion mo = pc.getModel( );
			
			mo.actualizar();
		}
	}

	public void cambiarFuente ( Font f ) {
		for (int i = 0; i < pestañas.getTabCount( ); i ++ ) {
			Component c = pestañas.getComponentAt( i );
			if( c instanceof JPanelEdicion ) {
				( ( JPanelEdicion ) c  ).cambiarFuente( f );
			}
		}
	}
	
	/******************* pestañas ***********************/
	
	public Vector < ModeloEdicion > getModelos () {
		Vector < ModeloEdicion > vModelos = new Vector < ModeloEdicion >();
		
		for (int i = 0; i < pestañas.getTabCount( ); i ++ ) {
			Component c = pestañas.getComponentAt( i );
			if( c instanceof JPanelEdicion ) {
				ModeloEdicion moE = ( ( JPanelEdicion ) c ).getModel( );
				if ( moE != null )
					vModelos.add( moE );
			}
		}
		
		return vModelos;
	}
	
	public void cerrarPestañaActiva () {
		final int indicePestañaActiva = pestañas.getSelectedIndex();
		if (indicePestañaActiva != -1) {

			String cuestión = new String( idiomas.getString( "eti_edicion_seguroCerrar" ) );
			String título = new String( idiomas.getString( "eti_cerrar" ) );
			int está_usted_seguro= JOptionPane.showConfirmDialog(gui,
					cuestión,
					título,
					JOptionPane.YES_NO_OPTION );

			if(está_usted_seguro==JOptionPane.YES_OPTION){
					if( pestañas.getTabCount( ) > 1 ) {
							pestañas.removeTabAt(indicePestañaActiva);
						} else if( pestañas.getTabCount() == 1 ) {
							pestañas.removeAll( );
							pestañas.add( new JPanelEdicion () );
						}

			}
		}
	}

	/**
	 * @return
	 */
	public JToolBar getBarraDeTareas() {
		return barraDeTareas;
	}
	
	/** devuelve el modelo compilación del panel activo. Null si no hay ninguno; */
	public ModeloEdicion getModeloEdicionActivo() {
		ModeloEdicion mo = null;
		JPanelEdicion pc = getPanelActivo();
		if ( pc != null ) {
			mo = pc.getModel( );
		}
		return mo;
	}
	
	/** devuelve el panel activo. Null si no hay ninguno; */
	private JPanelEdicion getPanelActivo () {
		JPanelEdicion pc = null;
		if ( pestañas.getComponentCount() > 0 ) {
			Component c = pestañas.getSelectedComponent();
			if ( c instanceof JPanelEdicion ) {
				pc = (JPanelEdicion) c;
			}
		}
		return pc;
	}
	
	/****************Visibilidad********************/
	boolean visible = true;
	
	public void setActivo(boolean b){
		visible = b;
		setVisible(b);
		//también hay que anular las acciones que no correspondan.
		activarActions ( b );

		barraDeTareas.setEnabled(b);
		barraDeTareas.setVisible(b);
	}
	public boolean isActivo () {
		return visible;
	}
	public void activarActions ( boolean vistaActiva ) {
/*		boolean panelNoVacio = false;
		boolean siguientePaso = false;
		if ( vistaActiva && this.getPanelActivo( ) != null && !this.getPanelActivo( )
				.isVacio( ) ) {
			panelNoVacio = true;
		}
		if ( CrearYGestionarAcciones.getInstance( )!= null ) {
			CrearYGestionarAcciones.getInstance().getAccionEditar( ).setEnabled( panelNoVacio );
			CrearYGestionarAcciones.getInstance().getAccionActualizar( ).setEnabled( panelNoVacio );
			CrearYGestionarAcciones.getInstance().getAccionCerrar( ).setEnabled( panelNoVacio );
		
		if ( panelNoVacio == true && this.getPanelActivo( ).getModel( ).isCompilable( ) ) {
			siguientePaso = true;
		}
			CrearYGestionarAcciones.getInstance().getAccionCompilarFuente( ).setEnabled( siguientePaso );
		}*/
		
		boolean panelNoVacio = false;
		boolean siguientePaso = false;
		if ( vistaActiva && this.getPanelActivo( ) != null && !this.getPanelActivo( )
				.isVacio( ) ) {
			panelNoVacio = true;
		}
		if ( panelNoVacio == true && this.getPanelActivo( ).getModel( ).isCompilable( ) ) {
			siguientePaso = true;
		}
		
		CrearYGestionarAcciones a = CrearYGestionarAcciones.getInstance( );
		
		a.activarActionsEdicion( vistaActiva, panelNoVacio , siguientePaso );
	}

	public void cerrarTodo ( ) {
		pestañas.removeAll( );		
	}


}
