/**
 * Clase JPanelEjecucion.java es una aplicación Java 6.
 * Clase que especializa un panel (JPanel). En este panel se muestra
 *  la salida de la ejecución de un programa.
 * Usa un objeto ILineaComando para la ejecución del programa. 
 * Pertenece a la aplicación guigprof para el proyecto fin de carrera. 
 * Autor: Ángel García Fernández.
 * Tutor: Francisco Palomo Lozano e Inmaculada Medina Bulo
 * Fecha: Agosto de 2007.
 */
package interfaz.vejecucion;

import java.awt.Color;
import java.io.File;
import java.io.IOException;
import java.util.ResourceBundle;
import java.awt.Font;
//import javax.swing.JButton;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import modelo.vejecucion.ModeloEjecucion;
import negocio.PreferenciasSingleton;
import negocio.ejecutarOrdenes.InterfazOrdenes;
import interfaz.GUI;
import interfaz.acciones.CrearYGestionarAcciones;
import interfaz.dialogos.error.DialogoError;
import interfaz.semaforo.JSemaforo;

import net.miginfocom.swing.MigLayout;

/**
 * @author angel
 * 
 */
public class JPanelEjecucion extends JPanel implements ChangeListener,
		InterfazOrdenes {

	/**
	 * recursos para dar soporte sobre idiomas
	 */
	ResourceBundle					idiomas					= ResourceBundle
																	.getBundle( "interfaz.idiomas.Recursos" , 
																			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	/**
	 * recursos de iconos etc...
	 */
	ResourceBundle					recursos				= ResourceBundle
																	.getBundle( "interfaz.Recursos" , 
																			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	/** recursos de idiomas para errores */
	static ResourceBundle errores = ResourceBundle.getBundle("interfaz.errores.Errores", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );

	/************************ PROPIEDADES ***********************/

	/**
	 * 
	 */
	private static final long		serialVersionUID		= 1L;

	
	private JLabel				etiCmd				= new JLabel(
			idiomas
					.getString( "eti_comandoEjecucion" ) );	
	
	private JTextField			txtCmd				= new JTextField( );
	{
		txtCmd.setEditable( false );
	}
	
	private JLabel				etiStatus			= new JLabel(
			idiomas
					.getString( "eti_estado" ) );

	/*
	 * private JButton botArgumentos = new JButton (); { //coger acción, esto
	 * abre un diálogo para añadir los argumentos manualmente //vale por si hay
	 * que añadir alguno que sea una frase y pueda //haber problemas.... }
	 */
	private JLabel					etiDirectorioTrabajo	= new JLabel( );
	{
		etiDirectorioTrabajo.setText( idiomas.getString( "eti_directorioTrabajo" ) );
	}

	private JTextField				txtDirectorioTrabajo	= new JTextField( );
	{
		txtDirectorioTrabajo.setEditable( false );
	}

	private JSemaforo				semStatus				= new JSemaforo( );

	private JLabel					etiSalida				= new JLabel( );
	{
		etiSalida.setText( idiomas.getString( "eti_salida" ) );
	}

	private JTextArea				atxSalida				= new JTextArea( );
	{
		atxSalida.setEditable( false );
		atxSalida.setForeground( Color.WHITE );
		atxSalida.setBackground( Color.BLACK );
		atxSalida.setFont( atxSalida.getFont( ).deriveFont( Font.BOLD ) );
		atxSalida.setLineWrap( true );
		atxSalida.setWrapStyleWord( true );
	}

	private JLabel					etiEntrada				= new JLabel( );
	{
		etiEntrada.setText( idiomas.getString( "eti_entrada" ) );
	}
	
	private JLabel					etiArgumentos				= new JLabel( );
	{
		etiArgumentos.setText( idiomas.getString( "eti_argumentos" ) );
	}

	private JLabel					etiErrores				= new JLabel( );
	{
		etiErrores.setText( idiomas.getString( "eti_errores" ) );
	}



	private JLabel					etiEntradas			= new JLabel(
			idiomas
			.getString( "eti_entradas" ) );

	private JList				lstEntradas			= new JList( /*molstFicheros*/ );
	{
		lstEntradas
		.setSelectionMode( ListSelectionModel.MULTIPLE_INTERVAL_SELECTION );
	}

	private JList			lstArgumentos			= new JList();
	{
		lstArgumentos.setSelectionMode( ListSelectionModel.MULTIPLE_INTERVAL_SELECTION );
	}

	private JButton					botEjecutar				= new JButton( );
	private JButton					botAñadirArgumentos			= new JButton( );
	private JButton					botQuitarArgumentos			= new JButton( );
	private JButton					botAñadirEntradas			= new JButton( );
	private JButton					botQuitarEntradas			= new JButton( );
	private JButton					botAñadirFicheroEntradas	= new JButton( );
	private JButton					botCambiarDT		= new JButton( );

	{
		etiEntradas.setLabelFor( lstEntradas );
		etiArgumentos.setLabelFor( lstArgumentos );
		etiDirectorioTrabajo.setLabelFor( botCambiarDT );
	}
	
	/************************ CONTRUCTOR ***********************/
	private ModeloEjecucion	moEjecucion = null;

	public JPanelEjecucion ( GUI gui ) {
		super( new MigLayout( "insets panel,fill" ) );

		this.setModel( null );
		crearComponentes( );
	}

	public JPanelEjecucion ( GUI gui , ModeloEjecucion me ) {
		super( new MigLayout( "insets panel, fill" ) );

		setModel( me );
		crearComponentes( );

	}
	
	/************************ METODOS ***********************/
	public boolean isVacio () {
		return this.moEjecucion == null;
	}
	
	
	protected void setModel ( ModeloEjecucion me ) {
		this.moEjecucion = me;
		
		if ( this.moEjecucion != null ) {
			// inicializar pestaña a partir del modelo
			if ( this.moEjecucion.getDirectorioTrabajo( ) == null )
				this.moEjecucion.setDirectorioTrabajo( PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getWorkspaceEjecucion( ) );

			this.moEjecucion.addChangeListener( this );
			this.lstEntradas
					.setModel( this.moEjecucion.getEntradasListModel( ) );
			this.lstArgumentos.setModel( this.moEjecucion
					.getArgumentosListModel( ) );
		}
		actualizarModel( );
	}
	
	public ModeloEjecucion getModel () {
		return this.moEjecucion;
	}
	
	protected void crearComponentes ( ) {
		setBorder( new EtchedBorder( ) );
		setOpaque( true );

		// activar botones:
		{
			botAñadirEntradas.setHideActionText( true );
			botAñadirEntradas.setAction( CrearYGestionarAcciones.getInstance().getAccionAñadirEntradas( ) );

			botAñadirFicheroEntradas.setHideActionText( true );
			botAñadirFicheroEntradas.setAction( CrearYGestionarAcciones.getInstance().getAccionAñadirFicheroEntradas( ) );
						
			botQuitarEntradas.setHideActionText( true );
			botQuitarEntradas.setAction( CrearYGestionarAcciones.getInstance().getAccionQuitarEntradas( ) );

			botAñadirArgumentos.setHideActionText( true );
			botAñadirArgumentos.setAction( CrearYGestionarAcciones.getInstance().getAccionAñadirArgumentos( ) );

			botQuitarArgumentos.setHideActionText( true );
			botQuitarArgumentos.setAction( CrearYGestionarAcciones.getInstance().getAccionQuitarArgumentos( ) );
		
			botEjecutar.setHideActionText( true );
			botEjecutar.setAction( CrearYGestionarAcciones.getInstance().getAccionEjecutar( ) );
		}
		JPanel panelSuperiorDerecho = new JPanel( new MigLayout( "fill" ) );

		panelSuperiorDerecho.add( etiEntradas ,
				//"wmin 50lp, growx, pushx, cell 0 0 2 1" );
				"wmin 50lp, growx, pushx, cell 0 0 3 1" );
		panelSuperiorDerecho.add( botAñadirEntradas ,
				"gapbefore unrel, cell 1 1 1 2, flowy, top" );
		panelSuperiorDerecho.add( new JScrollPane( lstEntradas ) ,
				"wmin 50lp, grow, push, cell 0 1 1 2" );
		panelSuperiorDerecho.add( botQuitarEntradas , "gapbefore unrel, cell 1 1 1 2" );
		panelSuperiorDerecho.add( botAñadirFicheroEntradas ,
				"gapbefore rel, cell 2 1 1 2, top" );
				//"gapbefore unrel, cell 1 1 1 2" );

		
		JPanel panelSuperiorIzquierdo = new JPanel( new MigLayout( "fill" ) );

		panelSuperiorIzquierdo.add( etiArgumentos ,
				"wmin 50lp, growx, pushx, cell 0 0 2 1" );
		panelSuperiorIzquierdo.add( botAñadirArgumentos ,
				"gapbefore unrel, cell 1 1 1 2, flowy, top" );
		panelSuperiorIzquierdo.add( new JScrollPane( lstArgumentos ) ,
				"wmin 50lp, grow, push, cell 0 1 1 2" );
		panelSuperiorIzquierdo.add( botQuitarArgumentos , "gapbefore unrel, cell 1 1 1 2" );
		
		JSplitPane panelSuperior = new JSplitPane( JSplitPane.HORIZONTAL_SPLIT );
		panelSuperior.setRightComponent( panelSuperiorDerecho );
		panelSuperior.setLeftComponent( panelSuperiorIzquierdo );
		panelSuperior.setOneTouchExpandable( true );
		panelSuperior.setResizeWeight( 0.5 );
		/* panelSuperior.setDividerLocation( 0.5 ); // No hace nada, porque el split cambia de tamaño constantemente y se pierde. */

		{
			botEjecutar.setHideActionText( true );
			botEjecutar.setAction( CrearYGestionarAcciones.getInstance().getAccionEjecutar( ) );
			
			/*botNombrePrograma.setHideActionText( true );
			botNombrePrograma.setAction( CrearYGestionarAcciones.getInstance().getAccionNombreEjecutable( ) );*/
			
			this.botCambiarDT.setAction( CrearYGestionarAcciones.getInstance().getAccionCambiarDirectorioTrabajoEjecucion( ) );
			this.botCambiarDT.setIcon( null );
			this.botCambiarDT.setText( "..." );
			
		}

		JPanel panelInferior = new JPanel( new MigLayout( "fill" ) );

		JPanel pnlStatus;
		{
			pnlStatus = new JPanel ( new MigLayout() );
			pnlStatus.add( this.etiStatus , "wrap"  );
			pnlStatus.add( this.semStatus , "alignx center, aligny top" );
			pnlStatus.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
		}
		panelInferior.add( pnlStatus, "sizegroupy a" );
		
		JPanel pnlEjecutando;
		{
			pnlEjecutando = new JPanel ( new MigLayout() );
			pnlEjecutando.add( etiCmd , "sgy CmdDir");
			pnlEjecutando.add( txtCmd , "growx,pushx,wmin 100pt, sgy CmdDir, spanx, wrap" );
			pnlEjecutando.add( this.etiDirectorioTrabajo , "sgy CmdDir" );
			pnlEjecutando.add( this.txtDirectorioTrabajo , "growx,pushx,wmin 60pt, sgy CmdDir" );
			pnlEjecutando.add( botCambiarDT , "sgy CmdDir, wrap" );
			pnlEjecutando.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
		}
		panelInferior.add( pnlEjecutando , "growx, pushx, sizegroupy a" );
		panelInferior.add( botEjecutar , "gapbefore unrel, wrap" );
		/*panelInferior.add( botCompilar , "gapbefore unrel, split 2, flowy" );
		panelInferior.add( botNombrePrograma , "gapbefore unrel, wrap" );*/

		panelInferior.add( etiSalida , "wrap" );
		panelInferior.add( new JScrollPane( atxSalida ) , "span, grow" );
		/*panelInferior.add( etiErrores , "wrap,span" );
		panelInferior.add( new JScrollPane( atxErrores ) ,
				"grow, sizegroup igualT, wrap,span,hmin 20pt, wmin 100pt" );*/

		JSplitPane panelDivide = new JSplitPane( JSplitPane.VERTICAL_SPLIT );
		panelDivide.setTopComponent( panelSuperior );
		panelDivide.setBottomComponent( panelInferior );
		panelDivide.setOneTouchExpandable( true );

		add( panelDivide , "grow" ); 
	}

	public void empezarOrden ( String comando ) {
		if ( !isVacio( ) ) {
			semStatus.setAMBAR( );
			semStatus.setToolTipText( idiomas.getString( "eti_ejecutando" ) );
			txtCmd.setText( comando );
		}
	}

	@Override
	public void ordenErrorIndeterminado( String comando, File directorio, Exception e ){
		String saltoLinea = "\n";
		String descripcion;

		descripcion = errores.getString( "err_errorEjecuciónOrden" )
				+ saltoLinea + errores.getString( "err_errorEjecuciónOrden_orden" )
				+ saltoLinea + comando
				+ saltoLinea + errores.getString( "err_errorEjecución_directorio" )
				+ saltoLinea + directorio.getName( )
				+ saltoLinea
				+ saltoLinea + errores.getString( "err_detalles:" )
				+ saltoLinea + e.getMessage( );
		
		DialogoError.getInstance( ).showErrorDialog( descripcion );
	}

	
	public void terminarOrden ( int devolverResultadoEjecución ,
			String standar , String error ) {
		if ( !isVacio( ) ) {
			if ( devolverResultadoEjecución == 0 ) {
				semStatus.setVERDE( );
				semStatus.setToolTipText( idiomas
						.getString( "eti_SalidaEjecucionExito" ) );
			} else {
				semStatus.setROJO( );
				semStatus.setToolTipText( idiomas
						.getString( "eti_SalidaEjecucionFracaso" ) );
			}

			final String finalStandar = standar;
			final String finalError = error;

			atxSalida.setText( finalStandar + finalError );
			/*atxErrores.setText( finalError );*/

			/* Cambiar nombre a gmon.out. */
			try {
				File gmon = new File( this.moEjecucion.getDirectorioTrabajo( ) ,
				"gmon.out" );
				if ( gmon.exists( ) ) {
					File nuevoNombre = new File( this.moEjecucion
							.getDirectorioTrabajo( ) , this.moEjecucion
							.getNombreResultadosEjecucion( ) );
					if ( gmon.renameTo( nuevoNombre ) ) {
						this.moEjecucion.setFicheroResultados( nuevoNombre );
						DialogoError.getInstance( ).showInfoDialog(  
								errores.getString( "no-err_crearFichero" ) + nuevoNombre
								.getAbsolutePath( ) );
					} else {
						DialogoError.getInstance( ).showErrorDialog( 
								errores.getString( "err_renombrarGmon" ) + nuevoNombre
								.getCanonicalPath( ) ); 
					}
				} else {
					DialogoError.getInstance( ).showErrorDialog( 
							errores.getString( "err_crearGmon" ) + gmon
							.getCanonicalPath( ) ); 
				}
			} catch ( IOException e ) {
				DialogoError.getInstance( ).showErrorDialog(
						errores.getString( "err_io" )  );
				e.printStackTrace( );
			}
		}
	}
	
	/**
	 * obtener entradas seleccionadas de la lista
	 */
	public Object[] getEntradasSeleccionadas ( ) {
		return isVacio() ? null : this.lstEntradas.getSelectedValues( );
	}
	
	/**
	 * obtener argumentos seleccionados de la lista
	 */
	public Object [] getArgumentosSeleccionados ( ) {
		return isVacio() ? null : this.lstArgumentos.getSelectedValues( );
	}

	@Override
	public void stateChanged ( ChangeEvent e ) {
		if ( e.getSource( ) instanceof ModeloEjecucion ) {
			this.actualizarModel( );
		}
		
	}
	
	private void actualizarModel ( ) {

		if( this.moEjecucion != null ) {
		/* txtDir */
		String directorioTrabajo = new String( "" );
		if ( this.moEjecucion.getDirectorioTrabajo( ) != null ) {
			try {
				directorioTrabajo = this.moEjecucion.getDirectorioTrabajo( )
						.getCanonicalPath( );
			} catch ( IOException e ) {
				e.printStackTrace( );
				directorioTrabajo = idiomas
						.getString( "eti_errorDirectorioTrabajo" );
			}
		}
		txtDirectorioTrabajo.setText( directorioTrabajo );
		
		/* txtCmd */
		txtCmd.setText( moEjecucion.getComandoEjecucion() );
		} else {
			/* txtDir */
			txtDirectorioTrabajo.setText( "" );
			
			/* txtCmd */
			txtCmd.setText( "" );
		}

	}


}
