/**
 * 
 */
package interfaz.vejecucion;

import java.awt.Component;
import java.io.File;
import java.util.ArrayList;
import java.util.ResourceBundle;
import java.util.Vector;

import interfaz.GUI;
import interfaz.Vista;
import interfaz.acciones.CrearYGestionarAcciones;
import interfaz.acciones.InterfazAbrirEjecucion;
import interfaz.acciones.InterfazCambiarDirectorioTrabajo;
import interfaz.acciones.InterfazCambiarNombre;
import interfaz.dialogos.dialogoAbrir.DialogoAbrir;
import interfaz.dialogos.dialogoDirectorio.JDirectoryChooser;
import interfaz.dialogos.dialogoNuevoArgumento.DialogoNuevoArgumento;
import interfaz.dialogos.error.DialogoError;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import modelo.ficheros.UtilidadesFichero;
import modelo.vcompilacion.ModeloCompilacion;
import modelo.vejecucion.ModeloEjecucion;
import negocio.PreferenciasSingleton;
import negocio.guigprof;
import negocio.ejecutarOrdenes.EjecutorComandos;
import net.miginfocom.swing.MigLayout;

/**
 * @author angel2
 * 
 */
public class VistaEjecucion extends JPanel implements Vista, InterfazAbrirEjecucion,
	InterfazCambiarNombre, InterfazCambiarDirectorioTrabajo {

	/** serial version uid*/
	private static final long	serialVersionUID	= -2388969297453026524L;

	/** Esta es la clase principal, ventana principal del interfaz o gui. */
	private GUI gui;

	/** recursos para dar soporte sobre idiomas */
	static ResourceBundle idiomas = ResourceBundle.getBundle("interfaz.idiomas.Recursos", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	/** recursos de iconos etc... */
	static ResourceBundle recursos = ResourceBundle.getBundle("interfaz.Recursos", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	/** recursos de idiomas para errores */
	static ResourceBundle errores = ResourceBundle.getBundle("interfaz.errores.Errores", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );

	/** dialogo Abrir **/ 
	DialogoAbrir					dialogoAbrir		= new DialogoAbrir( );

	/** barra de tareas específica de esta vista.	 */
	JToolBar barraDeTareas = new JToolBar();
	
	/************* PANEL Y GESTION PANEL ***********/
	/** Panel general de la vista compilación */
	// private JPanel panel;
	/** Panel de pestañas inferior, donde se muestra la salida de la compilación */
	/**
	 * @param gui
	 */
	public VistaEjecucion ( GUI gui ) {
		super( new MigLayout( "fill" ) );
		this.gui = gui;
		crearPanel( );
		crearBarraTareas( );
	}

	private JTabbedPane	pestañas	= new JTabbedPane( );
	{
		ChangeListener cambioPestaña = new ChangeListener () {

			@Override
			public void stateChanged ( ChangeEvent arg0 ) {
				activarActions(visible);
			}
			
		};
		pestañas.addChangeListener( cambioPestaña );
	}

	/*
	 * public JPanel getPanel () { return panel; }
	 */

	protected void crearPanel ( ) {
		/*Aquí hay que hacer el panel*/
		setOpaque(true);

		/* Se añade las pestañas. */
		add(pestañas, "grow");
		
		/* A continuación, se crea un panel inicial vacío. */
		pestañas.addTab( "" , new JPanelEjecucion(gui, null) );
	}
	
	protected void añadirPestaña ( ModeloEjecucion mo ) {
		/** nombre futura pestaña */
		String nombrePrograma = mo.getNombreResultadosEjecucion( ) == null ? idiomas
				.getString( "nombreEstandarEjecucion" )
				: mo.getNombreResultadosEjecucion( );

		Icon icon = (Icon) recursos.getObject( "icn_Ejecutando" );
		
		JPanelEjecucion pestañaNueva = new JPanelEjecucion( gui , mo );
		
		/** Crear la pestaña correctamente */
		pestañas.addTab(nombrePrograma, pestañaNueva);
		int index = pestañas.indexOfComponent(pestañaNueva);
		
		javax.swing.JLabel tabRenderer = new javax.swing.JLabel ( nombrePrograma );
		tabRenderer.setIcon(icon);
		tabRenderer.setIconTextGap(5);
		tabRenderer.setHorizontalTextPosition(SwingConstants.RIGHT);
		//tabRenderer.setToolTipText(camino);
				
		pestañas.setTabComponentAt(index, tabRenderer);
		pestañas.setToolTipTextAt(index, nombrePrograma);
		
		pestañas.setSelectedComponent(pestañaNueva);
	}

	protected void crearBarraTareas ( ) {
		JButton botAbrirEjecucion = new JButton( );
		{
			botAbrirEjecucion.setHideActionText( true );
			botAbrirEjecucion.setAction( CrearYGestionarAcciones.getInstance().getAccionAbrirEjecucion( ) );
		}
		JButton botEjecutar = new JButton( );
		{
			botEjecutar.setHideActionText( true );
			botEjecutar.setAction( CrearYGestionarAcciones.getInstance().getAccionEjecutar( ) );
		}
		JButton botCerrar = new JButton( );
		{
			botCerrar.setHideActionText( true );
			botCerrar.setAction( CrearYGestionarAcciones.getInstance().getAccionCerrar( ) );
		}
		JButton botPasarAGprof = new JButton( );
		{
			botPasarAGprof.setHideActionText( true );
			botPasarAGprof.setAction( CrearYGestionarAcciones.getInstance().getAccionUsarGprofConEjecucion( ) );
		}
		JButton botNombre = new JButton( );
		{
			botNombre.setHideActionText( true );
			botNombre.setAction( CrearYGestionarAcciones.getInstance().getAccionNombreGmonOut( ) );
		}
		{
			barraDeTareas.setAutoscrolls( true );
			barraDeTareas.setFloatable( false );
			barraDeTareas.add( botAbrirEjecucion );
			barraDeTareas.add( botCerrar );
			barraDeTareas.addSeparator( );
			barraDeTareas.add( botNombre );
			barraDeTareas.add( botEjecutar );
			barraDeTareas.addSeparator( );
			barraDeTareas.add( botPasarAGprof );
		}
	}

	/**
	 * @return
	 */
	public JToolBar getBarraDeTareas ( ) {
		return barraDeTareas;
	}

	/**************** Métodos ********************/

	public void abrirPrograma ( ModeloEjecucion me ) {
		if ( me != null ) {

			if ( pestañas.getTabCount( ) == 1
					&& pestañas.getComponentAt( 0 ) instanceof JPanelEjecucion
					&& ( (JPanelEjecucion) pestañas.getComponentAt( 0 ) )
							.isVacio( ) ) {
				pestañas.removeAll( );
				añadirPestaña( me );
				
			} else {
				añadirPestaña( me );
			}
		}
	}
	

	public void abrirPrograma ( ) {
		File [] ficheros;
		// lanzar diálogo para elegir el fichero fuente
		// quizás mejor en el event dispaching thread...

		/** Mostrar el dialogo de abrir (DialogoAbrir). */
		if ( dialogoAbrir.showOpenDialogEjecucion( guigprof.getInstance( ).getGUI( ), true ) == JFileChooser.APPROVE_OPTION ) {
			// *podría hacerlo para múltiples ficheros seleccionados...
			ficheros = dialogoAbrir.getSelectedFiles( );

			/**
			 * Comprobar fichero seleccionado y abrir una nueva pestaña.
			 */
			for ( int i = 0 ; i < ficheros.length ; i++ ) {

				/**
				 * Comprobar fichero seleccionado y abrir una nueva
				 * pestaña.
				 */
				if ( ficheros[i].isFile( ) && ficheros[i].canExecute( ) ) {
					ModeloEjecucion me = new ModeloEjecucion();
					me.setPrograma( ficheros[i] );
					me.setNombreResultadosEjecucion( me.getPrograma( ).getName( ) );
					//esto quizás mejor con un try y el error de abajo...
					me.setDirectorioTrabajo( ficheros[i].getParentFile( ) );
					abrirPrograma( me );
				} else {
					DialogoError.getInstance( ).showErrorDialog(
							errores.getString("err_AbrirPrograma") );
				}
			}
		}
	}

	public void abrirPrograma ( ModeloCompilacion mo ) {
		if ( mo != null) {
			ModeloEjecucion me = new ModeloEjecucion();

			me.setDirectorioTrabajo( mo.getDirectorioTrabajo( ) );
			me.setPrograma( new File( mo.getDirectorioTrabajo( ), mo.getNombrePrograma( ) ) );
			me.setNombreResultadosEjecucion( me.getPrograma( ).getName( ) );
			abrirPrograma( me );
		}
	}

	/**
	 * Ejecutar un fichero ejecutable
	 * */
	public void ejecutarPrograma ( ) {

		JPanelEjecucion pe = this.getPanelActivo( );
		if ( pe != null ) {
			ModeloEjecucion me = this.getModeloEjecucionActivo( );
			if ( me != null ) {
				final ArrayList <String> argumentos = me.getListaArgumentos( );
				final ArrayList <String> input = me.getListaEntradas( );
				final File dirTrabajo = me.getDirectorioTrabajo( );
				final File programa = me.getPrograma( );
				
				try {
					/** ejecutar el programa*/
					final Thread ejecutor = new EjecutorComandos( programa, 
							argumentos.toArray( new String [argumentos.size( )] ), 
							input.toArray( new String [argumentos.size( )] ), dirTrabajo, pe );
					
					ejecutor.start( );
					
					/** hilo para actualizar botones cuando acabe la ejecución del programa */
					new Thread() {
						public void run () {
							try {
								ejecutor.join();
								
								activarActions(visible);
								
							} catch ( InterruptedException e ) {
								DialogoError.getInstance( ).showErrorDialog( 
										errores.getString("err_errorEjecucion")  );
								e.printStackTrace( );
							}
							
						}
					}.start( );
				} catch ( Exception e ) {

					DialogoError.getInstance( ).showErrorDialog( 
							errores.getString("err_errorEjecucion") );
				}
			}
											
		}
	}	

	public void cerrarPestañaActiva ( ) {
		//		Programar un trabajo para el hilo de despacho
		final int indicePestañaActiva = pestañas.getSelectedIndex();
		if (indicePestañaActiva != -1) {
			//usar el propiedades para estos comentarios...
			String cuestión= new String(idiomas.getString("eti_ejecucion_seguroCerrar"));
			String título= new String(idiomas.getString("eti_cerrar"));
			int está_usted_seguro= JOptionPane.showConfirmDialog(gui,
					cuestión,
					título,
					JOptionPane.YES_NO_OPTION);

			if(está_usted_seguro==JOptionPane.YES_OPTION){
				if( pestañas.getTabCount( ) > 1 ) {
					pestañas.removeTabAt(indicePestañaActiva);
				} else if( pestañas.getTabCount() == 1 ) {
					/*Component c = pestañas.getComponentAt( 0 );
					pestañas.setTitleAt( 0 , "" );
					pestañas.setIconAt( 0 , null );
					pestañas.setToolTipTextAt( 0 , null );
					if( c instanceof JPanelEjecucion ) {
						( (JPanelEjecucion) c).setModel( null );
						System.out.println("intentando cerrar");
					}*/
					pestañas.removeAll( );
					pestañas.add( new JPanelEjecucion ( gui ) );
				}
			}
		}
	}
	
	public void cerrarTodo ( ) {
		pestañas.removeAll( );
	}
	
	public void añadirArgumentos ( ) {
		ModeloEjecucion me = this.getModeloEjecucionActivo( );
		if ( me != null ) {
				DialogoNuevoArgumento dialogoAbrir = new DialogoNuevoArgumento();
		
				// elegir entrada
				String argumento = null;
				//lanzar diálogo para elegir entrada
				/** Mostrar el dialogo de abrir (DialogoNuevaEntrada). */
				if ( dialogoAbrir.showDialog( gui ) == DialogoNuevoArgumento.OK_OPTION ) {
					argumento = dialogoAbrir.getArgumento( );

					// añadir entrada
						me.addArgumento( argumento );
				}
		}				
	}

	public void quitarArgumentos ( ) {
		JPanelEjecucion pc = getPanelActivo();
		if ( pc != null ) {
			Object[] seleccionados = pc.getArgumentosSeleccionados( );
			if( seleccionados != null ) {
				for ( int i = 0 ; i < seleccionados.length ; i++ )
					pc.getModel( ).removeArgumento( seleccionados[i]);
			}
		}
	}

	public void quitarEntradas() {
		JPanelEjecucion pc = getPanelActivo();
		if ( pc != null ) {
			Object[] seleccionados = pc.getEntradasSeleccionadas( );
			if( seleccionados != null ) {
				for ( int i = 0 ; i < seleccionados.length ; i++ )
					pc.getModel( ).removeEntrada( seleccionados[i] );
			}
		}
	}
	
	public void añadirEntradas() {
		ModeloEjecucion me = this.getModeloEjecucionActivo( );
		if ( me != null ) {
				DialogoNuevaEntrada dialogo = new DialogoNuevaEntrada();
		
				// elegir entrada
				String entrada = null;
				//lanzar diálogo para elegir entrada
				/** Mostrar el dialogo de abrir (DialogoNuevaEntrada). */
				if ( dialogo.showDialog( gui ) == DialogoNuevaEntrada.OK_OPTION ) {
					entrada = dialogo.getEntrada( );

					// añadir entrada
					me.addEntrada( entrada );
				}
		}		
	}
	
	public void añadirEntradasFichero ( File f ) {
		ModeloEjecucion me = this.getModeloEjecucionActivo( );
		if ( me != null ) {
				String s = new UtilidadesFichero().leerFichero( f );
		
				String [] entradas = s.split( "\n" );
				for (int i = 0 ; i < entradas.length; i++ ) {
					me.getEntradasListModel( ).addElement( entradas[i] );
				}
				
		}		

		
	}

	public void añadirEntradasFichero ( ) {
		/** Mostrar el dialogo de abrir (DialogoAbrir). */
		if ( dialogoAbrir.showOpenDialogTODO( guigprof.getInstance( ).getGUI( ) , false ) == JFileChooser.APPROVE_OPTION ) {
			File fichero = dialogoAbrir.getSelectedFile( );

			if ( fichero != null && fichero.exists( ) && fichero.isFile( ) && fichero.canRead( )) {
				añadirEntradasFichero( fichero );
			}
		}
	}
	
	/**
	 * Método para cambiar el directorio de trabajo en la pestaña activa.
	 */
	public void cambiarDirectorioTrabajo ( ) {
		ModeloEjecucion me = this.getModeloEjecucionActivo( );
		if ( me != null ) {
			JDirectoryChooser dialogo = new JDirectoryChooser( new File( "/" ) );
			File directorio_inicial = me.getDirectorioTrabajo( );
			if ( directorio_inicial != null ) {
				dialogo.setSelectedDirectory( directorio_inicial );
			}
			if ( dialogo.showDialog( gui ) == JDirectoryChooser.OK_OPTION ) {
				File directorio_final = dialogo.getSelectedDirectory( );
				if ( directorio_final != null && directorio_final.isDirectory( ) ) {
					me.setDirectorioTrabajo( dialogo.getSelectedDirectory( ) );
				}
			}

		}
	}
	
	/**
	 * Método para editar el nombre del fichero gmon
	 */
	public void cambiarNombre ( ) {
		/** editarNombreGmon */
		JPanelEjecucion pe = getPanelActivo();
		if ( pe != null ) {
				ModeloEjecucion me = pe.getModel( );

				DialogoCambiarNombre dialogo = new DialogoCambiarNombre( me.getNombreResultadosEjecucion( ) );
				if( dialogo.showDialog( gui ) == DialogoCambiarNombre.OK_OPTION ) {
					String nombre = dialogo.getNombre();
					me.setNombreResultadosEjecucion( nombre );
					
					int index = pestañas.indexOfComponent( pe );
					
					pestañas.setTitleAt( index , nombre );
					( (javax.swing.JLabel) pestañas.getTabComponentAt( index ) ).setText(nombre);
					pestañas.setToolTipText( nombre );
				}
		}	
	}

/*	public void cambiarFuente ( Font f ) {
		for (int i = 0; i < pestañas.getTabCount( ); i ++ ) {
			Component c = pestañas.getComponentAt( i );
			if( c instanceof JPanelEjecucion ) {
				( ( JPanelEjecucion ) c  ).cambiarFuente( f );
			}
		}
	}*/
	
	/******************* pestañas ***********************/
	
	public Vector < ModeloEjecucion > getModelos () {
		Vector < ModeloEjecucion > vModelos = new Vector < ModeloEjecucion >();
		
		for (int i = 0; i < pestañas.getTabCount( ); i ++ ) {
			Component c = pestañas.getComponentAt( i );
			if( c instanceof JPanelEjecucion ) {
				ModeloEjecucion moJ = ( ( JPanelEjecucion ) c ).getModel( );
				if ( moJ != null )
					vModelos.add( moJ );
			}
		}
		
		return vModelos;
	}
	
	/** devuelve el modelo compilación del panel activo. Null si no hay ninguno; */
	public ModeloEjecucion getModeloEjecucionActivo() {
		ModeloEjecucion me = null;
		JPanelEjecucion pe = getPanelActivo();
		if ( pe != null ) {
			me = pe.getModel( );
		}
		return me;
	}
	
	/** devuelve el panel activo. Null si no hay ninguno; */
	private JPanelEjecucion getPanelActivo () {
		JPanelEjecucion pc = null;
		if ( pestañas.getComponentCount() > 0 ) {
			Component c = pestañas.getSelectedComponent();
			if ( c instanceof JPanelEjecucion ) {
				pc = (JPanelEjecucion) c;
			}
		}
		return pc;
	}

	/**************** Visibilidad ********************/
	boolean	visible	= true;

	public void setActivo ( boolean b ) {
		visible = b;
		setVisible( b );
		// también hay que anular las acciones que no correspondan.
		activarActions ( b );
		barraDeTareas.setEnabled( b );
		barraDeTareas.setVisible( b );
	}

	public void activarActions ( boolean vistaActiva ) {
		boolean panelNoVacio = false;
		boolean siguientePaso = false;
		if ( vistaActiva && this.getPanelActivo( ) != null && !this.getPanelActivo( )
				.isVacio( ) ) {
			panelNoVacio = true;
		}
		if ( panelNoVacio == true && this.getPanelActivo( ).getModel( ).isEjecuted( ) ) {
			siguientePaso = true;
		}
		
		CrearYGestionarAcciones a = CrearYGestionarAcciones.getInstance( );
		
		a.activarActionsEjecucion( vistaActiva, panelNoVacio , siguientePaso );

	}
	
 	public boolean isActivo ( ) {
		return visible;
	}


}
