package interfaz.vestadisticas;


import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.font.FontRenderContext;
import java.awt.font.LineBreakMeasurer;
import java.awt.font.TextAttribute;
import java.awt.font.TextLayout;



import java.text.AttributedCharacterIterator;
import java.text.AttributedString;
import java.util.HashMap;

import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.JTable;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import negocio.PreferenciasSingleton;



public class ColumnaTextoMultilineaTablaCellRenderer extends Object implements
		TableCellRenderer {

	/** serial uid version */
	private static final long			serialVersionUID		= 5847964157032708753L;

	private int							ancho					= 0;
	int									altura_requiere_la_fila	= 0;

	private HashMap < Integer , Celda >	celdas					= new HashMap < Integer , Celda >( );

	@Override
	public Component getTableCellRendererComponent ( JTable tabla ,
			Object valor , boolean seleccionado , boolean tieneElFoco ,
			int fila , int columna ) {

		Integer ifila = new Integer( fila );

		String texto = ( valor == null ) ? "" : valor.toString( );
		Celda x = new Celda ( );

		if ( celdas.containsKey( ifila ) ) {
			x = celdas.get( ifila );
			x.setText( texto );
		} else {
			x = new Celda( texto );
			celdas.put( ifila , x );
		}

		x.setFont( PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getEstadisticasFont( ) );

		x.setText( texto );
		
		Component c = new DefaultTableCellRenderer( )
		.getTableCellRendererComponent( tabla , valor ,
				seleccionado , tieneElFoco , fila , columna );
		
		x.setForeground( c.getForeground( ) );
		x.setBackground( c.getBackground( ) );
		
		if ( tieneElFoco ) 
			x.setBorder( UIManager
					.getBorder( "Table.focusCellHighlightBorder" ) );
		else
			x.setBorder( UIManager.getBorder( "Table.cellBorder" ) );


		ancho = tabla.getColumnModel( ).getColumn( columna ).getWidth( );

		x.setAncho( ancho );

		altura_requiere_la_fila = x.getAltoMultiLinea( );

		if ( altura_requiere_la_fila != tabla.getRowHeight( fila )
				&& altura_requiere_la_fila >= tabla.getRowHeight( ) )
			tabla.setRowHeight( fila , altura_requiere_la_fila );
		
		tabla.revalidate( );
		
		return x;

	}
}

class Celda extends JPanel {
	/** serial uid version */
	private static final long	serialVersionUID	= 5847964157032708753L;

	// el ancho mínimo es 20. Y el alto 10.
	static private int			ANCHO_MIN				= 20;
	static private int			ALTO_MIN				= 10;
	private int					ancho					= ANCHO_MIN;
	int							altoMultiLinea			= 1;
	String						texto					= new String( );
	
	public Celda () {
		super( new BorderLayout () );
	}
	
	public Celda ( String s ) {
		super( new BorderLayout () );
		this.setText( s );
	}
	
	public void setText(String s ) {
		texto = s;
		
		repaint();
		invalidate( );
		revalidate( );
	}
	
	public String getText() {
		return texto;
	}
	
	public void setAncho(int a) {
		if(a < ANCHO_MIN ) {
			ancho = ANCHO_MIN;
		}
		else {
			ancho = a;
		}
		
		repaint();
		invalidate( );
		revalidate( );
	}
	
	public int getAltoMultiLinea( ) {
		return altoMultiLinea;
	}
	
	@Override
	public void paint ( Graphics g ) {

		super.paint( g );
		
		if ( texto.length( ) > 0 ) {
			AttributedString attributedString = new AttributedString( texto );
			attributedString
					.addAttribute( TextAttribute.FONT , this.getFont( ) );

			attributedString.addAttribute( TextAttribute.FOREGROUND , this
					.getForeground( ) );
			attributedString.addAttribute( TextAttribute.BACKGROUND , 
					new Color(0,0,0,0) );

			Graphics2D g2d = (Graphics2D) g;

			int width = ancho - ANCHO_MIN; 
			int x = ANCHO_MIN / 2;
			int y = ALTO_MIN / 2;

			AttributedCharacterIterator characterIterator = attributedString
					.getIterator( );
			FontRenderContext fontRenderContext = g2d.getFontRenderContext( );
			LineBreakMeasurer measurer = new LineBreakMeasurer(
					characterIterator , fontRenderContext );

			while ( measurer.getPosition( ) < characterIterator.getEndIndex( ) ) {
				TextLayout textLayout = measurer.nextLayout( width );
				y += textLayout.getAscent( );
				textLayout.draw( g2d , x , y );
				y += textLayout.getDescent( ) + textLayout.getLeading( );
			}
			altoMultiLinea = y + ( ALTO_MIN / 2 );

			this.setPreferredSize( new Dimension(ancho, altoMultiLinea) );
		}
	}
	
	@Override
	public void setBackground( Color c ) {
		if ( c.getRGB( ) == -1 ) 
			this.setOpaque( false );
		else 
			this.setOpaque( true );

		super.setBackground( c );
	}
}

