package interfaz.vestadisticas;

import java.awt.event.ActionEvent;
import java.io.File;
import java.util.ResourceBundle;

import interfaz.dialogos.error.DialogoError;
import interfaz.vestadisticas.fuenteanotado.JPanelFuenteAnotado;
import interfaz.vestadisticas.grafollamadas.JPanelGrafoLlamadas;
import interfaz.vestadisticas.mostrarxml.JPanelXML;
import interfaz.vestadisticas.perfilplano.JPanelPerfilPlano;
import interfaz.vestadisticas.salidagprof.JPanelSalidaGprof;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import modelo.ficheros.UtilidadesFichero;
import modelo.vestadisticas.ModeloEstadistica;
import negocio.ejecutarOrdenes.InterfazOrdenes;
import net.miginfocom.swing.MigLayout;

public class JPanelEstadisticas extends JPanel implements ChangeListener, InterfazOrdenes {

	/** Serial version uid */
	private static final long serialVersionUID = 1333750981026880497L;

	/** recursos para dar soporte sobre idiomas	 */
	ResourceBundle idiomas = ResourceBundle.getBundle("interfaz.idiomas.Recursos");
	/** recursos de iconos etc...  */
	ResourceBundle recursos = ResourceBundle.getBundle("interfaz.Recursos");
	/** recursos de idioma para errores. */
	ResourceBundle errores = ResourceBundle.getBundle("interfaz.errores.Errores");


/************************PROPIEDADES***********************/	

	/**
	 * Etiqueta para el fichero.
	 */
	final protected JLabel etiFicheroXML = new JLabel(idiomas.getString( "eti_ficheroXML" ));
	
	/**
	 * Etiqueta que muestra el path del fichero de estadísticas fuente que se está mostrando.
	 * @see textArea
	 */
	protected JTextField txtNombreFicheroXML = new JTextField ("");
	{
		txtNombreFicheroXML.setEditable( false );
	}

	/**
	 * ComboBox que sirve para elegir la vista de las estadísticas.
	 */
	protected JComboBox lstElegirVista = new JComboBox();
	
	protected DefaultComboBoxModel modElegir = new DefaultComboBoxModel();
	
	//ActionListener del comboBox:
	private AbstractAction accionCombo = new AbstractAction () {
		static final long serialVersionUID=1;
		public void actionPerformed(ActionEvent e){
			String elegido=((String)((JComboBox)e.getSource()).getSelectedItem());
			if ( elegido.equals( PERFILPLANO ) ) {
				verPerfilPlano ( );
			} else if ( elegido.equals( GRAFOLLAMADAS ) ) {
				verGrafoLlamadas( );
			} else if ( elegido.equals( FUENTEANOTADO ) ) {
				verFuenteAnotado( );
			} else if ( elegido.equals( SALIDA ) ) {
				verSalida( );
/*			} else if ( elegido.equals( XML ) ) {
				setPanelVisible ( panelxml );*/
			} else 
				DialogoError.getInstance( ).showErrorDialog(
						errores.getString( "err_no_vista_disponible" ) );
		}
	};
	
	{
		lstElegirVista.addActionListener( accionCombo );
		lstElegirVista.setModel( modElegir );
	}
	
	/**
	 * Etiqueta con el comentario para elegir la vista 
	 */
	final protected JLabel etiElegirVista = new JLabel( idiomas.getString( "eti_eligeVista" ) );
	
	final protected String PERFILPLANO = idiomas.getString( "eti_PerfilPlano" );
	final protected String GRAFOLLAMADAS = idiomas.getString( "eti_GrafoLlamadas" );
	final protected String FUENTEANOTADO = idiomas.getString( "eti_FuenteAnotado" );
	final protected String SALIDA = idiomas.getString( "eti_VerVistaSalida" );
	
	private JPanelPerfilPlano panelperfilplano = new JPanelPerfilPlano();
	{
		panelperfilplano.setVisible(false);
	}
	
	private JPanelGrafoLlamadas panelgrafollamadas = new JPanelGrafoLlamadas();
	{
		panelgrafollamadas.setVisible(false);
	}
	
	private JPanelFuenteAnotado panelfuenteanotado = new JPanelFuenteAnotado();
	{
		panelfuenteanotado.setVisible(false);
	}
	
	private JPanelSalidaGprof panelsalida = new JPanelSalidaGprof();
	{
		panelsalida.setVisible(false);
	}
	
	private JPanelXML panelxml = new JPanelXML();
	{
		panelxml.setVisible(false);
	}
	
	

	/************************ CONTRUCTOR ***********************/
	
	private ModeloEstadistica moEstadistica = null;
	
	public JPanelEstadisticas () {
		super (new MigLayout ( "fill, hidemode 3" ) );
		
		setModel( null );
		crearComponentes();
	}
	
	public JPanelEstadisticas ( ModeloEstadistica me ) {
		super( new MigLayout( "fill, hidemode 3" ) );

		setModel( me );
		crearComponentes();
		ponerVisibleUnPanel();
	}	
	
	/************************ METODOS ***********************/
	
	public boolean isVacio () {
		return this.moEstadistica == null;
	}
	
	
	protected void ponerVisibleUnPanel() {
		
	}
	
	protected void setModel ( ModeloEstadistica mo ) {
		this.moEstadistica = mo;
		
		if( this.moEstadistica != null ) {
			panelperfilplano.setDatosXML( this.moEstadistica );
			panelgrafollamadas.setDatosXML( this.moEstadistica  );
			panelfuenteanotado.setDatosXML( this.moEstadistica  );
			panelsalida.setDatosXML( this.moEstadistica  );
			panelxml.setDatosXML( this.moEstadistica  );
			
			this.moEstadistica.addChangeListener( this );
		}
		actualizarModel();
		
	}
	
	public ModeloEstadistica getModel () {
		return this.moEstadistica;
	}

	protected void crearComponentes () {
		setBorder( new EtchedBorder() );
		setOpaque( true );
		
		
		JPanel pnlEstadisticas;
		{
			pnlEstadisticas = new JPanel ( new MigLayout() );
			pnlEstadisticas.add( etiFicheroXML , "sgy Cmd");
			pnlEstadisticas.add( txtNombreFicheroXML , "growx,pushx,wmin 100pt, sgy Cmd, wrap" );
			pnlEstadisticas.add( etiElegirVista , "sgy Cmd");
			pnlEstadisticas.add( lstElegirVista , "growx,pushx,wmin 100pt, sgy Cmd, wrap" );
			pnlEstadisticas.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
		}
		this.add( pnlEstadisticas , "growx, pushx, sizegroupy a" );
		
		/** agregar:*/
		add( pnlEstadisticas,"wrap,grow,pushx" );
		add( panelperfilplano,"wrap,grow,push" );
		add( panelgrafollamadas,"wrap,grow,push" );
		add( panelfuenteanotado,"wrap,grow,push" );
		add( panelsalida,"wrap,grow,push" );
		add( panelxml,"wrap,grow,push" );
		
	}	
	

	/**
	 * función para poner visible este panel panel (p).
	 * @param p
	 */
	protected void setPanelVisible ( JPanel p ) {
		panelxml.setVisible(false);
		panelsalida.setVisible(false);
		panelperfilplano.setVisible(false);
		panelgrafollamadas.setVisible(false);
		panelfuenteanotado.setVisible(false);
		p.setVisible(true);
	}
	
	@Override
	public void stateChanged ( ChangeEvent e ) {
		if ( e.getSource( ) instanceof ModeloEstadistica ) {
			this.actualizarModel( );
		}
		
	}
	
	private void actualizarModel ( ) {

		if ( this.moEstadistica != null ) {
			if ( this.moEstadistica.hasFicheroXML( ) ) {
				/* txtNombre */
				this.txtNombreFicheroXML.setText( new UtilidadesFichero( )
						.caminoFichero( moEstadistica.getFicheroXML( ) ) );

				/* cmbElegir */
				modElegir.removeAllElements( );
				
				if ( moEstadistica.hasPerfilPlano( ) ) {
					modElegir.addElement( PERFILPLANO );
				}
				if ( moEstadistica.hasGrafoLlamadas( ) ) {
					modElegir.addElement( GRAFOLLAMADAS );
				}
				if ( moEstadistica.hasFuenteAnotado( ) ) {
					modElegir.addElement( FUENTEANOTADO );
				}
				if ( moEstadistica.hasSalidaGprof( ) ) {
					modElegir.addElement( SALIDA );
				}
			}
		} else {
			/* txtNombre */
			this.txtNombreFicheroXML.setText( "" );

			/* cmbElegir */
			modElegir.removeAllElements( );
		}
	}

	public void verPerfilPlano ( ) {
		setPanelVisible ( panelperfilplano );		
	}

	public void verGrafoLlamadas ( ) {
		setPanelVisible ( panelgrafollamadas );
		
	}

	public void verFuenteAnotado ( ) {
		setPanelVisible ( panelfuenteanotado );
	}
	
	public void verSalida ( ) {
		setPanelVisible ( panelsalida );
	}

	@Override
	public void empezarOrden ( String comando ) {
		//Nada
	}

	@Override
	public void ordenErrorIndeterminado( String comando, File directorio, Exception e ){
		String saltoLinea = "\n";
		String descripcion;

		descripcion = errores.getString( "err_errorEjecuciónOrden" )
				+ saltoLinea + errores.getString( "err_errorEjecuciónOrden_orden" )
				+ saltoLinea + comando
				+ saltoLinea + errores.getString( "err_errorEjecución_directorio" )
				+ saltoLinea + directorio.getName( )
				+ saltoLinea
				+ saltoLinea + errores.getString( "err_detalles:" )
				+ saltoLinea + e.getMessage( );
		
		DialogoError.getInstance( ).showErrorDialog( descripcion );
	}
	
	@Override
	public void terminarOrden ( int resultado , String estandar , String errores ) {
		//Nada
	}

}
