/**
 * 
 */
package interfaz.vestadisticas;

import java.awt.Component;
import java.awt.Frame;
import java.io.File;
import java.io.IOException;
import java.util.ResourceBundle;
import java.util.Vector;

import interfaz.Vista;
import interfaz.acciones.CrearYGestionarAcciones;
import interfaz.acciones.InterfazAbrirEstadisticasXML;
import interfaz.dialogos.dialogoAbrir.DialogoAbrir;
import interfaz.dialogos.error.DialogoError;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import datos.GprofXMLParser;


import modelo.modeloXML.ModeloXML;
import modelo.vestadisticas.ModeloEstadistica;
import modelo.vtraduccion.ModeloTR;
import negocio.PreferenciasSingleton;
import negocio.guigprof;
import negocio.ejecutarOrdenes.EjecutorComandos;
import net.miginfocom.swing.MigLayout;

/**
 * @author angel2
 *
 */
public class VistaEstadisticas extends JPanel implements Vista,
		InterfazAbrirEstadisticasXML {

	/** serial version uid*/
	private static final long serialVersionUID = 759911159280491446L;
	
	/** Esta es la clase principal, ventana principal del interfaz o gui. */
	private Frame gui;

	/** recursos para dar soporte sobre idiomas */
	static ResourceBundle idiomas = ResourceBundle.getBundle("interfaz.idiomas.Recursos", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	/** recursos de iconos etc... */
	static ResourceBundle recursos = ResourceBundle.getBundle("interfaz.Recursos", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	/** recursos de idiomas para errores */
	static ResourceBundle errores = ResourceBundle.getBundle("interfaz.errores.Errores", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	
	/** dialogo Abrir **/ 
	DialogoAbrir					dialogoAbrir		= new DialogoAbrir( );
	
	/** barra de tareas específica de esta vista. */
	JToolBar barraDeTareas = new JToolBar();
	
	/**********Constructor PANEL Y GESTION PANEL***************/
	
	/**
	 * @param guIguigprof
	 */
	public VistaEstadisticas ( Frame gui) {
		super(new MigLayout("fill"));
		this.gui = gui;
		crearPanel();
		crearBarraTareas();
	}

	/** Panel general de la vista compilación */
	
//	private JPanel panel;
	
	/** Panel de pestañas inferior, donde se muestra la salida de la compilación */
	private JTabbedPane pestañas = new JTabbedPane ();
	{
		ChangeListener cambioPestaña = new ChangeListener () {

			@Override
			public void stateChanged ( ChangeEvent arg0 ) {
				activarActions(visible);
			}
			
		};
		pestañas.addChangeListener( cambioPestaña );
	}

	protected void crearBarraTareas() {
		JButton botAbrirXML = new JButton ();
		{
			botAbrirXML.setHideActionText( true );
			botAbrirXML.setAction( CrearYGestionarAcciones.getInstance().getAccionAbrirEstadisticasXML() );
		}
		JButton botCerrar = new JButton ();
		{
			botCerrar.setHideActionText(true);
			botCerrar.setAction(CrearYGestionarAcciones.getInstance().getAccionCerrar());
		}
		JButton botXML = new JButton ();
		{
			botXML.setHideActionText(true);
			botXML.setAction(CrearYGestionarAcciones.getInstance().getAccionEditarXML( ) );
		}
		{
			barraDeTareas.setAutoscrolls( true );
			barraDeTareas.setFloatable( false );
			barraDeTareas.add( botAbrirXML );
			barraDeTareas.add( botCerrar );
			barraDeTareas.addSeparator();
			barraDeTareas.add( botXML );
		}
	}

	/**
	 * @return
	 */
	public JToolBar getBarraDeTareas() {
		return barraDeTareas;
	}
	
	protected void crearPanel () {
		/** Aquí hay que hacer el panel*/
		setOpaque(true);

		/** Se añade las pestañas. */
		add(pestañas, "grow");
		
		/* A continuación, se crea un panel inicial vacío. */
		pestañas.addTab( "" , new JPanelEstadisticas() );		
	}
	
	protected void añadirPestaña ( ModeloEstadistica me ) {
		/** nombre futura pestaña */
		File f = me.getFicheroXML();
		
		String nombreFichero = f.getName( );

		Icon icon = (Icon) recursos.getObject( "icn_Estadisticas" );
		
		JPanelEstadisticas pestañaNueva = new JPanelEstadisticas(
				me );

		/** Crear la pestaña correctamente */
		pestañas.addTab(nombreFichero, pestañaNueva);
		int index = pestañas.indexOfComponent(pestañaNueva);
		
		javax.swing.JLabel tabRenderer = new javax.swing.JLabel ( nombreFichero );
		tabRenderer.setIcon(icon);
		tabRenderer.setIconTextGap(5);
		tabRenderer.setHorizontalTextPosition(SwingConstants.RIGHT);
		//tabRenderer.setToolTipText(camino);
				
		pestañas.setTabComponentAt(index, tabRenderer);
		pestañas.setToolTipTextAt(index, nombreFichero);
		
		pestañas.setSelectedComponent(pestañaNueva);
	}
	
	/*********** métodos *************/

	/** Se usa para recoger el fichero de la pestaña activa
	 * 
	 * @return
	 */
	public void editarXML ( ) {

		JPanelEstadisticas pe = getPanelActivo( );
		if ( pe != null ) {
			ModeloEstadistica me = pe.getModel( );
			if ( me != null ) {
				final String [] orden = new String [] { PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getEditorXML( ),
						me.getFicheroXML( ).getName( ) };
				final File dirTrabajo = me.getFicheroXML( ).getParentFile( );

				try {
					/** realizar edición*/
					final Thread editor = new EjecutorComandos( orden , dirTrabajo , pe );
					
					editor.start( );
					
					/** hilo para actualizar botones cuando acabe la ejecución de edición */
					new Thread() {
						public void run () {
							try {
								editor.join();
								
								activarActions(visible);
								
							} catch ( InterruptedException e ) {
								DialogoError.getInstance( ).showErrorDialog( 
										errores.getString("err_errorEdicionXML") );
								e.printStackTrace( );
							}
							
						}
					}.start( );
				} catch ( Exception e ) {

					DialogoError.getInstance( ).showErrorDialog( 
							errores.getString("err_errorEdicionXML") );
				}
			}
											
		}
	}
	
	public void abrirEstadisticaXML ( ModeloEstadistica me ) {

		if ( me != null ) {
			if ( me.getFicheroXML( ) != null ) {
				final File f = me.getFicheroXML( );

				if( me.getModeloXML( ) == null ) {
					ModeloXML x;
					try {
						x = new GprofXMLParser( ).leerModeloXML( f );

						me.setModeloXML( x );

					} catch ( NumberFormatException e ) {
						DialogoError.getInstance( ).showErrorDialog( 
								errores.getString( "err_errorLecturaXML" ) );
						e.printStackTrace( );
					} catch ( XPathExpressionException e ) {
						DialogoError.getInstance( ).showErrorDialog(  
								errores.getString( "err_errorLecturaXML" ) );
						e.printStackTrace( );
					} catch ( ParserConfigurationException e ) {
						DialogoError.getInstance( ).showErrorDialog( 
								errores.getString( "err_errorLecturaXML" ) );
						e.printStackTrace( );
					} catch ( SAXException e ) {
						DialogoError.getInstance( ).showErrorDialog( 
								errores.getString( "err_errorLecturaXML" ) );
						e.printStackTrace( );
					} catch ( IOException e ) {
						DialogoError.getInstance( ).showErrorDialog( 
								errores.getString( "err_errorLecturaXML" ) );
						e.printStackTrace( );
					}
				}

				if ( pestañas.getTabCount( ) == 1
						&& pestañas.getComponentAt( 0 ) instanceof JPanelEstadisticas
						&& ( (JPanelEstadisticas) pestañas.getComponentAt( 0 ) )
								.isVacio( ) ) {
					pestañas.removeAll();
					añadirPestaña( me );
					
				} else {
					añadirPestaña( me );
					
				}
			}
		}
	}
	
	public void abrirEstadisticaXML ( ModeloTR mt ) {
		ModeloEstadistica me = new ModeloEstadistica();
		File xml = mt.getFicheroXML( );
		if ( xml != null && xml.exists( ) ) {
			me.setFicheroXML( xml );
		}
		this.abrirEstadisticaXML( me );		
	}

/*	public void cambiarFuente ( Font f ) {
		for (int i = 0; i < pestañas.getTabCount( ); i ++ ) {
			Component c = pestañas.getComponentAt( i );
			if( c instanceof JPanelEstadisticas ) {
				( ( JPanelEstadisticas ) c  ).cambiarFuente( f );
				
			}
		}
	}*/
	
	/******************* pestañas ***********************/
	
	public Vector < ModeloEstadistica > getModelos () {
		Vector < ModeloEstadistica > vModelos = new Vector < ModeloEstadistica >();
		
		for (int i = 0; i < pestañas.getTabCount( ); i ++ ) {
			Component c = pestañas.getComponentAt( i );
			if( c instanceof JPanelEstadisticas ) {
				ModeloEstadistica moS = ( ( JPanelEstadisticas ) c ).getModel( ); 
				if ( moS != null )
					vModelos.add( moS );
			}
		}
		
		return vModelos;
	}
		
	public void cerrarPestañaActiva () {
		/** Programar un trabajo para el hilo de despacho */
		final int indicePestañaActiva = pestañas.getSelectedIndex();
		if (indicePestañaActiva != -1) {
			/** usar el propiedades para estos comentarios... */
			String cuestión= new String(idiomas.getString("eti_estadisticas_seguroCerrar"));
			String título= new String(idiomas.getString("eti_cerrar"));
			int está_usted_seguro= JOptionPane.showConfirmDialog(gui,
					cuestión,
					título,
					JOptionPane.YES_NO_OPTION);

			if(está_usted_seguro==JOptionPane.YES_OPTION){
				if( pestañas.getTabCount( ) > 1 ) {
					pestañas.removeTabAt(indicePestañaActiva);
				} else if( pestañas.getTabCount() == 1 ) {

					pestañas.removeAll( );
					pestañas.add( new JPanelEstadisticas ( ) );
				}

			}
		}
	}	
	


	public void cerrarTodo ( ) {
		pestañas.removeAll( );
	}
	
	/** devuelve el modelo estadistica del panel activo. Null si no hay ninguno; */
	public ModeloEstadistica getModeloEstadisticaActivo() {
		ModeloEstadistica mo = null;
		JPanelEstadisticas pc = getPanelActivo();
		if ( pc != null ) {
			mo = pc.getModel( );
		}
		return mo;
	}
	
	/** devuelve el panel activo. Null si no hay ninguno; */
	private JPanelEstadisticas getPanelActivo () {
		JPanelEstadisticas pe = null;
		if ( pestañas.getComponentCount() > 0 ) {
			Component c = pestañas.getSelectedComponent();
			if ( c instanceof JPanelEstadisticas ) {
				pe = (JPanelEstadisticas) c;
			}
		}
		return pe;
	}
	/****************Visibilidad********************/
	boolean visible = true;
	
	public void setActivo(boolean b){
		visible = b;
		setVisible(b);
		
		/** también hay que anular las acciones que no correspondan. */
		activarActions ( b );
		
		barraDeTareas.setEnabled(b);
		barraDeTareas.setVisible(b);
	}
	public boolean isActivo () {
		return visible;
	}
	
	public void activarActions ( boolean vistaActiva ) {
		boolean panelNoVacio = false;
		if ( vistaActiva && this.getPanelActivo( ) != null && !this.getPanelActivo( )
				.isVacio( ) ) {
			panelNoVacio = true;
		}
		
		ModeloEstadistica me = this.getModeloEstadisticaActivo( );
		
		/** Ahora comprobar si tiene todas las vistas. */
		boolean bPP = false, bGL = false, bFA = false, bSG = false;
		if ( panelNoVacio == true ) {
			if ( me.hasPerfilPlano( ) )
				bPP = true;
			if ( me.hasGrafoLlamadas( ) )
				bGL = true;
			if ( me.hasFuenteAnotado( ) )
				bFA = true;
			if ( me.hasSalidaGprof( ) )
				bSG = true;
		}

		CrearYGestionarAcciones a = CrearYGestionarAcciones.getInstance( );
		
		a.activarActionsEstadisticas( vistaActiva, panelNoVacio , bPP, bGL, bFA, bSG);		
	}

	public void verPerfilPlano ( ) {
		JPanelEstadisticas pe = getPanelActivo();
		
		if ( pe != null ) {
			pe.verPerfilPlano( );
		}
		
	}

	public void verGrafoLlamadas ( ) {
JPanelEstadisticas pe = getPanelActivo();
		
		if ( pe != null ) {
			pe.verGrafoLlamadas( );
		}
	}

	public void verFuenteAnotado ( ) {
		JPanelEstadisticas pe = getPanelActivo();
		
		if ( pe != null ) {
			pe.verFuenteAnotado( );
		}		
	}

	public void verSalidaGprof ( ) {
		JPanelEstadisticas pe = getPanelActivo();
		
		if ( pe != null ) {
			pe.verSalida( );
		}	
	}
	
	@Override
	public void abrirEstadisticasXML ( ) {
		File [] ficheros;
		/** Mostrar el dialogo de abrir (DialogoAbrir). */
		if ( dialogoAbrir.showOpenDialogXML( guigprof.getInstance( ).getGUI( ), true ) == JFileChooser.APPROVE_OPTION ) {
			// *podría hacerlo para múltiples ficheros seleccionados...
			ficheros = dialogoAbrir.getSelectedFiles( );

			for ( int i = 0 ; i < ficheros.length ; i++ ) {

				/**
				 * Comprobar fichero seleccionado y abrir una nueva
				 * pestaña.
				 */
				if ( ficheros[i].isFile( ) && ficheros[i].canRead( ) ) {
					ModeloEstadistica me = new ModeloEstadistica ();
					me.setFicheroXML( ficheros[i] );
					abrirEstadisticaXML( me );
				} else {
					DialogoError.getInstance( ).showErrorDialog(  
							errores.getString("err_FicheroXML") );
				}
			}
		}		
	}


}
