package interfaz.vestadisticas.grafollamadas;

import interfaz.grafico.JGrafico;
import interfaz.vestadisticas.ColumnaTextoMultilineaTablaCellRenderer;
import interfaz.vestadisticas.CabeceraMultilineaTableCellRenderer;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.border.EtchedBorder;
import javax.swing.table.TableColumn;
import javax.swing.tree.DefaultTreeSelectionModel;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Enumeration;
import java.util.ResourceBundle;

import modelo.modeloXML.EntradaFuncion;
import modelo.modeloXML.InterfazModeloXML;
import modelo.vestadisticas.ModeloEstadistica;
import modelo.vestadisticas.grafollamadas.ModeloArbolGrafoLlamadas;
import modelo.vestadisticas.grafollamadas.ModeloGraficoFLlamadas;
import modelo.vestadisticas.grafollamadas.ModeloGraficoFLlamadoras;
import modelo.vestadisticas.grafollamadas.ModeloListaFunciones;
import modelo.vestadisticas.grafollamadas.ModeloTablaFuncionSenalada;
import modelo.vestadisticas.grafollamadas.ModeloTablaFuncionesLlamadas;
import modelo.vestadisticas.grafollamadas.ModeloTablaFuncionesLlamadoras;
import net.miginfocom.swing.MigLayout;

/**
 * Este panel muestra el contenido la parte del Grafo de Llamadas de un
 * ModeloXML- Este contenido se mostrará visualmente más agradablemente que como
 * lo hace gprof.
 * 
 * @author Ángel García Fernández
 * @version 0.1
 * @since 0.0.8
 */
public class JPanelGrafoLlamadas extends JPanel implements InterfazModeloXML, ActionListener {

	/** recursos para dar soporte sobre idiomas */
	ResourceBundle							idiomas				= ResourceBundle
																		.getBundle( "interfaz.idiomas.Recursos" );
	/** recursos de iconos etc... */
	ResourceBundle							recursos			= ResourceBundle
																		.getBundle( "interfaz.Recursos" );

	/************************ PROPIEDADES ***********************/
	/** serial version uid */
	private final static long				serialVersionUID	= 2;
	
	/** Etiqueta fija del panel del Grafo de Llamadas. */
	private final JLabel					etiFseñalada		= new JLabel(
																		idiomas
																				.getString( "eti_Fseñalada" ) );

	/** Etiqueta fija del panel del Grafo de Llamadas. */
	private final JLabel					etiFllamadoras		= new JLabel(
																		idiomas
																				.getString( "eti_Fllamadoras" ) );

	/** Etiqueta fija del panel del Grafo de Llamadas. */
	private final JLabel					etiFllamadas		= new JLabel(
																		idiomas
																				.getString( "eti_Fllamadas" ) );

	/*	*//** Etiqueta fija del panel del Grafo de Llamadas. */
	/*
	 * private final JLabel etiListaFunciones = new JLabel ( idiomas.getString(
	 * "eti_ListaFunciones" ) );
	 */
	/** Etiqueta fija del panel del Grafo de Llamadas. */
	private final JLabel					etiArbol			= new JLabel(
																		idiomas
																				.getString( "eti_Arbol" ) );

	private ModeloTablaFuncionSenalada		modTabFSeñalada = new ModeloTablaFuncionSenalada( );

	/** Datos de la función señalada en el árbol. */
	private JTable							tabFseñalada		= new JTable( );
	{
		tabFseñalada.setModel( modTabFSeñalada );

		// Renderers
		Enumeration < TableColumn > v = tabFseñalada.getColumnModel( )
				.getColumns( );
		while ( v.hasMoreElements( ) )
			v.nextElement( ).setHeaderRenderer( new CabeceraMultilineaTableCellRenderer( ) );
		tabFseñalada.setDefaultRenderer( String.class ,
				new ColumnaTextoMultilineaTablaCellRenderer( ) );
		tabFseñalada.setDefaultRenderer( Integer.class ,
				new TabFLLSenaladaCellRenderer( ) );
		tabFseñalada.setDefaultRenderer( Double.class ,
				new TabFLLSenaladaCellRenderer( ) );

		// Política de cambio de tamaño.
		tabFseñalada.setAutoResizeMode( JTable.AUTO_RESIZE_NEXT_COLUMN );
		tabFseñalada.getColumnModel( ).getColumn( 0 ).setMinWidth( 20 );
		tabFseñalada.getColumnModel( ).getColumn( 1 ).setMinWidth( 10 );
		tabFseñalada.getColumnModel( ).getColumn( 2 ).setMinWidth( 10 );
		tabFseñalada.getColumnModel( ).getColumn( 3 ).setMinWidth( 10 );
		tabFseñalada.getColumnModel( ).getColumn( 4 ).setMinWidth( 7 );
		tabFseñalada.getColumnModel( ).getColumn( 5 ).setMinWidth( 7 );
		tabFseñalada.getColumnModel( ).getColumn( 6 ).setMinWidth( 7 );
		tabFseñalada.getColumnModel( ).getColumn( 0 ).setPreferredWidth( 100 );
		tabFseñalada.getColumnModel( ).getColumn( 1 ).setPreferredWidth( 30 );
		tabFseñalada.getColumnModel( ).getColumn( 2 ).setPreferredWidth( 50 );
		tabFseñalada.getColumnModel( ).getColumn( 3 ).setPreferredWidth( 50 );
		tabFseñalada.getColumnModel( ).getColumn( 4 ).setPreferredWidth( 20 );
		tabFseñalada.getColumnModel( ).getColumn( 5 ).setPreferredWidth( 20 );
		tabFseñalada.getColumnModel( ).getColumn( 6 ).setPreferredWidth( 20 );
	}
	
	/** Modelo y Tabla de funciones Llamadoras. */
	private ModeloTablaFuncionesLlamadoras	modTabFLlamadoras = new ModeloTablaFuncionesLlamadoras( );

	private JTable							tabFllamadoras		= new JTable( );
	{
		tabFllamadoras.setModel( modTabFLlamadoras );

		// renderers
		Enumeration < TableColumn > v = tabFllamadoras.getColumnModel( )
				.getColumns( );
		while ( v.hasMoreElements( ) )
			v.nextElement( ).setHeaderRenderer( new CabeceraMultilineaTableCellRenderer( ) );
		tabFllamadoras.setDefaultRenderer( String.class ,
				new ColumnaTextoMultilineaTablaCellRenderer( ) );
		tabFllamadoras.setDefaultRenderer( Integer.class ,
				new TabFLLCellRenderer( ) );
		tabFllamadoras.setDefaultRenderer( Double.class ,
				new TabFLLCellRenderer( ) );

		// Política de cambio de tamaño.
		tabFllamadoras.setAutoResizeMode( JTable.AUTO_RESIZE_NEXT_COLUMN );
		tabFllamadoras.getColumnModel( ).getColumn( 0 ).setMinWidth( 20 );
		tabFllamadoras.getColumnModel( ).getColumn( 1 ).setMinWidth( 10 );
		tabFllamadoras.getColumnModel( ).getColumn( 2 ).setMinWidth( 10 );
		tabFllamadoras.getColumnModel( ).getColumn( 3 ).setMinWidth( 10 );
		tabFllamadoras.getColumnModel( ).getColumn( 4 ).setMinWidth( 7 );
		tabFllamadoras.getColumnModel( ).getColumn( 5 ).setMinWidth( 7 );
		tabFllamadoras.getColumnModel( ).getColumn( 0 ).setPreferredWidth( 100 );
		tabFllamadoras.getColumnModel( ).getColumn( 1 ).setPreferredWidth( 30 );
		tabFllamadoras.getColumnModel( ).getColumn( 2 ).setPreferredWidth( 50 );
		tabFllamadoras.getColumnModel( ).getColumn( 3 ).setPreferredWidth( 50 );
		tabFllamadoras.getColumnModel( ).getColumn( 4 ).setPreferredWidth( 20 );
		tabFllamadoras.getColumnModel( ).getColumn( 5 ).setPreferredWidth( 20 );
	}
	
	/** Modelo y Grafico de de funciones llamadoras. */
	private ModeloGraficoFLlamadoras		modGraFLlamadoras;
	{
		modGraFLlamadoras = new ModeloGraficoFLlamadoras( );
	}

	private JGrafico							graFllamadoras		= new JGrafico( );
	{
		graFllamadoras.setModel( modGraFLlamadoras );
	}

	/** Modelo y Tabla de funciones Llamadas. */	
	private ModeloTablaFuncionesLlamadas	modTabFLlamadas = new ModeloTablaFuncionesLlamadas( );

	private JTable							tabFllamadas		= new JTable( );
	{
		tabFllamadas.setModel( modTabFLlamadas );

		// renderers
		Enumeration < TableColumn > v = tabFllamadas.getColumnModel( )
				.getColumns( );
		while ( v.hasMoreElements( ) )
			v.nextElement( ).setHeaderRenderer( new CabeceraMultilineaTableCellRenderer( ) );
		tabFllamadas.setDefaultRenderer( String.class ,
				new ColumnaTextoMultilineaTablaCellRenderer( ) );
		tabFllamadas.setDefaultRenderer( Integer.class ,
				new TabFLLCellRenderer( ) );
		tabFllamadas.setDefaultRenderer( Double.class ,
				new TabFLLCellRenderer( ) );

		// Política de cambio de tamaño.
		tabFllamadas.setAutoResizeMode( JTable.AUTO_RESIZE_NEXT_COLUMN );
		tabFllamadas.getColumnModel( ).getColumn( 0 ).setMinWidth( 20 );
		tabFllamadas.getColumnModel( ).getColumn( 1 ).setMinWidth( 10 );
		tabFllamadas.getColumnModel( ).getColumn( 2 ).setMinWidth( 10 );
		tabFllamadas.getColumnModel( ).getColumn( 3 ).setMinWidth( 10 );
		tabFllamadas.getColumnModel( ).getColumn( 4 ).setMinWidth( 7 );
		tabFllamadas.getColumnModel( ).getColumn( 5 ).setMinWidth( 7 );
		tabFllamadas.getColumnModel( ).getColumn( 0 ).setPreferredWidth( 100 );
		tabFllamadas.getColumnModel( ).getColumn( 1 ).setPreferredWidth( 30 );
		tabFllamadas.getColumnModel( ).getColumn( 2 ).setPreferredWidth( 50 );
		tabFllamadas.getColumnModel( ).getColumn( 3 ).setPreferredWidth( 50 );
		tabFllamadas.getColumnModel( ).getColumn( 4 ).setPreferredWidth( 20 );
		tabFllamadas.getColumnModel( ).getColumn( 5 ).setPreferredWidth( 20 );
	}

	/** Modelo y Grafico de de funciones llamadoras. */
	private ModeloGraficoFLlamadas			modGraFllamadas = new ModeloGraficoFLlamadas( );

	private JGrafico							graFllamadas		= new JGrafico( );
	{
		graFllamadas.setModel( modGraFllamadas );
	}



	
	/*********** grafo ************************************************/
	private ModeloArbolGrafoLlamadas		modArbGrafoLlamadas;
	{
		modArbGrafoLlamadas = new ModeloArbolGrafoLlamadas( );
	}
	
	/**
	 * El arbol del panel va a ser sustituido temporalmente por una lista de
	 * funciones
	 * 
	 */
	private ModeloListaFunciones			modListaFunciones;
	{
		modListaFunciones = new ModeloListaFunciones( );
	}
	
	/**
	 * Arbol del panel del Grafo de Llamadas. Muestra todo el grafo de llamadas
	 * del programa en forma de árbol.
	 */
	private JTree							arbGrafoLlamadas	= new JTree( );
	{
		arbGrafoLlamadas.setModel( modArbGrafoLlamadas );
		// resto de cosas para enlazarlos con los otros gadget
		DefaultTreeSelectionModel modeloSeleccion = new DefaultTreeSelectionModel( );
		modeloSeleccion
				.setSelectionMode( DefaultTreeSelectionModel.SINGLE_TREE_SELECTION );
		arbGrafoLlamadas.setSelectionModel( modeloSeleccion );
		arbGrafoLlamadas.setCellRenderer( new FuncionTreeRenderer( ) );
		// añadir listeners
		arbGrafoLlamadas.addTreeSelectionListener( modListaFunciones );
		arbGrafoLlamadas.addTreeSelectionListener( modArbGrafoLlamadas );

		/** el treelistener del modListafuncion se encarga de llamar a todos los demás... */
		/*arbGrafoLlamadas.addTreeSelectionListener( modGraFllamadas );
		arbGrafoLlamadas.addTreeSelectionListener( modGraFLlamadoras );
		arbGrafoLlamadas.addTreeSelectionListener( modTabFLlamadas );
		arbGrafoLlamadas.addTreeSelectionListener( modTabFLlamadoras );
		arbGrafoLlamadas.addTreeSelectionListener( modTabFSeñalada );*/

	}

	/*
	 * protected JList lisListaFunciones = new JList(); {
	 * lisListaFunciones.setSelectionMode
	 * (javax.swing.ListSelectionModel.SINGLE_SELECTION);
	 * lisListaFunciones.setModel(modListaFunciones);
	 * lisListaFunciones.setCellRenderer( new EntradaFuncion_CellRenderer () );
	 * lisListaFunciones.addListSelectionListener(modGraFllamadas);
	 * lisListaFunciones.addListSelectionListener(modGraFLlamadoras);
	 * lisListaFunciones.addListSelectionListener(modTabFLlamadas);
	 * lisListaFunciones.addListSelectionListener(modTabFLlamadoras);
	 * lisListaFunciones.addListSelectionListener(modTabFSeñalada); }
	 */

	protected JComboBox						lisListaFunciones	= new JComboBox( );
	{
		lisListaFunciones.setModel( modListaFunciones );
		//lisListaFunciones.setRenderer( new EntradaFuncion_CellRenderer( ) );
		lisListaFunciones.setRenderer( new FuncionListRenderer( ) );
		lisListaFunciones.addActionListener( modGraFllamadas );
		lisListaFunciones.addActionListener( modGraFLlamadoras );
		lisListaFunciones.addActionListener( modTabFLlamadas );
		lisListaFunciones.addActionListener( modTabFLlamadoras );
		lisListaFunciones.addActionListener( modTabFSeñalada );
		lisListaFunciones.setBackground( Color.WHITE );
		lisListaFunciones.addActionListener( modArbGrafoLlamadas );
		lisListaFunciones.addActionListener( this );
	}
	
	private JPanel panSeñalada = new JPanel( new MigLayout( "insets panel, fill") );
	{
		panSeñalada.add( etiFseñalada , "sgy Cmd, split 2,gapafter rel" );
		panSeñalada.add( lisListaFunciones ,
				"growx  , pushx,wmin 100pt, sgy Cmd, wrap" );
		panSeñalada.add( tabFseñalada.getTableHeader( ) ,
				"gaptop unrel, growx,wrap" );
		panSeñalada.add( tabFseñalada , "growx,wrap" );
		/*
		 * panelderecho.add( new JScrollPane(tabFseñalada),
		 * "gaptop unrel, grow, push,  wrap" );
		 */	
		
		panSeñalada.setBorder( new EtchedBorder() );
	}
	
	private JPanel panLlamadoras = new JPanel( new MigLayout( "insets panel, fill") ); 
	{
		panLlamadoras.add( etiFllamadoras , "gaptop unrel, wrap" );
		panLlamadoras.add( tabFllamadoras.getTableHeader( ) , " growx,wrap" );
		panLlamadoras.add( tabFllamadoras , "growx,wrap" );
		/*
		 * panelderecho.add( new JScrollPane(tabFllamadoras),
		 * "gaptop unrel, growx,  wrap" );
		 */panLlamadoras.add( graFllamadoras , "grow, wrap, push,sgy gra, height 40:pref:pref" );
		 
		 panLlamadoras.setBorder( new EtchedBorder() );
	}
	
	private JPanel panLlamadas = new JPanel( new MigLayout( "insets panel, fill") ); 
	{
		panLlamadas.add( etiFllamadas , "gaptop unrel, wrap" );
		panLlamadas.add( tabFllamadas.getTableHeader( ) , "growx,wrap" );
		panLlamadas.add( tabFllamadas , "growx,wrap" );
		/*
		 * panelderecho.add( new JScrollPane(tabFllamadas),
		 * "gaptop unrel, growx,  wrap" );
		 */panLlamadas.add( graFllamadas , "grow, push, wrap, sgy gra, height 40:pref:pref" );
		 
		 panLlamadas.setBorder( new EtchedBorder() );
	}

	private JSplitPane splitpane = new JSplitPane( JSplitPane.HORIZONTAL_SPLIT );
	/************************ CONSTRUCTOR ***********************/
	/**
	 * Crea un panel estadistica de grafo de llamadas, con el ModeloXML
	 * referenciado por <code>x<code>.
	 * 
	 * @param x
	 *            ModeloXML
	 */
	public JPanelGrafoLlamadas ( ModeloEstadistica x ) {
		super( new MigLayout( "insets 0, fill" ) );

		setDatosXML( x );

		addComponentes( );
	}

	/**
	 * Crea un panel estadistica de grafo de llamadas
	 */
	public JPanelGrafoLlamadas ( ) {
		super( new MigLayout( "insets 0,fill" ) );

		addComponentes( );

	}

	protected void addComponentes ( ) {
		// crear split pane (panel principal)
		//JSplitPane splitpane = new JSplitPane( JSplitPane.HORIZONTAL_SPLIT );
		// añadir split pane
		add( splitpane , "grow" );

		// añadir componente izquierdo... el árbol
		JPanel panelizquierdo = new JPanel(
				new MigLayout( "insets panel, fill" ) );

		/*
		 * panelizquierdo.add( etiListaFunciones, "wrap" );
		 * panelizquierdo.add(new JScrollPane(lisListaFunciones),
		 * "span, wrap, grow, push");
		 */

		panelizquierdo.add( etiArbol , "gaptop unrel, wrap" );
		panelizquierdo.add( new JScrollPane( arbGrafoLlamadas ) ,
				"span, wrap, grow, push" );

		// crear componente derecho
		JPanel panelderecho = new JPanel( new MigLayout( "insets panel, fillx, hidemode 3" ) );
		panelderecho.add( panSeñalada, "growx, wrap, aligny top , gapbottom unrel" );
		panelderecho.add( panLlamadoras, "growx, wrap, gapbottom unrel" );
		panelderecho.add( panLlamadas, "growx, wrap" );
		
		// añadir componente izquierdo al split
		splitpane.setLeftComponent( panelizquierdo );
		// añadir componente derecho al split
		splitpane.setRightComponent( new JScrollPane( panelderecho ) );

		// Propiedades al splitpane
		// Tamañs mínimos para los componenetes
		Dimension tamañoMinimo = new Dimension( 300 , 300 );
		panelizquierdo.setMinimumSize( tamañoMinimo );
		panelderecho.setMinimumSize( tamañoMinimo );
		// Tamaño de la barra divisora
		splitpane.setDividerSize( 8 );
		// Que se pueda expandir y contraer
		splitpane.setOneTouchExpandable( true );
	}

	/************************ METODOS ***********************/

	@Override
	public void setDatosXML ( ModeloEstadistica x ) {

		modTabFLlamadoras.setDatosXML( x );

		modTabFLlamadas.setDatosXML( x );

		modTabFSeñalada.setDatosXML( x );

		modGraFLlamadoras.setDatosXML( x );

		modGraFllamadas.setDatosXML( x );

		modListaFunciones.setDatosXML( x );

		modArbGrafoLlamadas.setDatosXML( x );
		
	}

	public void actualizarTablas ( EntradaFuncion x ) {
		
		if ( x.getHijos( ).size( ) + x.getHijosMiembroCiclo( ).size( ) > 0 ) {
			/*this.etiFllamadas.setVisible( false );
			this.tabFllamadas.setVisible( false );
			this.tabFllamadas.getTableHeader( ).setVisible( false );
			this.graFllamadas.setVisible( false );*/
			this.panLlamadas.setVisible( true );
		} else {
			/*this.etiFllamadas.setVisible( true );
			this.tabFllamadas.setVisible( true );
			this.tabFllamadas.getTableHeader( ).setVisible( true );
			this.graFllamadas.setVisible( true );*/
			this.panLlamadas.setVisible( false );
		}
		if ( x.getPadres( ).size( ) + x.getPadresMiembroCiclo( ).size( ) == 0 ) {
			/*this.etiFllamadoras.setVisible( false );
			this.tabFllamadoras.setVisible( false );
			this.tabFllamadoras.getTableHeader( ).setVisible( false );
			this.graFllamadoras.setVisible( false );*/
			this.panLlamadoras.setVisible( false );
		} else {
			/*this.etiFllamadoras.setVisible( true );
			this.tabFllamadoras.setVisible( true );
			this.tabFllamadoras.getTableHeader( ).setVisible( true );
			this.graFllamadoras.setVisible( true );*/
			this.panLlamadoras.setVisible( true );
		}
	}

	/**
	 * Sirve para que el panel escuche los cambios en la funcion seleccionada y reaccione.
	 * (escucha el comboBox y quita o deja las f. llamadas o llamadoras).
	 */
	@Override
	public void actionPerformed ( ActionEvent e ) {
		EntradaFuncion fichero = null;

		if ( e.getSource( ) instanceof JComboBox ) {
			JComboBox x = (JComboBox ) e.getSource( );
			if( x.getSelectedItem( ) instanceof EntradaFuncion ) {
				fichero = (EntradaFuncion) ( x.getSelectedItem( ) );
				if ( fichero != null ) {
					actualizarTablas( fichero );
				}
			}
		}
		
	}
}
