package interfaz.vtraductor;

import interfaz.acciones.CrearYGestionarAcciones;
import interfaz.dialogos.error.DialogoError;
import interfaz.semaforo.JSemaforo;

import java.awt.Color;
import java.awt.Font;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ResourceBundle;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import modelo.vtraduccion.ModeloTR;
import negocio.PreferenciasSingleton;
import negocio.ejecutarOrdenes.InterfazOrdenes;
import net.miginfocom.swing.MigLayout;

public class JPanelTR extends JPanel implements ChangeListener , InterfazOrdenes {

	/** recursos para dar soporte sobre idiomas */
	static ResourceBundle idiomas = ResourceBundle.getBundle("interfaz.idiomas.Recursos", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	/** recursos de iconos etc... */
	static ResourceBundle recursos = ResourceBundle.getBundle("interfaz.Recursos", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	/** recursos de idiomas para errores */
	static ResourceBundle errores = ResourceBundle.getBundle("interfaz.errores.Errores", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );

	private JLabel				etiSalida			= new JLabel(
															idiomas
																	.getString( "eti_salidaGprofTR" ) );
	private JTextField			txtSalida			= new JTextField( );
	{
		txtSalida.setEditable( false );
	}

	private JLabel				etiCmd				= new JLabel(
															idiomas
																	.getString( "eti_comandoTR" ) );

	private JTextField			txtCmd				= new JTextField( );
	{
		txtCmd.setEditable( false );
	}

	private JLabel				etiDir				= new JLabel(
															idiomas
																	.getString( "eti_directorioTrabajo" ) );
	private JTextField			txtDir				= new JTextField( );
	{
		txtDir.setEditable( false );
	}
	
	private JButton				botTraducir			= new JButton( );
	private JButton				botNombreXML	= new JButton( );
	private JButton				botCambiarDT		= new JButton( );
	private JButton				botEditarSalida		= new JButton( );
	private JButton				botEditarComando	= new JButton( );
	
	private JLabel				etiStatus			= new JLabel(
			idiomas
					.getString( "eti_estado" ) );
	private JSemaforo			semStatus			= new JSemaforo( );

	private JLabel				etiXML				= new JLabel(
			idiomas
					.getString( "eti_XMLtraduccion" ) );

	private JTextArea			atxXML			= new JTextArea( );
	{
		atxXML.setEditable( false );
		atxXML.setBackground( Color.WHITE );// fondo blanco
		atxXML.setForeground( new Color(26,97,26) );// letras verde oscuro
		//atxXML.setFont( atxXML.getFont( ).deriveFont( Font.BOLD ) );// negritas
		atxXML.setFont( PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getEstadisticasFont( ).deriveFont( Font.BOLD ) );
		atxXML.setLineWrap( true );
		atxXML.setWrapStyleWord( true );
	}
	
	/** Serial version uid */
	private static final long	serialVersionUID	= -7191529819222353003L;

	/************************ PROPIEDADES ***********************/
	private ModeloTR			moTR				= null;

	public boolean isVacio () {
		return this.moTR == null;
	}
	
	public void cambiarFuente ( Font f ) {
		atxXML.setFont( f );
	}
	
	public JPanelTR ( ModeloTR mg ) {
		super( new MigLayout( "fill" ) );

		crearComponentes( );
		setModel( mg );
	}
	
	public JPanelTR ( ) {
		super( new MigLayout( "fill" ) );

		crearComponentes( );
		setModel( null );	
	}

	protected void setModel ( ModeloTR mg ) {
		this.moTR = mg;
		
		if( moTR != null ) {			
			if ( this.moTR.getDirectorioTrabajo( ) == null ) {
				if ( this.moTR.getFicheroSalida( ) != null ) {
					this.moTR.setDirectorioTrabajo( this.moTR
							.getFicheroSalida( ).getParentFile( ) );
				} else {
					this.moTR.setDirectorioTrabajo( 
							PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getWorkspaceTR( ) ); // gprof
				}
			}
			this.moTR.addChangeListener( this );
		}
		
		actualizarModel( );
	}

	public ModeloTR getModel ( ) {
		return this.moTR;
	}

	protected void crearComponentes ( ) {
		setBorder( new EtchedBorder( ) );
		setOpaque( true );

		{
			
			
			botTraducir.setHideActionText( true );
			botTraducir.setAction( CrearYGestionarAcciones.getInstance().getAccionTraducir( ) );
			
			botNombreXML.setHideActionText( true );
			botNombreXML.setAction( CrearYGestionarAcciones.getInstance().getAccionNombreXML( ) );
			
			this.botCambiarDT.setAction( CrearYGestionarAcciones.getInstance().getAccionCambiarDirectorioTrabajoTR( ) );
			this.botCambiarDT.setIcon( null );
			this.botCambiarDT.setText( "..." );
			
			this.botEditarSalida.setAction( CrearYGestionarAcciones.getInstance().getAccionEditarSalidaGprof( ) );
			this.botEditarSalida.setIcon( null );
			this.botEditarSalida.setText( "..." );

			this.botEditarComando.setAction( CrearYGestionarAcciones.getInstance().getAccionEditarComandoTR( ) );
			this.botEditarComando.setIcon( null );
			this.botEditarComando.setText( "..." );
		}

		JPanel pnlEstatus;
		{
			pnlEstatus = new JPanel( new MigLayout( ) );
			pnlEstatus.add( etiStatus , "wrap" );
			pnlEstatus.add( semStatus , "alignx center, aligny top" );
			pnlEstatus.setBorder( BorderFactory
					.createEtchedBorder( EtchedBorder.LOWERED ) );
		}
		this.add( pnlEstatus , "sizegroupy a, gapafter unrel, split 3" );
		
		JPanel pnltr;
		{
			pnltr = new JPanel ( new MigLayout() );
			pnltr.add( etiSalida , "sgy Cmd");
			pnltr.add( txtSalida , "growx,pushx,wmin 100pt, sgy Cmd" );
			pnltr.add( botEditarSalida , "sgy Cmd, wrap" );
			pnltr.add( etiCmd , "sgy Cmd");
			/*pnltr.add( txtCmd , "growx,pushx,wmin 100pt, sgy Cmd, span, wrap" );*/
			pnltr.add( txtCmd , "growx,pushx,wmin 100pt, sgy Cmd" );
			pnltr.add( botEditarComando , "sgy Cmd, wrap" );
			pnltr.add( etiDir , "sgy Cmd" );
			pnltr.add( txtDir , "growx,pushx,wmin 60pt, sgy Cmd" );
			pnltr.add( botCambiarDT , "sgy Cmd, wrap" );
			pnltr.setBorder( BorderFactory.createEtchedBorder( EtchedBorder.LOWERED ) );
		}
		this.add( pnltr , "growx, pushx, sizegroupy a" );
		this.add( botTraducir , "gapbefore unrel, wrap" );

		this.add( etiXML , "wrap" );
		this.add( new JScrollPane( atxXML ) , "span, grow" );

	}

	@Override
	public void empezarOrden ( String comando ) {
		if ( !isVacio() ) {
			semStatus.setAMBAR( );
			semStatus.setToolTipText( idiomas.getString( "eti_ejecutandoTR" ) );
			this.atxXML.setText( idiomas.getString( "eti_trInicio" ) + "\n" +
					this.txtCmd.getText( ) );
		}
	}

	@Override
	public void ordenErrorIndeterminado( String comando, File directorio, Exception e ){
		String saltoLinea = "\n";
		String descripcion;

		descripcion = errores.getString( "err_errorEjecuciónOrden" )
				+ saltoLinea + errores.getString( "err_errorEjecuciónOrden_orden" )
				+ saltoLinea + comando
				+ saltoLinea + errores.getString( "err_errorEjecución_directorio" )
				+ saltoLinea + directorio.getName( )
				+ saltoLinea
				+ saltoLinea + errores.getString( "err_detalles:" )
				+ saltoLinea + e.getMessage( );
		
		DialogoError.getInstance( ).showErrorDialog( descripcion );
	}
	
	@Override
	public void terminarOrden ( int devolverResultadoEjecución ,
			String salida, String error ) {
		if ( !isVacio( ) ) {
			if ( devolverResultadoEjecución == 0 ) {

				this.atxXML.setText( salida );
				this.moTR.setSalidaXML( salida );
				semStatus.setVERDE( );
				semStatus.setToolTipText( idiomas
						.getString( "eti_SalidaTraduccionExito" ) );
				try {
					File nuevoFicheroResultado = moTR.getFicheroXML( );
					if ( !nuevoFicheroResultado.exists( ) ) {
						if ( nuevoFicheroResultado.createNewFile( ) )
							DialogoError.getInstance().showInfoDialog(  
									errores.getString( "no-err_crearFichero" ) + nuevoFicheroResultado
									.getCanonicalPath( ) ); 
						else
							DialogoError.getInstance( ).showErrorDialog(  
									errores.getString( "err_crearFichero" ) + nuevoFicheroResultado
									.getCanonicalPath( ) );
					} else {
						/** hacer lo que toca después */
						if ( !nuevoFicheroResultado.delete( ) )
							DialogoError.getInstance( ).showErrorDialog( 
										errores.getString( "err_borrarFichero" ) + nuevoFicheroResultado
										.getAbsolutePath( ) );

						if ( nuevoFicheroResultado.createNewFile( ) )
							DialogoError.getInstance().showInfoDialog(  
									errores.getString( "no-err_crearFichero" ) + nuevoFicheroResultado
									.getCanonicalPath( ) ); 
						else
							DialogoError.getInstance( ).showErrorDialog(  
									errores.getString( "err_crearFichero" ) + nuevoFicheroResultado
									.getCanonicalPath( ) );
					}
					/** escribir */
					if ( !nuevoFicheroResultado.exists( )
							|| !nuevoFicheroResultado.canWrite( ) ) {
						nuevoFicheroResultado.setWritable( true );
					}
					FileWriter escribirFichero = new FileWriter(
							nuevoFicheroResultado );
					BufferedWriter escribirBuffer = new BufferedWriter(
							escribirFichero );
					escribirBuffer.write( moTR.getSalidaXML( ) );
					escribirBuffer.close( );
					moTR.setFicheroXML( nuevoFicheroResultado );

				} catch ( IOException e ) {
					DialogoError.getInstance( ).showErrorDialog( 
							errores.getString( "err_io" )  );
					e.printStackTrace( );
				}
			} else {
				this.atxXML.setText( idiomas.getString( "eti_gprofError" )
						+ "\n" + error );
				semStatus.setROJO( );
				semStatus.setToolTipText( idiomas
						.getString( "eti_SalidaTraduccionFracaso" ) );
			}
		}
	}


	@Override
	public void stateChanged ( ChangeEvent e ) {
		if ( e.getSource( ) instanceof ModeloTR ) {
			this.actualizarModel( );
		}
		
		
	}
	
	private void actualizarModel ( ) {
		
		if ( moTR != null ) {

			/* txtDir */
			String directorioTrabajo = new String( "" );
			if ( this.moTR.getDirectorioTrabajo( ) != null ) {
				try {
					directorioTrabajo = this.moTR.getDirectorioTrabajo( )
							.getCanonicalPath( );
				} catch ( IOException e ) {
					e.printStackTrace( );
					directorioTrabajo = idiomas
							.getString( "eti_errorDirectorioTrabajo" );
				}
			}
			txtDir.setText( directorioTrabajo );

			/* txtCmd */
			String [] comp = this.moTR.getComandoTRAuto( );
			String comps = new String( );
			for ( int i = 0 ; i < comp.length ; i++ ) {
				comps += " " + comp[i];
			}
			comps.trim( );
			this.txtCmd.setText( comps );

			/* txtSalida */
			String salida = new String( "" );
			if ( this.moTR.getFicheroSalida( ) != null ) {
				try {
					salida = this.moTR.getFicheroSalida( ).getCanonicalPath( );
				} catch ( IOException e ) {
					e.printStackTrace( );
					salida = idiomas.getString( "eti_errorNombreSalida" );
				}
			}
			this.txtSalida.setText( salida );

			/* atxXML */
			if ( this.moTR.getSalidaXML( ) != null ) {
				this.atxXML.setText( moTR.getSalidaXML( ) );
			}
		} else {
			/* txtDir */
			txtDir.setText( "" );

			/* txtCmd */
			this.txtCmd.setText( "" );

			/* txtSalida */
			this.txtSalida.setText( "" );

			/* atxXML */
			this.atxXML.setText( "" );
		}
	}
}
