package modelo.ficheros;

import interfaz.dialogos.error.DialogoError;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import negocio.PreferenciasSingleton;

public class UtilidadesFichero {

	ResourceBundle		errores				= ResourceBundle
	.getBundle( "interfaz.errores.Errores", 
			PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );
	
	/**
	 * get relative path of File 'f' with respect to 'home' directory example :
	 * home = /a/b/c f = /a/d/e/x.txt s = getRelativePath(home,f) =
	 * ../../d/e/x.txt
	 * 
	 * @param home
	 *            base path, should be a directory, not a file, or it doesn't
	 *            make sense
	 * @param f
	 *            file to generate path for
	 * @return path from home to f as a string
	 * @throws IOException
	 */
	public String getRelativePath ( File home , File destino )
			throws IOException {
		List < String > homelist;
		List < String > filelist;
		String s;
	
		if ( !home.isDirectory( ) ) {
			home = home.getParentFile( );
		}
	
		homelist = getPathList( home );
		filelist = getPathList( destino );
		s = matchPathLists( homelist , filelist );
	
		return s;
	}

	/**
	 * test the function
	 */
	public static void main ( String args[] ) {
		if ( args.length != 2 ) {
			System.out.println( "RelativePath <home> <file>" );
			return;
		}
	
		File home = new File( args[0] );
		File dest = new File( args[1] );
		try {
			System.out.println( "home = " + home.getAbsolutePath( ) );
			System.out.println( "file = " + dest.getCanonicalPath( ) );
	
			System.out.println( "path = " + new UtilidadesFichero().getRelativePath( home , dest ) );
		} catch ( IOException e ) {
			e.printStackTrace( );
		}
	}

	/******************************/
	
	/**
	 * break a path down into individual elements and add to a list. example :
	 * if a path is /a/b/c/d.txt, the breakdown will be [d.txt,c,b,a]
	 * 
	 * @param f
	 *            input file
	 * @return a List collection with the individual elements of the path in
	 *         reverse order
	 * @throws IOException
	 */
	private  List < String > getPathList ( File f ) throws IOException {
		List < String > l = new ArrayList < String >( );
		File r;
	
		r = f.getCanonicalFile( );
		while ( r != null ) {
			l.add( r.getName( ) );
			r = r.getParentFile( );
		}
	
		return l;
	}

	/**
	 * figure out a string representing the relative path of 'f' with respect to
	 * 'r'
	 * 
	 * @param r
	 *            home path
	 * @param f
	 *            path of file
	 */
	private  String matchPathLists ( List < String > r , List < String > f ) {
		int i;
		int j;
		String s;
		// start at the beginning of the lists
		// iterate while both lists are equal
		s = "";
		i = r.size( ) - 1;
		j = f.size( ) - 1;
	
		// first eliminate common root
		while ( ( i >= 0 ) && ( j >= 0 ) && ( r.get( i ).equals( f.get( j ) ) ) ) {
			i--;
			j--;
		}
	
		// for each remaining level in the home path, add a ..
		for ( ; i >= 0 ; i-- ) {
			s += ".." + File.separator;
		}
	
		// for each level in the file path, add the path
		for ( ; j >= 1 ; j-- ) {
			s += f.get( j ) + File.separator;
		}
	
		// file name
		s += f.get( j );
		return s;
	}

	/**
	 * función que devuelve el path (o camino) de un fichero (f). Si esto no
	 * fuera posible por algún error del sistema o de entrada salida, devolverá
	 * únicamente el nombre.
	 * 
	 * @param f
	 * @return Path del fichero.
	 */
	public String caminoFichero ( File f ) {
		String str = new String( );
	
		try {
			str = f.getCanonicalPath( );
			/*
			 * Throws: IOException - If an I/O error occurs, which is possible
			 * because the construction of the canonical pathname may require
			 * filesystem queries SecurityException - If a required system
			 * property value cannot be accessed, or if a security manager
			 * exists and its SecurityManager.checkRead(java.io.FileDescriptor)
			 * method denies read access to the file
			 */
		} catch ( Exception e ) {
			// tratar error correctamente!!!
			str = f.getName( );
			System.err.println( errores.getString( "err_NoCanonicalPath" )
					+ str );
		}
	
		return str;
	}

	/**
	 * función que trata de leer el contenido de un fichero (f) y devolver un
	 * String con su contenido.
	 * 
	 * @param f
	 * @return
	 */
	public String leerFichero ( File f ) {
		/** Obtener el contenido del fichero en strCompleto */
		// a partir de aquí es meter el contenido del fichero en el area de
		// texto (atxTexto)
		String str = new String( );
		String strCompleto = new String( );
		try { // Meter esto en un nuevo método
			BufferedReader in = new BufferedReader( new FileReader( f ) );
			// funciona en windows
			String newline = System.getProperty( "line.separator" );
			while ( ( str = in.readLine( ) ) != null ) {
				strCompleto = strCompleto + str + newline;
			} // Obtener el string del archivo línea a línea.
		} catch ( Exception e2 ) {
			String err = errores.getString( "err_errorLectura" ) + f.getName( );
			DialogoError.getInstance( ).showErrorDialog( err );
			e2.printStackTrace( );
		}
	
		return strCompleto;
	}

}
