package modelo.modeloXML;

import java.util.HashMap;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

public class EntradaFuncion {

	/**
	 * Número de índice que gprof asigna a esta Entrada.
	 */
	private Integer indice = null;
	
	/**
	 * Nombre de la función que representa.
	 */
	private String nombreFuncion = null;
	//No sé si será necesario hacer una clase Función :S
	
	/**
	 * Número total de llamadas recibidas por esta Función. Quedan excluidas las llamadas recursivas a sí misma.
	 * NOTA: Es derivado, está por evitar calcularlo, se redunda.
	 */
	private Integer numeroLlamadas = null;
	
	/**
	 * Número de llamadas recursivas de esta Función. 
	 */
	private Integer numeroLlamadasRecursivas = null;
	
	/**
	 * Tiempo en segundos, que se gasta exclusivamente en el cuerpo de esta Función. Quedando excluido el 
	 * tiempo gastado en funciones llamadas por esta Función.
	 */
	private Double tiempoPropio = null;
	
	/**
	 * Tiempo en segundos, que se gasta en dentro de las funciones que son llamadas por esta función, cuando son
	 * llamadas por ésta.
	 */
	private Double tiempoHijos = null;
	
	/**
	 * El vector padres, contiene una lista con todos los padres de esta Función. 
	 * Está simplemente por referenciarlos. 
	 * NOTA: Es un dato que se podría obtener haciendo un árbol y viendo cuando es hijo esta función 
	 * de cualquier otra para saber que es su padre. Pero para ahorrar tiempo y gasto innecesario, se redunda
	 * la información.
	 */
	private Vector < EntradaFuncion > padres = new Vector < EntradaFuncion > (0);
	
	/**
	 * HashMap que contiene una lista con todos los hijos de esta Función. Y el número de veces que 
	 * es llamada por ésta Función.
	 * NOTA: No están incluidos, los hijos que pertenezcan a su propio ciclo.
	 */
	private HashMap < EntradaFuncion, Integer > hijos = new HashMap < EntradaFuncion, Integer  > (0); 
	
	/**
	 * Ciclo al que pertenece esta función. Si no pertenece a ningún ciclo, se mantiene en null.
	 */
	private Ciclo ciclo = null;
	
	/**
	 * HashMap que contiene una lista con todas los hijos de esta Función, que pertenecen a su propio ciclo.
	 * Y el número de veces que son llamados por esta Función.
	 */
	private HashMap < EntradaFuncion, Integer > hijosMiembroCiclo = new HashMap < EntradaFuncion, Integer > (0);
	
	/**
	 * El vector padresMiembroCiclo, contiene una lista con todos los padres de esta Función, que pertenecen a
	 * su propio ciclo. 
	 * Está simplemente por referenciarlos. 
	 * NOTA: Es un dato que se podría obtener haciendo un árbol y viendo cuando es hijo esta función 
	 * de cualquier otra para saber que es su padre. Pero para ahorrar tiempo y gasto innecesario, se redunda
	 * la información.
	 */
	private Vector < EntradaFuncion > padresMiembroCiclo = new Vector < EntradaFuncion > (0);
	
	/**
	 * @return the indice
	 */
	public Integer getIndice() {
		return indice;
	}

	/**
	 * @return el nombreFuncion
	 */
	public String getNombreFuncion() {
		return nombreFuncion;
	}
	/**
	 * @return the numeroLlamadas
	 */
	public Integer getNumeroLlamadas() {
		return numeroLlamadas;
	}

	/**
	 * @return the numeroLlamadasRecursivas
	 */
	public Integer getNumeroLlamadasRecursivas() {
		return numeroLlamadasRecursivas;
	}

	/**
	 * @return the tiempoPropio
	 */
	public Double getTiempoPropio() {
		return tiempoPropio;
	}

	/**
	 * @return the tiempoHijos
	 */
	public Double getTiempoHijos() {
		return tiempoHijos;
	}

	/**
	 * @return the hijos
	 */
	public Collection < EntradaFuncion > getHijos() {
		return hijos.keySet();
	}

	/**
	 * @return the padres
	 */
	public Collection < EntradaFuncion > getPadres() {
		return padres;
	}
	
	/**
	 * @return los hijosMiembroCiclo, si esta Función pertenece a un Ciclo. Si no, devuelve null.
	 */
	public Collection < EntradaFuncion > getHijosMiembroCiclo() {
		return hijosMiembroCiclo.keySet(); //hasCiclo() ? hijosMiembroCiclo.keySet() : null;
	}
	
	/**
	 * @return los padresMiembroCiclo, si esta Función pertenece a un Ciclo. Si no, devuelve null.
	 */
	public Collection < EntradaFuncion > getPadresMiembroCiclo() {
		return padresMiembroCiclo;//hasCiclo() ? padresMiembroCiclo : null;
	}
	
	
	public Integer getNumeroLlamadasHijo ( EntradaFuncion hijo ) {
		/* ponerse a uno mismo como hijo.
		Integer llamadas = new Integer(0);
		if ( hijo.equals( this ) )
			llamadas = this.getNumeroLlamadasRecursivas( );
		else if ( hijos.containsKey( hijo ) ) 
			llamadas = hijos.get( hijo );
		else
			llamadas = this.getNumeroLlamadasHijoMiembroCiclo( hijo );
			
		return llamadas;
		 */
		return hijos.containsKey( hijo ) ? hijos.get(hijo) : getNumeroLlamadasHijoMiembroCiclo( hijo );
	}
	
	public Double getEstimacionTiempoPropioHijo ( EntradaFuncion hijo ) {
		return hijos.containsKey( hijo ) ? hijo.getEstimacionTiempoPropioPadre( this ) : null;
	}
	
	public Double getEstimacionTiempoHijosHijo ( EntradaFuncion hijo ) {
		return hijos.containsKey( hijo ) ? hijo.getEstimacionTiempoHijosPadre( this ) : null;
	}
	
	public Integer getNumeroLlamadasPadre ( EntradaFuncion padre ) {
		return padres.contains( padre ) ? padre.getNumeroLlamadasHijo( this ) : getNumeroLlamadasPadreMiembroCiclo( padre );
	}
	
	public Double getEstimacionTiempoPropioPadre ( EntradaFuncion padre ) {
		Double estimacion = null;
		if ( padres.contains( padre )) {
			//la estimación del tiempo propio del padre, es el tiempo propio de esta funcion, dividido por el número de 
			//llamadas recibidas por esta función (en casos de funciones que pertenezcan a un ciclo, se divide entre el 
			//número de llamadas del ciclo), multiplicada por el número de llamadas que realiza la 
			//función padre a esta función. 
			if ( this.hasCiclo() ) {
				estimacion = this.getCiclo().getTiempoPropio() / this.getCiclo().getNumeroLlamadas().doubleValue( ) * padre.getNumeroLlamadasHijo( this ).doubleValue( );
			} else {
				estimacion = this.getTiempoPropio() / this.getNumeroLlamadas().doubleValue( ) * padre.getNumeroLlamadasHijo( this ).doubleValue( );
			}

		}
		return estimacion;
	}
	
	
	public Double getEstimacionTiempoHijosPadre ( EntradaFuncion padre ) {
		Double estimacion = null;
		if ( padres.contains( padre ) ) {
			/** la estimación del tiempo en hijos del padre, es el 
			tiempo en hijos propio de esta funcion, dividido por el número de 
			llamadas recibidas por esta función, multiplicada por el número de
			llamadas que realiza la función padre a esta función. */
			if ( this.hasCiclo() ) {
				estimacion = this.getCiclo().getTiempoHijos() / this.getCiclo().getNumeroLlamadas().doubleValue( ) * padre.getNumeroLlamadasHijo( this ).doubleValue( );
			} else {
				estimacion = this.getTiempoHijos() / this.getNumeroLlamadas().doubleValue( ) * padre.getNumeroLlamadasHijo( this ).doubleValue( );
			}		
		}
		return estimacion;
	}
	
	public Integer getNumeroLlamadasHijoMiembroCiclo ( EntradaFuncion hijo ) {
		return ( this.hasCiclo() && hijosMiembroCiclo.containsKey( hijo ) ) ? hijosMiembroCiclo.get(hijo) : /*null*/new Integer ( 0 );
	}

	public Integer getNumeroLlamadasPadreMiembroCiclo ( EntradaFuncion padre ) {
		return ( this.hasCiclo() && padresMiembroCiclo.contains( padre ) ) ? padre.getNumeroLlamadasHijo( this ) : /*null*/new Integer ( 0 );
	}
	//TODO mirar si los getNumeroLlamadasHijo || getNumeroLlamadasPadre || getNumeroLlamadasHijoMiembroCiclo || getNumeroLlamadasPadreMiembroCiclo  
	// sus referencias usan el null y cambiar por el Integer (0).
	
	/**
	 * @return the ciclo
	 */
	public Ciclo getCiclo() {
		return ciclo;
	}

	/**
	 * @return Devuelve verdadero si esta Función pertenece a algún ciclo. Y falso si no.
	 */
	public boolean hasCiclo () {
		return ciclo == null ? false : true;
	}
	
	/**
	 * @return Devuelve verdadero si esta Función es espontánea. Y falso si no.
	 */
	public boolean isEspontanea () {
		return ( padres.isEmpty() && padresMiembroCiclo.isEmpty() ) ? true : false;
	}
	
	/**
	 * @param indice the indice to set
	 */
	public void setIndice ( Integer indice ) {
		this.indice = indice;
	}

	/**
	 * @param nombreFuncion el nombre de la función que poner.
	 */
	public void setNombreFuncion ( String nombreFuncion ) {
		this.nombreFuncion = nombreFuncion;
	}
	/**
	 * @param numeroLlamadas the numeroLlamadas to set
	 */
	public void setNumeroLlamadas(Integer numeroLlamadas) {
		this.numeroLlamadas = numeroLlamadas;
	}

	/**
	 * @param numeroLlamadasRecursivas the numeroLlamadasRecursivas to set
	 */
	public void setNumeroLlamadasRecursivas(Integer numeroLlamadasRecursivas) {
		this.numeroLlamadasRecursivas = numeroLlamadasRecursivas;
	}

	/**
	 * @param tiempoPropio the tiempoPropio to set
	 */
	public void setTiempoPropio(Double tiempoPropio) {
		this.tiempoPropio = tiempoPropio;
	}

	/**
	 * @param tiempoHijos the tiempoHijos to set
	 */
	public void setTiempoHijos(Double tiempoHijos) {
		this.tiempoHijos = tiempoHijos;
	}


	/** Coloca todas las funciones hija de esta función y sus respectivos número de llamadas, como un Map de funciones y
	 * número de llamadas.
	 * @param hijo
	 * @param numeroLlamadas
	 */
	public void setHijos ( Map < EntradaFuncion, Integer > hijos ) {
		this.hijos = new HashMap < EntradaFuncion, Integer > (hijos);
		
		//iterar para añadir padres
		Iterator < Map.Entry < EntradaFuncion, Integer > > it = this.hijos.entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry < EntradaFuncion, Integer > e = it.next();
			e.getKey().addPadre( this );
		}
		
	}
	
	/** Añade la Función "hijo" como hijo de esta Función. Y además añade el número de llamadas
	 * que se hacen desde esta función a aquella.
	 * @param hijo
	 * @param numeroLlamadas
	 */
	public void addHijo ( EntradaFuncion hijo, Integer numeroLlamadas ) {
		if ( !hijo.equals( this ) ) {
			/*comprobar réplica*/
			if ( ! this.hijos.containsKey( hijo ) ) {
				this.hijos.put( hijo, numeroLlamadas );
			
				/*añadir este como padre*/
				hijo.addPadre ( this );
			}
		}
		
	}

	/** Coloca todas las funciones hija de esta función y sus respectivos número de llamadas, como un Map de funciones y
	 * número de llamadas.
	 * @param hijo
	 * @param numeroLlamadas
	 */
	public void setHijosMiembroCiclo ( Map < EntradaFuncion, Integer > hijos ) {
		this.hijosMiembroCiclo = new HashMap < EntradaFuncion, Integer > (hijos);
		
		//iterar para añadir padres
		Iterator < Map.Entry < EntradaFuncion, Integer > > it = this.hijosMiembroCiclo.entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry < EntradaFuncion, Integer > e = it.next();
			e.getKey().addPadreMiembroCiclo( this );
		}
		
	}
	
	/** Añade la Función "hijo" como hijo de esta Función. Y además añade el número de llamadas
	 * que se hacen desde esta función a aquella.
	 * @param hijo
	 * @param numeroLlamadas
	 */
	public void addHijoMiembroCiclo ( EntradaFuncion hijo, Integer numeroLlamadas ) {
		if ( !hijo.equals(  this  ) ) {
			/*comprobar réplica*/
			if ( ! this.hijosMiembroCiclo.containsKey( hijo ) ) {
				this.hijosMiembroCiclo.put( hijo, numeroLlamadas );
			
				/*añadir este como padre*/
				hijo.addPadreMiembroCiclo ( this );
			}
		}
	}
	
	/**
	 * @param ciclo the ciclo to set
	 */
	public void setCiclo (Ciclo ciclo) {
		if ( ciclo != null ) {
			ciclo.addMiembro( this );
			this.ciclo = ciclo;
		}
	}
	
		
	private void addPadre ( EntradaFuncion padre ) {
		//comprobar réplica
		if( ! padres.contains( padre ) ) {
			padres.add( padre );
		}
	}
	
	private void addPadreMiembroCiclo ( EntradaFuncion padre ) {
		//comprobar réplica
		if( ! padresMiembroCiclo.contains( padre ) ) {
			padresMiembroCiclo.add( padre );
		}
	}

	public String toString () {
		String resultado ="";
		Iterator <EntradaFuncion> i = padres.iterator();
		while( i.hasNext() ) {
			EntradaFuncion e = i.next();
			resultado += " <- " + e.getIndice() + "\t" + e.getTiempoPropio() + "\t" + e.getTiempoHijos() + "\t" + getNumeroLlamadasPadre(e) + "\t" + e.getNombreFuncion() + "\n";
		}
		Iterator <EntradaFuncion> j = padresMiembroCiclo.iterator();
		while( j.hasNext() ) {
			EntradaFuncion e = j.next();
			resultado += " <-* " + e.getIndice() + "\t" + "\t" + "\t" + getNumeroLlamadasPadreMiembroCiclo(e) + "\t" + e.getNombreFuncion() + "\n";
		}

		resultado += "" + getIndice() + "\t" + getTiempoPropio() + "\t" + getTiempoHijos() + "\t" + ( isEspontanea() ? "espontánea" : getNumeroLlamadas() + "+" + getNumeroLlamadasRecursivas() ) + "\t" + getNombreFuncion() + "\n";

		Iterator <EntradaFuncion> k = hijosMiembroCiclo.keySet().iterator();
		while( k.hasNext() ) {
			EntradaFuncion e = k.next();
			resultado += " ->* " + e.getIndice() + "\t" + "\t" + "\t" + getNumeroLlamadasHijoMiembroCiclo(e) + "\t" + e.getNombreFuncion() + "\n";
		}

		Iterator <EntradaFuncion> l = hijos.keySet().iterator();
		while( l.hasNext() ) {
			EntradaFuncion e = l.next();
			resultado += " -> " + e.getIndice() + "\t" + e.getTiempoPropio() + "\t" + e.getTiempoHijos() + "\t" + getNumeroLlamadasHijo(e) + "\t" + e.getNombreFuncion() + "\n";
		}
		return resultado;
	}


	public Double getPorcentajeLlamadoras ( EntradaFuncion f ) {
		// TODO Auto-generated method stub
		Double result = new Double ( 0.0 );
		
		if ( this.hasCiclo() && f.hasCiclo( ) && this.getCiclo( ) == f.getCiclo( ) )
			result = null; 
		else
			result = ( this.getEstimacionTiempoPropioPadre( f ) + 
					this.getEstimacionTiempoHijosPadre( f ) ) /
					( this.getTiempoHijos( ) + this.getTiempoPropio() );
		
		return result;
	}

	public Double getPorcentajeLlamadas ( EntradaFuncion f ) {
		// TODO Auto-generated method stub
		Double result = new Double ( 0.0 );
		
		if ( this.hasCiclo() && f.hasCiclo( ) && this.getCiclo( ) == f.getCiclo( ) )
			result = null; 
		else
			result = ( this.getEstimacionTiempoPropioHijo( f ) + 
					this.getEstimacionTiempoHijosHijo( f ) ) /
					this.getTiempoHijos( );
		
		return result;
	}
	
	public boolean equals ( EntradaFuncion f ) {
		return this.getIndice( ).equals( f.getIndice( ) );
	}
}
