package modelo.modeloXML;

public class LineaPerfilPlano {
	/**
	 * Nombre de la función que representa.
	 */
	String nombreFuncion = null;
	/**
	 * Tiempo en segundos, que se gasta exclusivamente en el cuerpo de esta Función. Quedando excluido el 
	 * tiempo gastado en funciones llamadas por esta Función. (en segundos)
	 */
	Double tiempoPropio = null;
	/**
	 * Número total de llamadas recibidas por esta Función. Quedan excluidas las llamadas recursivas a sí misma.
	 * NOTA: Es derivado, está por evitar calcularlo, se redunda para no buscarlo en el grafo.
	 */
	Integer numeroLlamadas = null;
	
	/**
	 * Estimación lineal del tiempo total (propio + hijos) gastado por esta Función. Quedan excluidas llamadas
	 * cíclicas. (en milisegundos)
	 * NOTA: Es derivado, está por evitar calcularlo, se redunda para no buscarlo en el grafo.
	 */
	Double estimacionTiempoTotalPorLlamada = null;

	public LineaPerfilPlano () {
	}
	
	public LineaPerfilPlano ( String nombre, Double tiempo, Integer llamadas ) {
		this.nombreFuncion = nombre;
		this.tiempoPropio = tiempo;
		this.numeroLlamadas = llamadas;
	}

	public String getNombreFuncion() {
		return nombreFuncion;
	}

	public Double getTiempoPropio() {
		return tiempoPropio;
	}

	public Integer getNumeroLlamadas() {
		return numeroLlamadas;
	}

	/**
	 *  Devuelve en milisegundos una estimación lineal del tiempo propio gastado por llamada.
	 */
	public Double getEstimacionTiempoPropioPorLlamada () {
		return tiempoPropio / numeroLlamadas * 1000;
	}
	
	public Double getEstimacionTiempoTotalPorLlamada () {
		return estimacionTiempoTotalPorLlamada;
	}
	
	public boolean isEspontanea () {
		return ( getNumeroLlamadas() == null );
	}
	
	public void setNombreFuncion(String nombreFuncion) {
		this.nombreFuncion = nombreFuncion;
	}

	public void setTiempoPropio(Double tiempoPropio) {
		this.tiempoPropio = tiempoPropio;
	}

	public void setNumeroLlamadas(Integer numeroLlamadas) {
		this.numeroLlamadas = numeroLlamadas;
	}
	
	public void setEstimacionTiempoTotalPorLlamada ( Double tiempo ) {
		this.estimacionTiempoTotalPorLlamada = tiempo;
	}
	
	public String toString () {
		String resultado ="";
		resultado += getNombreFuncion() + ":" + getTiempoPropio() + ":" + ( isEspontanea() ? "espontánea" : ( getNumeroLlamadas()  + ":" + getEstimacionTiempoPropioPorLlamada () + ":" +   getEstimacionTiempoTotalPorLlamada () ) );
		
		return resultado;
	}
}
