package modelo.vcompilacion;

import interfaz.dialogos.error.DialogoError;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;

import modelo.ficheros.TipoFichero;
import modelo.ficheros.UtilidadesExtensionFichero;
import modelo.ficheros.UtilidadesFichero;
import negocio.PreferenciasSingleton;

public class ModeloCompilacion {

	private DefaultListModel	molstFicheros	= new DefaultListModel( );
	
	static ResourceBundle errores = ResourceBundle
		.getBundle( "interfaz.errores.Errores" , 
				PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getLocale( ) );

	private TipoFichero tipo = TipoFichero.DESCONOCIDO; 

	private class ControladorModeloLista implements ListDataListener {
		@Override
		public void intervalAdded ( ListDataEvent arg0 ) {
			OnChange( );
		}

		@Override
		public void intervalRemoved ( ListDataEvent arg0 ) {
			OnChange( );
		}

		@Override
		public void contentsChanged ( ListDataEvent arg0 ) {
			OnChange( );
		}
	}

	{
		molstFicheros.addListDataListener( new ControladorModeloLista( ) );
	}

	private File					directorioTrabajo	= PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getWorkspaceCompilacion( );

	/**admite null*/
	private String					nombrePrograma		= null;

	private String					comando				= new String( );

	private boolean					comandoAuto			= true;

	ArrayList < ChangeListener >	listeners			= new ArrayList < ChangeListener >( 0 ); // listeners

	
	private Vector < String > compilador = PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getComandoCompilar( );
	
	public void addChangeListener ( ChangeListener listener ) {
		listeners.add( listener );
	}

	public void removeChangeListener ( ChangeListener listener ) {
		listeners.remove( listener );
	}

	private void OnChange ( ) {
		ChangeEvent e = new ChangeEvent( this );
		Iterator < ChangeListener > it = listeners.iterator( );
		while ( it.hasNext( ) ) {
			it.next( ).stateChanged( e );
		}
	}

	public DefaultListModel getFicherosListModel ( ) {
		return molstFicheros;
	}

	
	@SuppressWarnings ( "unchecked" )
	public ArrayList < File > getListaFicheros ( ) {
		ArrayList < File > ficheros = (ArrayList < File >) Collections
				.list( molstFicheros.elements( ) );
		return ficheros;
	}

	public File getDirectorioTrabajo ( ) {
		return directorioTrabajo;
	}

	public void setDirectorioTrabajo ( File dir ) {
		if( dir != null && dir.isDirectory( ) )
			this.directorioTrabajo = dir;
		else 
			this.directorioTrabajo = PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getWorkspaceCompilacion( );
		OnChange( );
	}

	public String getNombrePrograma ( ) {
		return nombrePrograma;
	}

	public void setNombrePrograma ( String nombrePrograma ) {
		if ( nombrePrograma != null && nombrePrograma.compareTo( "" ) != 0 )
			this.nombrePrograma = nombrePrograma;
		else 
			this.nombrePrograma = null;
		OnChange( );
	}

	public String [] getComandoCompilacion ( ) {
		String [] resultado;
		if ( this.isComandoAuto() == false ) {
			String [] comillas = this.comando.split("\""); //primero quitar los reservados con comillas (").
			
			LinkedList <String> resultadoLL = new LinkedList <String>();
			
			for ( int i = 0; i < comillas.length ; i++) {
				if ( i%2 == 0 ) {
					resultadoLL.addAll( Arrays.asList( comillas[i].trim( ).split( "\\s" ) ) );
				} else {
					resultadoLL.add( comillas[i].trim( ) );
				}
			}
				
			/*resultado = new String [] { comando };
			for ( int i = 0; i < resultado.length; i++ ) System.out.println(resultado[i]);
			System.out.println("-------");
			
			resultado = comando.split(" (?!(\\\\))"); //primero quitar los reservados
			for ( int i = 0; i < resultado.length; i++ ) System.out.println(resultado[i]);
			System.out.println("-------");
			
			resultado = comando.split(" (?<=(\\\\))"); //primero quitar los reservados
			for ( int i = 0; i < resultado.length; i++ ) System.out.println(resultado[i]);
			System.out.println("-------");
			
			resultado = comando.split(" (?<!(\\\\))"); //primero quitar los reservados
			for ( int i = 0; i < resultado.length; i++ ) System.out.println(resultado[i]);
			System.out.println("-------");*/
			
			String [] resultadoF = new String [ resultadoLL.size( ) ];
			resultado = resultadoLL.toArray(resultadoF );
			
		} else {
			resultado = this.getComandoCompilacionAuto( );
		}
		return resultado;
	}

	public String [] getComandoCompilacionAuto ( ) {

		LinkedList < String > resultado = new LinkedList < String >( );
		// 1º) Preparar orden de Compilación ****************

		// 1.a) orden de compilación (gcc normalmente)
		resultado.addAll( this.getCompilador() );

		// 1.b) colección de ficheros fuente
		ArrayList < File > ficheros = this.getListaFicheros( );
		Iterator < File > i = ficheros.iterator( );
		File normal = null;
		if ( PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getUseCanonicalFileName( ) ) {
			while ( i.hasNext( ) ) {
				normal = i.next( );
				if (normal != null && normal.exists( ) ) {
					try {
						resultado.add( normal.getCanonicalPath( ) );
					} catch ( IOException e ) {
						resultado.add( normal.getAbsolutePath( ) );
					}
				}
			}
		}
		else { //cortos
			while ( i.hasNext( ) ) {
				normal = i.next( );
				if (normal != null && normal.exists( ) ) {
					try {
						resultado.add( new UtilidadesFichero().getRelativePath(getDirectorioTrabajo( ), normal ) );
					} catch ( IOException e ) {
						DialogoError.getInstance( ).showErrorDialog( 
								errores.getString( "err_relativepath" ) );
					}
				}	
			}			
		}
		
		
		// 1.c) nombre del ejecutable.
		if ( nombrePrograma != null ) {
			resultado.add( "-o" );
			resultado.add( nombrePrograma );
		} else {
			/** si hubiera que poner un nombre por defecto, sería aquí*/
		}

		String [] t = new String [resultado.size( )];
		return resultado.toArray( t );

	}

	/**
	 * Devuelve el comando-compilador que se usará para este archivo
	 * (guardado previamente con setCompilador).
	 * @return
	 */
	public Vector <String> getCompilador ( ) {
		return compilador;
	}
	
	public void setCompilador ( Vector <String> vc) {
		if ( vc != null && vc.size( ) > 0 ) {
			compilador = vc;
		
			OnChange();
		}
	}
	
	public String getCadenaCompilacionDefinidaUsuario() {
		return ( comando != null )? comando : "";
	}
	
	public void setCadenaCompilacionDefinidaUsuario( String s ) {
		comando = s;
		
		OnChange( );
	}

	public void setComandoCompilacion ( String s ) {
		this.setComandoAuto ( false );
		this.comando = s;
		this.OnChange( );
	}

	public void setComandoAuto ( boolean b ) {
		this.comandoAuto = b;
		this.OnChange( );
	}

	public boolean isComandoAuto () {
		return this.comandoAuto;
	}
	
	public void addFichero( File f ){
		this.getFicherosListModel( ).addElement( f );
		if( getTipo( ) == TipoFichero.DESCONOCIDO ) {
			setTipo( UtilidadesExtensionFichero.getTipo(f) );
		}
		this.OnChange( );
	}
	
	public void removeFichero( Object seleccionados) {
		this.getFicherosListModel( ).removeElement( seleccionados );
		if( this.getFicherosListModel( ).isEmpty( ) ) {
			setTipo( TipoFichero.DESCONOCIDO );
		}
		this.OnChange( );
	}
	
	public TipoFichero getTipo () {
		return tipo;
	}
	
	public void setTipo( TipoFichero t ) {
		tipo = t;
		compilador = PreferenciasSingleton.getPreferenciasSingleton( ).getPreferencias( ).getComandoCompilar( tipo );
		
		OnChange();
	}

	public boolean isCompiled ( ) {
		File f = null;
		if( this.getDirectorioTrabajo( ) != null && this.getNombrePrograma( ) != null )
			f = new File( this.getDirectorioTrabajo( ), this.getNombrePrograma( ) );
		return f != null && f.exists( );
	}
}
