/**
 * 
 */
package modelo.vestadisticas.grafollamadas;

import modelo.modeloXML.EntradaFuncion;
import modelo.modeloXML.InterfazModeloXML;
import modelo.modeloXML.ModeloXML;
import modelo.vestadisticas.ModeloEstadistica;
import interfaz.grafico.GraficoModel;
import interfaz.grafico.GraficoModelListener;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;

import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;


/**
 * @author angel
 *
 */
public class ModeloGraficoFLlamadoras implements GraficoModel, InterfazModeloXML, ListSelectionListener, ActionListener, TreeSelectionListener {

	ModeloXML moXML;
	
	private Double tiempoTotal = null;
	private EntradaFuncion funcion = null;
	private Vector < EntradaFuncion > funcionesLlamadoras = new Vector < EntradaFuncion > (0);
	//private Vector < EntradaFuncion > funcionesLlamadasMiembro = new Vector < EntradaFuncion > (0);
	//Las funciones miembro no entran en los cálculos.
	


	private LinkedList <GraficoModelListener> listeners = new LinkedList<GraficoModelListener>();
	
	protected void fireDataChanged() {
		ListDataEvent evento = new ListDataEvent(this, 0, 0, 0);//consultar ayuda
		
		int i;

		for (i=0; i<listeners.size(); i++)
		    ((GraficoModelListener)listeners.get(i)).graphicChanged(evento);

	}
	
	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#removeGraficoModelListener(interfaz.grafico.GraficoModelListener)
	 */
	public void removeGraficoModelListener(GraficoModelListener l) {
		listeners.remove(l);
	}
	
	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#addGraficoModelListener(interfaz.grafico.GraficoModelListener)
	 */
	public void addGraficoModelListener(GraficoModelListener l) {
		listeners.add(l);
	}

	public ModeloGraficoFLlamadoras () {
		
	}
	
	public ModeloGraficoFLlamadoras ( ModeloEstadistica x ) {
		setDatosXML( x );
	}
	
	public void setDatosXML ( ModeloEstadistica me ) {
		ModeloXML x = me.getModeloXML( );
		if ( x != null && x.hasGrafoLlamadas() ) {
			moXML = x;
			fireDataChanged();
		}
	}
	

	/* (non-Javadoc)
	 * @see javax.swing.event.ListSelectionListener#valueChanged(javax.swing.event.ListSelectionEvent)
	 */
	public void valueChanged( ListSelectionEvent e ) {
		EntradaFuncion funcion = null;

		if(e.getSource() instanceof JList ) {
			funcion = (EntradaFuncion) ((JList)e.getSource()).getSelectedValue();
			setFuncionActivo( funcion.getIndice() );
		}
	}
	/**
	 * @param string
	 */
	private void setFuncionActivo( Integer indiceFuncion ) {
		if( moXML != null && moXML.hasGrafoLlamadas() && moXML.getEntradasFuncion().containsKey(indiceFuncion) ) {
			this.funcion = moXML.getEntradasFuncion().get(indiceFuncion);
			this.funcionesLlamadoras = new Vector < EntradaFuncion > ( funcion.getPadres() );
			//this.funcionesLlamadasMiembro = new Vector < EntradaFuncion > ( funcion.getHijosMiembroCiclo());
			this.tiempoTotal = null;
			this.fireDataChanged();
		}
	}
	
	/*public HashMap valores() {
		return datosEntradas;
	}*/

	/* (non-Javadoc)
	 * @see javax.swing.event.TreeSelectionListener#valueChanged(javax.swing.event.TreeSelectionEvent)
	 */
	public void valueChanged(TreeSelectionEvent e) {

		EntradaFuncion funcion = null;
		//String nombreFuncion = new String();
		TreePath tp = e.getPath();

		DefaultMutableTreeNode n = null;
		if(tp.getLastPathComponent() instanceof DefaultMutableTreeNode) {
			n = (DefaultMutableTreeNode) tp.getLastPathComponent();
			if( n.getUserObject() instanceof EntradaFuncion ) {
				funcion = (EntradaFuncion) n.getUserObject();
				
				setFuncionActivo( funcion.getIndice() );
			}
		}
	}
	
	
	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#getPorcionName(int)
	 */
	public String getPorcionName( int porcionIndex ) {
		String resultado;
		if ( porcionIndex >= 0 && porcionIndex < funcionesLlamadoras.size() )
			resultado = funcionesLlamadoras.get(porcionIndex).getNombreFuncion();
		else
			resultado = null;
		return resultado;
	}

	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#getSize()
	 */
	public int getSize() {
		
		return funcionesLlamadoras.size();
	}

	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#getValueAt(int)
	 */
	public Double getValueAt(int porcionIndex) {
		Double resultado;
		if(porcionIndex >= 0 && porcionIndex < funcionesLlamadoras.size() )
			resultado = funcion.getEstimacionTiempoPropioPadre( funcionesLlamadoras.get( porcionIndex ) );
		else
			resultado = null;
		return resultado;
	}

	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#isPorcionEditable(int)
	 */
	public boolean isPorcionEditable(int porcionIndex) {
		return false;
	}

	
	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#getSum()
	 */
	public Double getSum() {
		if ( tiempoTotal == null ) {
			Double resultado = 0.0;
			Iterator < EntradaFuncion > i = funcionesLlamadoras.iterator();
			while ( i.hasNext() ) {
				resultado+= funcion.getEstimacionTiempoPropioPadre( i.next() );
			}
			tiempoTotal = resultado;
		}
		
		return tiempoTotal;
	}

	@Override
	public void addValue(Double porcionValue, String porcionName) {
		//No hace falta porque no es editable		
	}


	@Override
	public void setValueAt(Double porcionValue, String porcionName,
			int porcionIndex) {
		//No hace falta porque no es editable
	}
	
	public Color getPorcionColor( int i ){
		Color color=Color.WHITE;
		switch (i % 8){
			case 0: color = Color.BLUE; break;
			case 1: color = Color.GREEN; break;
			case 2: color = Color.MAGENTA; break;
			case 3: color = Color.ORANGE; break;
			case 4: color = Color.CYAN; break;
			case 5: color = Color.PINK; break;
			case 6: color = Color.YELLOW; break;
			case 7: color = Color.RED; break;
			default: break;
		}
		return color;
	}
	
	@Override
	public void actionPerformed ( ActionEvent e ) {
		EntradaFuncion fichero = null;

		if ( e.getSource( ) instanceof JComboBox ) {
			JComboBox x = (JComboBox ) e.getSource( );
			if( x.getSelectedItem( ) instanceof EntradaFuncion ) {
				fichero = (EntradaFuncion) ( x.getSelectedItem( ) );
				if ( fichero != null ) {
					setFuncionActivo( fichero.getIndice( ) );
				}
			}
		}
	}
}