/**
 * 
 */
package modelo.vestadisticas.perfilplano;

import interfaz.grafico.JGrafico;
import interfaz.grafico.GraficoModel;
import interfaz.grafico.GraficoModelListener;

import java.awt.Color;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;

import javax.swing.event.ListDataEvent;

import datos.GprofXMLParser;

import modelo.modeloXML.InterfazModeloXML;
import modelo.modeloXML.LineaPerfilPlano;
import modelo.modeloXML.ModeloXML;
import modelo.vestadisticas.ModeloEstadistica;


/**
 * @author angel
 *
 */
public class ModeloGraficoPerfilPlano implements GraficoModel, InterfazModeloXML {

	ModeloXML moXML;
	
	private Double tiempoTotal = null;
	private Vector < LineaPerfilPlano > perfilPlano = new Vector < LineaPerfilPlano > (0);
	//private Vector < LineaPerfilPlano > funcionesLlamadasMiembro = new Vector < LineaPerfilPlano > (0);
	//Las funciones miembro no entran en los cálculos.
	


	private LinkedList <GraficoModelListener> listeners = new LinkedList<GraficoModelListener>();
	
	protected void fireDataChanged() {
		ListDataEvent evento = new ListDataEvent( this, 0, 0, 0 );//consultar ayuda
		
		int i;

		for ( i = 0; i < listeners.size(); i++)
		    ( (GraficoModelListener) listeners.get( i ) ).graphicChanged( evento );

	}
	
	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#removeGraficoModelListener(interfaz.grafico.GraficoModelListener)
	 */
	public void removeGraficoModelListener(GraficoModelListener l) {
		listeners.remove( l );
	}
	
	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#addGraficoModelListener(interfaz.grafico.GraficoModelListener)
	 */
	public void addGraficoModelListener(GraficoModelListener l) {
		listeners.add( l );
	}
////////////////////////////////////////////
	
	public ModeloGraficoPerfilPlano () {

	}
	
	public ModeloGraficoPerfilPlano ( ModeloEstadistica x ) {
		setDatosXML( x );
	}
	
	public void setDatosXML ( ModeloEstadistica me ) {
		ModeloXML x = me.getModeloXML( );
		if ( x != null && x.hasGrafoLlamadas() ) {
			moXML = x;
			perfilPlano = moXML.getPerfilPlano();
			fireDataChanged();
		}
	}
	

	
	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#getPorcionName(int)
	 */
	public String getPorcionName( int porcionIndex ) {
		String resultado;
		if ( porcionIndex >= 0 && porcionIndex < perfilPlano.size() )
			resultado = perfilPlano.get(porcionIndex).getNombreFuncion();
		else
			resultado = null;
		return resultado;
	}

	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#getSize()
	 */
	public int getSize() {
		return perfilPlano.size();
	}

	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#getValueAt(int)
	 */
	public Double getValueAt( int porcionIndex ) {
		Double resultado = new Double(0);
		if( porcionIndex >= 0 && porcionIndex < perfilPlano.size() )
			resultado = perfilPlano.get( porcionIndex ).getTiempoPropio( );
		
		return resultado;
	}

	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#isPorcionEditable(int)
	 */
	public boolean isPorcionEditable ( int porcionIndex ) {
		return false;
	}

	
	/* (non-Javadoc)
	 * @see interfaz.grafico.GraficoModel#getSum()
	 */
	public Double getSum() {
		if ( tiempoTotal == null ) {
			Double resultado = 0.0;
			Iterator < LineaPerfilPlano > i = perfilPlano.iterator();
			while ( i.hasNext() ) {
				resultado += i.next().getTiempoPropio( );
			}
			tiempoTotal = resultado;
		}
		
		return tiempoTotal;
	}

	@Override
	public void addValue( Double porcionValue, String porcionName ) {
		//No hace falta porque no es editable		
	}


	@Override
	public void setValueAt(Double porcionValue, String porcionName,
			int porcionIndex) {
		//No hace falta porque no es editable
	}
	
	
	public Color getPorcionColor( int i ){
		Color color=Color.WHITE;
		switch (i % 8){
			case 0: color = Color.BLUE; break;
			case 1: color = Color.GREEN; break;
			case 2: color = Color.MAGENTA; break;
			case 3: color = Color.ORANGE; break;
			case 4: color = Color.CYAN; break;
			case 5: color = Color.PINK; break;
			case 6: color = Color.YELLOW; break;
			case 7: color = Color.RED; break;
			default: break;
		}
		return color;
	}
	
	
	public static void main (String[] args) {
		ModeloGraficoPerfilPlano moG = new ModeloGraficoPerfilPlano();
		JGrafico g = new JGrafico(moG);
		javax.swing.JPanel p = new javax.swing.JPanel ();
		p.add(g);
		javax.swing.JFrame v = new javax.swing.JFrame ();
		v.setContentPane(p);
		
		javax.swing.JFileChooser f = new javax.swing.JFileChooser();
		
		if ( f.showOpenDialog( new javax.swing.JFrame() ) == javax.swing.JFileChooser.APPROVE_OPTION ) {
			ModeloXML mo;
			try {
				mo = new GprofXMLParser( ).leerModeloXML( f.getSelectedFile() );
						
				ModeloEstadistica me = new ModeloEstadistica();
				me.setFicheroXML( f.getSelectedFile() );
				me.setModeloXML( mo );
				moG.setDatosXML(me);
			} catch ( Exception e ) {
				javax.swing.JOptionPane.showMessageDialog( v , 
						"fallo lectura xml",
						"error" , javax.swing.JOptionPane.ERROR_MESSAGE );
				e.printStackTrace( );
			}
		}
		
        v.setDefaultCloseOperation(javax.swing.JFrame.EXIT_ON_CLOSE);
        v.pack();
        v.setVisible(true);
	}
}
