package negocio;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import datos.PreferenciasXMLParser;
import modelo.Preferencias;

public class PreferenciasSingleton {
	/** Esta propiedad sirve para implementar el singleton. */
	private static PreferenciasSingleton INSTANCE = null;
	
	private Preferencias preferencias = new Preferencias( );
	
	private boolean preferenciasModificadas = false;
	
	 /** Un singleton no debe clonarse*/	
    public Object clone() throws CloneNotSupportedException {
        throw new CloneNotSupportedException(); 
	}
    
    /** PreferenciasSingleton maneja las preferencias y es un Singleton. */
	public static PreferenciasSingleton getPreferenciasSingleton ( ) {
		if (INSTANCE == null) INSTANCE = new PreferenciasSingleton( );
        return INSTANCE;
	}
	
	/** PreferenciasSingleton maneja las preferencias y es un Singleton. */
	private PreferenciasSingleton ( ) {
		
	}
	
	/** Obtener las preferencias actuales */
	public Preferencias getPreferencias() {
		return preferencias;
	}
	
	/** Modificar preferencias */
	public void setPreferencias( Preferencias p ) {
		preferenciasModificadas = true;
		preferencias = p;
	}
	
	/** Devuelve true si las preferencias han sido Cambiadas a través de setPreferencias. */
	public boolean isBeenModified ( ) {
		return this.preferenciasModificadas;
	}
	
	/** Cargar preferencias iniciales de un fichero f. 
	 * @throws IOException 
	 * @throws SAXException 
	 * @throws ParserConfigurationException 
	 * @throws XPathExpressionException */
	public void cargarPreferencias ( File f )
		throws XPathExpressionException, ParserConfigurationException, 
			SAXException, IOException {
		this.preferenciasModificadas = false;
		if ( f.exists( ) )
			preferencias = new PreferenciasXMLParser( ).leerDeFichero( f );
	}

	/** guardar preferencias en el fichero f .
	 * @throws TransformerException 
	 * @throws ParserConfigurationException */
	public void guardarPreferencias ( File f ) throws ParserConfigurationException, TransformerException {
		new PreferenciasXMLParser( ).guardarEnFichero( f, preferencias  );
	}
	
}
