package negocio.ejecutarOrdenes;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;

import javax.swing.SwingUtilities;

import negocio.ejecutarOrdenes.ILineaComando;
import negocio.ejecutarOrdenes.LineaComando;

public class EjecutorComandos extends Thread {

	/**
	 * recursos para dar soporte de idiomas sobre errores
	 */
	ResourceBundle	errores	= ResourceBundle
									.getBundle( "interfaz.errores.Errores" );

	ILineaComando lC = null;
	
	InterfazOrdenes interfaz = null;
	
	private void CONSTRUIR ( String [] ordenes , String [] input ,
			File directorioTrabajo , final InterfazOrdenes interfaz )
	throws NullPointerException {
		
		if ( ordenes == null )
			throw new NullPointerException( );

		if ( directorioTrabajo == null )
			directorioTrabajo = new File( System.getProperty( "user.home" ) );

		/** 2º) Realizar ejecución en segundo plano ****************/
		lC = new LineaComando( ordenes , directorioTrabajo );

		/** Añadir input si hay */
		if ( input != null ) {
			for ( int i = 0 ; i < input.length ; i++ ) {
				lC.addInput( input[i] );
			}
		}
		
		if ( interfaz == null )
			throw new NullPointerException( );
		else
			this.interfaz = interfaz;
	}
	
	/**
	 * Función que se encarga de ejecutar una orden ("ordenes") como si fuera
	 * hecha en línea de comandos. Presupone directorioTrabajo como el
	 * directorio de trabajo. Si es null, usa el directorio de casa. interfaz,
	 * es una clase que implementa InterfazOrdenes, y que deberá manejar la
	 * salida de la órden. Devuelve una instancia del hilo que se encarga de
	 * ejecutar la orden. Por si se quiere llevar cierto control del mismo.
	 * 
	 * @param ordenes
	 * @param input
	 * @param directorioTrabajo
	 * @param interfaz
	 * @return Thread
	 */
	public EjecutorComandos ( String [] ordenes , String [] input ,
			File directorioTrabajo , final InterfazOrdenes interfaz )
			throws NullPointerException {

		CONSTRUIR ( ordenes, input, directorioTrabajo, interfaz );

	}

	public EjecutorComandos ( String [] ordenes , File directorioTrabajo ,
			final InterfazOrdenes interfaz ) throws NullPointerException {
		CONSTRUIR ( ordenes , null , directorioTrabajo , interfaz );
	}

	/**
	 * Función que se encarga de ejecutar una orden ("ordenes") como si fuera
	 * hecha en línea de comandos. Presupone directorioTrabajo como el
	 * directorio de trabajo. Si es null, usa el directorio de casa. interfaz,
	 * es una clase que implementa InterfazOrdenes, y que deberá manejar la
	 * salida de la órden. Devuelve una instancia del hilo que se encarga de
	 * ejecutar la orden. Por si se quiere llevar cierto control del mismo.
	 * 
	 * @param ordenes
	 * @param directorioTrabajo
	 * @param interfaz
	 * @return Thread
	 */
	public EjecutorComandos ( File programa , String [] argumentos ,
			String [] input , File dirTrabajo , final InterfazOrdenes ie ) {

		ArrayList < String > ordenEjecutar = new ArrayList < String >( );

		try {
			ordenEjecutar.add( programa.getCanonicalPath( ) );
		} catch ( IOException e ) {
			ordenEjecutar.add( programa.getAbsolutePath( ) );
		}

		ordenEjecutar.addAll( Arrays.asList( argumentos ) );

		CONSTRUIR ( ordenEjecutar.toArray( new String [ordenEjecutar
				.size( )] ) , input , dirTrabajo , ie );
	}


	/** Hilo para ejecución en segundo plano. */
	public void run ( ) {
		try {
			/** Ejecutar la orden. */
			int resultadoCompilación = -1;

			/**
			 * Ejecutar método para que se muestre en la interfaz de usuario
			 * antes de la ejecución. Como conlleva interactuar con el GUI
			 * debe hacerse desde el EDT.
			 */
			SwingUtilities.invokeAndWait( new Runnable( ) {
				public void run ( ) {
					interfaz.empezarOrden( lC.getCommand( ) );
				}
			} );

			/** Comienza ejecución de la orden. */
			/*try {*/
			lC.start( );
			/*} catch ( IOException e ) {
				DialogoError.getInstance( ).showErrorDialog( errores.getString( "err_ejecucion_comando" ) + 
						"\n" + lC.getCommand( ) +
						"\n" + e.getMessage( ) );
				e.printStackTrace();
			}*/

			resultadoCompilación = lC.waitFor( );

			/** Devolver resultado de la ejecución. */
			final String output = lC.getOutput( );
			final String error = lC.getError( );
			final int resultadoCompilaciónFinal = resultadoCompilación;

			/**
			 * Ejecutar método para que se muestre en la interfaz de usuario
			 * después de la ejecución. Como conlleva interactuar con el GUI
			 * debe hacerse desde el EDT.
			 */
			SwingUtilities.invokeAndWait( new Runnable( ) {
				public void run ( ) {
					interfaz.terminarOrden( resultadoCompilaciónFinal ,
							output , error );
				}
			} );
		} catch ( Exception e ) {
			interfaz.ordenErrorIndeterminado( lC.getCommand( ) , lC.getDirectory( ), e );
		}

	}
}
