package negocio.ejecutarOrdenes;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.lang.ProcessBuilder;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;



/**
 * @author angel
 *
 */
public class LineaComando extends Object implements ILineaComando {
	
	static final long serialVersionUID = 20081028;
	
	private boolean start = false;//
	//no se puede modificar cmd, env, in ni dir, cuando ya ha empezado
	//la ejecución del proceso.
	
	//Al igual que no se puede ejecutar con seguridad getOutput ni getError hasta que 
	//no haya empezado.
	
	
	private ProcessBuilder pB = new ProcessBuilder();
	private Process p;
	
	private Vector <String> in = new Vector <String> ();
	private String out;
	private String err;
	
	private HiloEntrada hin;
	private HiloSalida hout;
	private HiloError herr;
	
	public LineaComando (String [] cmd, File dir) {
		pB.command(cmd);
		pB.directory(dir);
	}
	
	public LineaComando (String [] cmd, String [] env, File dir) {
		pB.command(cmd);
		pB.directory(dir);
		this.setEnviroment(env);
	}
	

	public LineaComando () {	}
	
	
	/* (non-Javadoc)
	 * @see sistema.lineaComando#addInput(java.lang.String)
	 */
	public void addInput (String s) {
		if ( start == false ) {
			in.add(s);
		}
	}
	

	/* (non-Javadoc)
	 * @see sistema.lineaComando#getError()
	 */
	public String getError() {
		err = herr.getError();
		return err;
	}
	

	/* (non-Javadoc)
	 * @see sistema.lineaComando#getOutput()
	 */
	public String getOutput() {
		out = hout.getOutput();
		return out;
	}
	

	/**
	 * @return
	 */
	public String getCommand () {
		String cmd = new String();
		Iterator <String> i = pB.command().iterator();
		while (i.hasNext()) {
			cmd += (String) i.next() + " ";
		}
		return cmd;
	}
	
	
	/* (non-Javadoc)
	 * @see sistema.lineaComando#setCommand(java.lang.String)
	 */
	public void addArgument (String s) {
		if (start == false) {
			java.util.LinkedList <String> ls = new java.util.LinkedList <String> (pB.command());
			ls.add(s);
			pB.command(ls);
		}
	}
	

	/* (non-Javadoc)
	 * @see sistema.lineaComando#setCommand(java.lang.String[])
	 */
	public void setCommand (String[] cmd) {
		if (start == false) {
			pB.command().clear();
			pB.command( Arrays.asList(cmd) );
		}
	}
	

	/* (non-Javadoc)
	 * @see sistema.lineaComando#setDirectoy(java.io.File)
	 */
	public void setDirectory (File dir) {
		pB.directory(dir);
	}
	
	public File getDirectory () {
		File f = new File(System.getProperty("user.dir"));
		if (pB.directory() != null) {
			f = pB.directory();
		}
		return f;
	}
	
	/* (non-Javadoc)
	 * @see sistema.lineaComando#setEnviroment(java.lang.String[])
	 */
	public void setEnviroment (String[] env) {
		pB.environment().clear();
		for ( String envLine : env ) {
			int indexOfIgual = envLine.indexOf("=");
			if( indexOfIgual >= 0 ) {
				String value = envLine.substring(0, indexOfIgual);
				String name = envLine.substring(indexOfIgual);
				pB.environment().put(name, value);
			}
		}	
	}
	
	public void setEnviroment ( Map <String,String> env ) {
		pB.environment().clear();
		pB.environment().putAll(env);
	}
	
	public void modifyEnviroment ( String [] env ) {
		for ( String envLine : env ) {
			int indexOfIgual = envLine.indexOf("=");
			if( indexOfIgual >= 0 ) {
				String name = envLine.substring(0, indexOfIgual);
				String value = envLine.substring(indexOfIgual+1);
				pB.environment().put(name, value);
			}
		}
	}
	
	public void modifyEnviroment ( Map <String,String> env ) {
		pB.environment().putAll(env);
	}
	
	public Map <String, String> getEnviroment () {
		return pB.environment();
	}
	
	public void abort ()  throws InterruptedException {
		if (start) {
			p.destroy();
			hout.join();
			herr.join();
			hin.join();
		}
	}

	/* (non-Javadoc)
	 * @see sistema.lineaComando#start()
	 */
	public void start () throws IOException {
		
		start = true;

		/** Pasar todos las entradas por la entrada estándar*/
		String [] arrayIN = new String [in.size()];
		in.toArray(arrayIN);
		
		p = pB.start(); /** p es el proceso.*/

		/** Iniciamos los hilos que se encargan de introducir
		 * la entrada y recoger la salida estándar y de errores. */
		hin = new HiloEntrada(p,arrayIN);
		herr = new HiloError(p);
		hout = new HiloSalida(p);
		
		hin.start();
		herr.start();
		hout.start();
	}
	


	/* (non-Javadoc)
	 * @see sistema.lineaComando#waitFor()
	 */
	public int waitFor() throws InterruptedException {
		int wF = p.waitFor();
		hin.join();
		herr.join();
		hout.join();
		if ( (hout.getStatus() == HiloStatus.FALLO) 
				|| (herr.getStatus() == HiloStatus.FALLO) 
				|| (hout.getStatus() == HiloStatus.FALLO) ) {
			//throw new StatusFailException();
		}
		start = false;
		return wF;
	}
}

class HiloStatus extends Thread {
	final static int FALLO = 0;
	final static int EXITO = 1;
	
	int status = FALLO;
	
	public int getStatus() {
		return status;
	}
	
	protected void setExito() {
		status = EXITO;
	}
	
	protected void setFallo() {
		status = FALLO;
	}

}

class HiloSalida extends HiloStatus {

	private Process m_process;

	private String out = new String();
	
	public HiloSalida ( Process process ) throws NullPointerException {
		if ( process == null ) throw new NullPointerException();
		m_process = process;
	}
	
	public String getOutput() {
		//wait...
		return out;
	}
	//public String getErrorActual() { return err; flush?? }

	public void run() {
        try {
        	BufferedReader stdout = new BufferedReader(new InputStreamReader(m_process.getInputStream()));
            String line;
            for(int l = 0; (line = stdout.readLine()) != null; ) {
                if (line.length() > 0) l++;
                out += line + "\n";
            }
//            System.out.println("\nRead " + l + " lines from stderr.");
            stdout.close();
        }
        catch(IOException ie) {
            System.err.println("IO exception on stderr: " + ie);
        }
        setExito();
	}
}

class HiloError extends HiloStatus {

	private Process m_process;

	private String err = new String();
	
	public HiloError( Process process)  throws NullPointerException {
		if ( process == null ) throw new NullPointerException();
		m_process = process;
	}
	
	public String getError() {
		//wait...
		return err;
	}
	//public String getErrorActual() { return err; flush?? }

	public void run() {
        try {
        	BufferedReader stderr = new BufferedReader(new InputStreamReader(m_process.getErrorStream()));
            String line;
            for(int l = 0; (line = stderr.readLine()) != null; ) {
                if (line.length() > 0) l++;
                err += line + "\n";
            }
            stderr.close();
        }
        catch(IOException ie) {
            System.err.println("IO exception on stderr: " + ie);
        }
        setExito();
	}
}

class HiloEntrada extends HiloStatus {

	private Process m_process;

	private Vector <String> in = new Vector <String> ();
	
	public HiloEntrada (Process process) {
		m_process = process;
	}
	
	public HiloEntrada (Process process, String in) {
		m_process = process;
		this.in.addElement(in);
	}
	
	public HiloEntrada (Process process, String [] in) {
		m_process = process;
		this.in.addAll(Arrays.asList(in));
	}
	
	public void addInput (String in) {
		this.in.addElement(in);
	}
	
	public void run() {
		try {
			BufferedOutputStream stdin = new BufferedOutputStream(m_process.getOutputStream());
			PrintWriter commandInput = new PrintWriter((new OutputStreamWriter(stdin)), true);
			for(String inLine : in)
				commandInput.println(inLine);
			commandInput.close();
		} catch( Exception e ) {
			System.err.println(e);
		}
		setExito();
	}
}